/**
 * Script of the table of contents block.
 *
 * @since 0.27.0
 * @package WordPress
 * @subpackage ANRGHG Publishing Helper
 *
 * @courtesy A White Pixel and Novo-Media.
 * @link https://novo-media.ch/en/programming-coding/gutenberg-plugin-development-wp-data-wordpress-rest-api/
 * @link https://awhitepixel.com/blog/wordpress-gutenberg-create-custom-block-part-2-register-block/
 * @link https://awhitepixel.com/blog/wordpress-gutenberg-create-custom-block-tutorial/
 */


import { registerBlockType } from '@wordpress/blocks';
import { __ } from '@wordpress/i18n';
import { TextControl } from '@wordpress/components';


const pipeInfo = __( 'Used as a delimiter, the vertical bar “|” is escaped.', 'anrghg' );
var   escaPipeAlertToDo;
if ( anrghgContents.showEscaPipeAlert ) {
	escaPipeAlertToDo = true;
} else {
	escaPipeAlertToDo = false;
}

function escaPipe( str ) {
	if ( str.indexOf( '|' ) >= 0 ) {
		str = str.replace( /\|/g, '&#x7C;' );
		if ( escaPipeAlertToDo ) {
			alert( pipeInfo );
			escaPipeAlertToDo = false;
		}
	}
	return str;
}


registerBlockType( 'anrghg/table-of-contents', {
	title: __( 'Table of contents', 'anrghg' ),
	description: __( 'Determine the position of the table of contents; else it will be as configured in the ANRGHG Publishing Helper.', 'anrghg' ),
	icon: 'editor-justify',
	category: 'design',
	keywords: [
		__( 'headings', 'anrghg' ),
		__( 'toc', 'anrghg' ),
		__( 'list', 'anrghg' ),
		__( 'direct links', 'anrghg' ),
		__( 'quick access', 'anrghg' ),
		__( 'fragment identifiers', 'anrghg' ),
	],
	supports: {
		multiple: false,
	},

	attributes: {

		contentsLabel: {
			type: 'string',
			default: '',
		}

	},

	edit: ( props ) => {

		const { attributes, setAttributes } = props;
		const purpose = __( 'Here goes the table of contents', 'anrghg' );

		return (

			<div
				style={ {
					border: '1px solid #000',
					paddingTop: '20px',
					fontSize: 'smaller',
				} }
				>

				<p
					style={ {
						fontStyle: 'italic',
						textAlign: 'center',
					} }
					>{ purpose }</p>

			</div>

		);
	},

	save: ( props ) => {

		const { attributes } = props;
		let positioner = '[anrghg_toc';

		positioner += ']';

		return ( positioner );

	},

} );
