<?php
/**
 * Plugin Name: ANRGHG Publishing Helper
 * Plugin Initialism: 'Act Now, Reduce GreenHouse Gases'
 * Plugin Slug: anrghg
 * Plugin URI: https://anrghg.website/publishing-helper
 * Plugin Website Hosting: GreenGeeks Web Hosting
 * Description: Features to help publish content with WordPress in a way reducing GHG. AMP compatible. Add Table of Contents and fragment IDs to paragraphs/headings, reuse complements across your website, add last modified date, list notes and references separately.
 * Requires at least: 5.5
 * Tested up to: 5.8
 * Requires PHP: 5.6
 * Tested PHP up to: 8.0
 * CAUTION: The following field is parsed in Stable Tag for upgrade configuration:
 * Version: 0.30.0
 * Author: ANRGHG
 * Author WordPress.org slug: @anrghg
 * Author email: anrghg@anrghg.website
 * License: GPLv2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: anrghg
 * Domain Path: /translations
 * @package WordPress
 * @subpackage ANRGHG Publishing Helper
 *
 * @todo Options page settings for list labels with plural/dual/singular.
 * @todo Option to configure the section block UI elements.
 * @todo Settings to add custom CSS classes to labels and other elements.
 * @todo Complete settings page.
 * @todo General: Options for metabox position and priority.
 * @todo Complements: Option to display tooltips on click.
 * @todo Complete language configuration file.
 * @todo Security: Options to configure text and styles of the high-profile message.
 * @todo Appended Thank-you: Add styling options.
 * @todo Appended Thank-you: Add posts greenlist/redlist.
 * @todo Appended Thank-you: Positioner code.
 * @todo Date meta: Option to activate only last modified or published date meta tag.
 * @todo Supply `shortcode_unautop()` functionality: Support all positioners as configured.
 * @todo Derive shareable fragment identifiers: Add more conversions.
 * @todo Support heading numbers before (.anrghg_heading_number_before).
 * @todo Table of contents: Expect release of AMP-HTML #35795 `toggleChecked()`.
 * @todo Table of contents: Add buttons in block toolbar for collapsed state and alignment.
 * @todo Complements: Support custom post types.
 * @todo Complements: Optionally use h2 as a section divider.
 * @todo Complements: Change default `[/excerpt]` to `[/tooltip]` once configuration option is implemented.
 * @todo Complements: Support combining identical complements w/ identical # and w/o backlinks.
 * @todo Complements: Bibliography block, with an alternative based on list delimiter codes.
 * @todo Complements: Additional global heading as in Wikipedia.
 * @todo Tooltips: Full-width scheme with display below and triangle pointing to referrer.
 * @todo Conversion script manual footnotes ➔ plugin-compatible footnotes.
 * @todo Conversion tool Word ➔ WordPress with plugin-compatible footnotes.
 * @todo Color picker: Add setting, deactivate by default.
 * @todo Integrate color picker for security to avoid recommending an unmaintained plugin.
 */
defined( 'ABSPATH' ) or exit( nl2br( "\r\n\r\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;This PHP file cannot be displayed in the browser." ) );

/**
 * Helps materialize the plugin version.
 *
 * For a state-of-the-art versioning scheme covering all use cases,
 * a plugin needs three version number fields, two of which already
 * provided, one added. The reason for adding a third field is that
 * the provided fields are parsed for release/update configuration,
 * and can therefore not be used to increment a 4th version number,
 * as usually required to account for changes made to published (or
 * otherwise shared) files. Given that the readme in the Stable Tag
 * is the source code of the plugin’s pages in the Plugin Directory
 * and must be updatable anytime, we are advised that it is okay to
 * update metadata and display data without releasing a new version
 * throughout, beyond testing compatibility with a new WP Core. The
 * added 'Package Version' is now in the readme header only as this
 * is the most update-prone file, and the string is all for humans.
 * @link https://meta.trac.wordpress.org/ticket/5652
 *
 * @since 0.9.0
 * 0.9.0 is this plugin’s initial release.
 *
 * @var string  Plugin version constant.
 */
const C_S_ANRGHG_VER = '0.30.0';

/**
 * Gets the upgrade notice displayed in the plugin page below the entry.
 *
 * WARNING: UPGRADE NOTICES ABOUT JUST BUGFIX RELEASES WILL BE IGNORED!
 * NO UPGRADE NOTICE IS DISPLAYED, IF THE UPGRADE IS NOT AT LEAST FROM
 * THE PRECEDING MINOR VERSION. INCREMENTING TO NEXT MINOR IS REQUIRED.
 *
 * @since 0.27.2
 * @courtesy Andi Dittrich
 * @link https://andidittrich.com/2015/05/howto-upgrade-notice-for-wordpress-plugins.html
 * Formerly, the upgrade notice was displayed by default in this place.
 * This workaround reestablishes the legacy behavior, because else, the
 * upgrade notice is hidden in the UI and goes unnoticed in practice.
 * @link https://developer.wordpress.org/reference/hooks/in_plugin_update_message-file/
 *
 * @since 0.27.4
 * Fix trailing empty paragraph triggering an additional icon instance below,
 * due to the Upgrade Notice’s p tags interfering with the default p tags.
 * Easy conformant fix: Delete closing p tag before closing div tag at l.589.
 * @see wp-admin/includes/update.php:479..589
 *
 * @since 0.30.0
 * Warning about the minor version upgrade requirement.
 * Add the 'Upgrade Notice' label as a fixed beginning.
 *
 * @param object $p_o_cur  Current plugin metadata.
 * @param object $p_o_new  New plugin metadata.
 * @return void  For a streamlined code, actions are added calling `add_filter()`.
 * @see * Core functionality callback functions.
 */
add_filter( 'in_plugin_update_message-' . plugin_basename( __FILE__ ), function( $p_o_cur, $p_o_new ) {
	if ( isset( $p_o_new->upgrade_notice ) && strlen( trim( $p_o_new->upgrade_notice ) ) > 0 ) {
		echo __( 'Upgrade Notice:', 'anrghg' ) . ' ' . trim( $p_o_new->upgrade_notice );
		echo '<style>p:empty{display:none;}</style>';
   }
}, 10, 2);

/**
 * Method and documentation.
 *
 * @link https://stackoverflow.com/questions/6703/when-is-oop-better-suited-for
 * @link https://stackoverflow.com/questions/8344008/is-oop-necessary-in-php-sites-cant-i-just-apply-its-concept-to-procedural-code
 * @link https://www.oreilly.com/content/object-oriented-vs-functional-programming/
 * If preferred, expected or required, OOP will eventually be used if and only if
 * the code would be refactored for the purpose later on. But for the time being,
 * most of PHP, CSS, HTML and JS is in these two files, to facilitate review, and
 * because the development of this plugin has been fast-tracked to address delays
 * and started from scratch to give way to necessary innovation, while reusing as
 * many parts as appropriate from those set out in that intent, or contributed to
 * other projects before, as acknowledged in the contributors list and docblocks.
 *
 * A @contributor** or a @reporter**, whose tag is followed by a double asterisk,
 * has authored the linked contribution (code, design or both) for another plugin
 * but is acknowledged in this plugin for using their code/advice/report/feedback
 * under the GNU GPL. The @author tag is not used, per WordPress Coding Standard.
 *
 * A @contributor* with a single asterisk has contributed indirectly, e.g. when a
 * change was contributed to a plugin, then the fix is backported to this plugin.
 *
 * The PHP Documentor tags need to be completed with @type and @hooked as used in
 * the WordPress ecosystem, as well as with FOSS related tags, like @contributor.
 * @see ./customized-documentation-schema.txt
 *
 * To avoid overloading the core files with docblocks, the numerous configuration
 * filter hooks are documented in the configuration filter hook template file.
 * @see ./templates-for-configuration-filters.php
 */

/**
 * External stylesheets.
 *
 * External stylesheets have many downsides, such as failures to cache bust (when
 * version number URL parameters are removed by third parties for security), even
 * sometimes stylesheet dequeuing when CSS is internalized as well as replacement
 * overriding by Custom CSS, that is often inherently out of sync with changes in
 * the plugin. For flexibility, part of the rules can only be internal CSS. Since
 * the AMP-WP plugin uses internal after tree-shaking, external CSS is pointless.
 * @see * Outputs internal CSS.
 */

/**
 * Template parts.
 *
 * External template parts likewise are not flexible enough, so that their number
 * tends to explode as options add, which screws maintenance up. Customized files
 * are even harder to keep in sync. Loading partials has certainly downsides from
 * a performance perspective. By contrast, this single-file plugin is super fast.
 * @see * Core functionality callback functions.
 */

/**
 * Code formatting.
 *
 * @link https://developer.wordpress.org/coding-standards/wordpress-coding-standards/php/
 * @link https://make.wordpress.org/core/2020/03/20/updating-the-coding-standards-for-modern-php/
 * @link https://developer.wordpress.org/coding-standards/wordpress-coding-standards/javascript/
 * This plugin follows the WordPress Coding Standards except for spacing rules in
 * array offset notation beyond the case when the offset is passed by reference.
 * As cross-language consistency is not a goal of WPCS, and for a lower workload,
 * this file applies consistent spacing, also in whatever array offset notation.
 * @link https://developer.wordpress.org/coding-standards/wordpress-coding-standards/php/#space-usage
 * @link https://stackoverflow.com/questions/58206819/phpstorm-code-style-space-within-brackets-not-functioning
 * @link https://developer.wordpress.org/plugins/plugin-basics/best-practices/#boilerplate-starting-points
 * Last but not least, block comments start with `/**` throughout, whether it’s a
 * docblock or an ordinary comment since after `/*` the formatter used in VS Code
 * doesn’t add automatic ` * ` on hitting Enter, and also for workload reduction.
 */

/**
 * Actions at plugin activation (provisional).
 *
 * @link https://developer.wordpress.org/plugins/plugin-basics/activation-deactivation-hooks/#activation
register_activation_hook( __FILE__, 'anrghg_activate' );
function anrghg_activate() {
}
 */

/**
 * Actions at plugin deactivation (provisional).
 *
 * @link https://developer.wordpress.org/plugins/plugin-basics/activation-deactivation-hooks/#deactivation
register_deactivation_hook( __FILE__, 'anrghg_deactivate' );
function anrghg_deactivate() {
}
 */

/**
 * Actions at plugin uninstallation (provisional).
 *
 * @link https://developer.wordpress.org/plugins/plugin-basics/uninstall-methods/#method-1-register_uninstall_hook
register_uninstall_hook(__FILE__, 'anrghg_uninstall');
function anrghg_uninstall() {
}
 */

/**
 * Loads localized strings once available.
 *
 * @since 0.24.4
 * @return void
 */
add_filter( 'init', function() {
	load_plugin_textdomain( 'anrghg', false, plugin_basename( __DIR__ ) . '/translations' );
} );

/**
 * Adds complementary links to the plugin's list entry.
 *
 * @since 0.30.0
 * @param  array $p_a_links  Links added by default.
 * @return array $p_a_links  Unreordered with new links appended.
 */
add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), function ( $p_a_links ) {
	$l_a_options = get_option( 'anrghg' );
	if ( 'sub' === $l_a_options[ 'anrghg_menu_level' ] ) {
		$p_a_links[] = '<a href="' . admin_url( 'options-general.php?page=anrghg' ) . '">' . __( 'Settings', 'anrghg' ) . '</a>';
		$p_a_links[] = '<a href="' . admin_url( 'tools.php?page=anrghg-reuse' ) . '">' . __( 'Reusables', 'anrghg' ) . '</a>';
	} else {
		$p_a_links[] = '<a href="' . admin_url( 'admin.php?page=anrghg' ) . '">' . __( 'Settings', 'anrghg' ) . '</a>';
		$p_a_links[] = '<a href="' . admin_url( 'admin.php?page=anrghg-reuse' ) . '">' . __( 'Reusables', 'anrghg' ) . '</a>';
	}
	return $p_a_links;
} );

/**
 * Introduces global string for the format conversion page.
 *
 * @since 0.24.6
 * @var string $g_s_anrghg_convert  Content.
 */
$g_s_anrghg_convert = get_option( 'anrghg_convert' );
if ( null === $g_s_anrghg_convert ) {
	$g_s_anrghg_convert = '';
}

/**
 * Introduces global array for reusable complements.
 *
 * @since 0.9.0
 * @var array $g_a_anrghg_reuse  Reusable complements.
 */
$g_a_anrghg_reuse = get_option( 'anrghg_reuse' );
if ( empty( $g_a_anrghg_reuse ) ) {
	$g_a_anrghg_reuse = array( 'key' => array(), 'val' => array() );
}

/**
 * Introduces global array for fragment ID disambiguation.
 *
 * @since 0.9.0
 * @var array $g_a_anrghg_fragment_ids  Helps disambiguate IDs across both headings and paragraphs.
 */
$g_a_anrghg_fragment_ids = array();

/**
 * Introduces global string to store footer lists.
 *
 * @since 0.9.0
 * @var string $g_s_anrghg_footer  Complement lists if output in the footer.
 */
$g_s_anrghg_footer = '';

/**
 * Introduces global array for configuration.
 *
 * @since 0.9.0
 * Out of the DB, all configuration data is strings.
 * Therefore all defaults defined below are strings.
 * This failed a test:
 *
 *     $g_a_anrghg_config = wp_parse_args( get_option( 'anrghg' ) );
 *
 * @var array $g_a_anrghg_config  Configuration keys and values.
 */
$g_a_anrghg_config = array();

/**
 * Defines fast-tracked configuration defaults and loads all values.
 *
 * @since 0.25.3
 * Some early required values cannot wait for the moment when
 * translation manager plugins can filter DB calls. Therefore
 * these need to be fast-tracked.
 *
 * @var array $g_a_fast_tracked_defaults
 */
$g_a_fast_tracked_defaults = array(

	/**
	 * Complement lists output in footer.
	 * 
	 * @since 0.25.3
	 * @since 0.30.0
	 * The `anrghg_complement_list_footer_deferral` key has no '-1' anymore.
	 * Backcompat with pre-0.30.0 config requires a '-1' recovery conversion.
	 */
	'anrghg_complement_list_footer_deferral'  => '0', // 0,1
	'anrghg_complement_list_output_buffering' => '0',

);
$g_a_registered = get_option( 'anrghg' );
foreach ( $g_a_fast_tracked_defaults as $l_s_key => $l_s_value ) {
	$g_a_anrghg_config[ $l_s_key ] = isset( $g_a_registered[ $l_s_key ] ) ?
		$g_a_registered[ $l_s_key ] :
		$g_a_fast_tracked_defaults[ $l_s_key ];
}
if ( '-1' === $g_a_fast_tracked_defaults[ 'anrghg_complement_list_footer_deferral' ] ) {
	$g_a_fast_tracked_defaults[ 'anrghg_complement_list_footer_deferral' ] = '0';
}

/**
 * Defines all other configuration defaults and loads all values again.
 *
 * @since 0.9.0
 *
 * @since 0.24.4
 * Filters cannot be applied here as they aren’t added yet.
 * Do nothing before the `plugins_loaded` hook is fired, else other plugins
 * are unable to filter configurable strings as they’re loaded from the DB.
 * @contributor** @chouby
 * @link https://wordpress.org/support/topic/plugin-string-translations-ineffective/#post-14714185
 * @link https://polylang.pro/dont-take-any-action-before-plugins_loaded-is-fired/
 * @reporter** @neondlight
 * @link https://wordpress.org/support/topic/reference-container-heading-translations-and-a-collapse-feature/#post-14717295
 *
 * Also, the keys should be prefixed like filters, to avoid namespace conflicts and
 * for ease of retrieval in string translation panels as well as in web DB editors.
 * The keys of all configurable strings must be in the language configuration file.
 * @link https://wpml.org/documentation/support/language-configuration-files/
 *
 * @global array $g_a_anrghg_config
 * @global array $g_a_fast_tracked_defaults
 * @return void
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config, $g_a_fast_tracked_defaults;
	$l_a_defaults = array(

		/**
		 * General options.
		 */
		'anrghg_keep_reusables_history'          => '1',
		'anrghg_keep_settings_history'           => '1',
		'anrghg_menu_level'                      => 'sub', // top,sub
		'anrghg_menu_position'                   => 'low', // top,mid,low
		'anrghg_menu_remove_duplicate'           => '0',
		'anrghg_format_conversion_active'        => '0', // DEACTIVATE WHILE UNDER CONSTRUCTION.
		'anrghg_meta_box_active'                 => '0', // Deactivate by default for users to opt in rather than out.
		'anrghg_meta_box_contents_insert'        => '1',
		'anrghg_meta_box_complements_dir'        => '1',
		'anrghg_meta_box_list_labels'            => '1',
		'anrghg_meta_box_list_output_footer'     => '1', // @todo
		'anrghg_section_block_default_dir_notes' => '-11', // -11,-1,0,1 @todo
		'anrghg_section_block_default_dir_refs'  => '-11', // -11,-1,0,1 @todo
		'anrghg_section_block_settings'          => '2', // 0,1,2 @todo
		'anrghg_block_escape_pipe_alert'         => '1', // @todo

		/**
		 * Security.
		 */
		'anrghg_login_deactivation_profile' => '0', // -1,0,1
		'anrghg_auth_duration_edit'         => '0',
		'anrghg_auth_expiration_days'       => '31',

		/**
		 * Append a message to posts or pages.
		 */
		'anrghg_thank_you_active'          => '0', // 0,1,2,3 (none,posts,pages,both)
		'anrghg_thank_you_content'         => __( 'Thank you for reading!', 'anrghg' ),
		'anrghg_thank_you_content_page'    => __( 'Thank you for reading this page!', 'anrghg' ),
		'anrghg_thank_you_priority_select' => '0', // -1,0,1
		'anrghg_thank_you_priority_input'  => '10',

		/**
		 * Display last modified (and/or published) date.
		 */
		'anrghg_dates_active'               => '0',
		'anrghg_dates_post_top_modif'       => '1',
		'anrghg_dates_post_top_publi'       => '0',
		'anrghg_dates_post_end_modif'       => '0',
		'anrghg_dates_post_end_publi'       => '0',
		'anrghg_dates_page_top_modif'       => '0',
		'anrghg_dates_page_top_publi'       => '0',
		'anrghg_dates_page_end_modif'       => '1',
		'anrghg_dates_page_end_publi'       => '1',
		'anrghg_dates_chronological'        => '0',
		'anrghg_dates_label_uni'            => '1',
		'anrghg_dates_label_modified'       => __( 'Last modified:', 'anrghg' ),
		'anrghg_dates_label_published'      => __( 'Published:', 'anrghg' ),
		'anrghg_dates_label_post_top_modif' => __( 'Post last modified:', 'anrghg' ),
		'anrghg_dates_label_post_top_publi' => __( 'Post published:', 'anrghg' ),
		'anrghg_dates_label_post_end_modif' => __( 'This post was last modified on', 'anrghg' ),
		'anrghg_dates_label_post_end_publi' => __( 'This post was published on', 'anrghg' ),
		'anrghg_dates_label_page_top_modif' => __( 'Page last modified:', 'anrghg' ),
		'anrghg_dates_label_page_top_publi' => __( 'Page published:', 'anrghg' ),
		'anrghg_dates_label_page_end_modif' => __( 'This page was last modified on', 'anrghg' ),
		'anrghg_dates_label_page_end_publi' => __( 'This page was published on', 'anrghg' ),
		'anrghg_dates_priority_select'      => '0', // -1,0,1
		'anrghg_dates_priority_input'       => '1500',

		/**
		 * Add date meta tags in the page head.
		 */
		'anrghg_date_meta_tags_active' => '0',
		'anrghg_date_meta_tags_format' => 'common', // 1[Open Graph],common[fallback]

		/**
		 * Links to paragraphs and list items.
		 */
		'anrghg_paragraph_links_active'   => '0',
		'anrghg_paragraph_link_character' => '¶',

		/**
		 * Table of contents and heading links.
		 */
		'anrghg_heading_links_active'         => '0', // 0,1
		'anrghg_table_of_contents_active'     => '0', // -1,0,1
		'anrghg_table_of_contents_label'      => __( 'Table of contents', 'anrghg' ),
		'anrghg_table_of_contents_min_number' => '2',
		'anrghg_table_of_contents_depth'      => '6', // 6,5,4,3,2
		'anrghg_table_of_contents_position'   => '0', // 1,0,-1 (top,before-first-heading,end)
		'anrghg_table_of_contents_positioner' => '[TOC]',
		'anrghg_table_of_contents_collapse'   => '1', // 0,1,-1 (no,collapsible-collapsed,collapsible-expanded)

		/**
		 * Complements as end notes and references.
		 */
		'anrghg_complements_active'              => '0',
		'anrghg_note_delimiter_set'              => '1', // -1[easy],0[free],1[safe]
		'anrghg_free_note_start'                 => '[σημ]',
		'anrghg_free_note_end'                   => '[/σημ]',
		'anrghg_note_start'                      => '',
		'anrghg_note_end'                        => '',
		'anrghg_ref_delimiter_set'               => '1', // -1[easy],0[free],1[safe]
		'anrghg_free_ref_start'                  => '[упом]',
		'anrghg_free_ref_end'                    => '[/упом]',
		'anrghg_reference_start'                 => '',
		'anrghg_reference_end'                   => '',
		// Translators: List heading label for 3 or more notes.
		'anrghg_list_label_notes_plural'         => _x( 'Notes', 'Plural', 'anrghg' ), // @todo
		// Translators: List heading label for 3 or more references.
		'anrghg_list_label_references_plural'    => _x( 'References', 'Plural', 'anrghg' ), // @todo
		'anrghg_note_list_layout'                => '1', // 1,2,3 columns
		'anrghg_reference_list_layout'           => '1', // 1,2,3 columns
		'anrghg_display_urls_note_list'          => '0',
		'anrghg_display_urls_reference_list'     => '0',
		// Incomplete!

		/**
		 * Use the URL slug as a CSS selector.
		 */
		'anrghg_slug_body_class_active' => '0',

		/**
		 * Option to deactivate WPTexturize.
		 */
		'anrghg_wptexturize_active' => '1',

		/**
		 * Additional cascading style sheet
		 */
		'anrghg_additional_css' => '',

		// Incomplete!
	);

	/**
	 * Prepares the final configuration array.
	 */
	$l_a_defaults      = $l_a_defaults + $g_a_fast_tracked_defaults;
	$l_a_registered    = get_option( 'anrghg' );
	$g_a_anrghg_config = array();
	foreach ( $l_a_defaults as $l_s_key => $l_s_value ) {
		$g_a_anrghg_config[ $l_s_key ] = isset( $l_a_registered[ $l_s_key ] ) ?
			$l_a_registered[ $l_s_key ] :
			$l_a_defaults[ $l_s_key ];
	}

	switch ( $g_a_anrghg_config[ 'anrghg_note_delimiter_set' ] ) {
		case '-1':
			$g_a_anrghg_config[ 'anrghg_note_start' ] = '{{{';
			$g_a_anrghg_config[ 'anrghg_note_end' ]   = '}}}';
			break;
		case '0':
			$g_a_anrghg_config[ 'anrghg_note_start' ] = $g_a_anrghg_config[ 'anrghg_free_note_start' ];
			$g_a_anrghg_config[ 'anrghg_note_end' ]   = $g_a_anrghg_config[ 'anrghg_free_note_end' ];
			break;
		case '1':
		default:
			$g_a_anrghg_config[ 'anrghg_note_start' ] = '[note]';
			$g_a_anrghg_config[ 'anrghg_note_end' ]   = '[/note]';
	}

	switch ( $g_a_anrghg_config[ 'anrghg_ref_delimiter_set' ] ) {
		case '-1':
			$g_a_anrghg_config[ 'anrghg_reference_start' ] = '[[[';
			$g_a_anrghg_config[ 'anrghg_reference_end' ]   = ']]]';
			break;
		case '0':
			$g_a_anrghg_config[ 'anrghg_reference_start' ] = $g_a_anrghg_config[ 'anrghg_free_ref_start' ];
			$g_a_anrghg_config[ 'anrghg_reference_end' ]   = $g_a_anrghg_config[ 'anrghg_free_ref_end' ];
			break;
		case '1':
		default:
			$g_a_anrghg_config[ 'anrghg_reference_start' ] = '[ref]';
			$g_a_anrghg_config[ 'anrghg_reference_end' ]   = '[/ref]';
	}
	if ( '-1' === $g_a_anrghg_config[ 'anrghg_complement_list_footer_deferral' ] ) {
		$g_a_anrghg_config[ 'anrghg_complement_list_footer_deferral' ] = '0';
	}
	
} );

/**
 * Calls the administration pages.
 *
 * @since 0.14.1
 * Conditional loading.
 * @link https://developer.wordpress.org/plugins/plugin-basics/best-practices/#conditional-loading
 */
if ( is_admin() ) {
	include_once plugin_dir_path( __FILE__ ) . 'admin/anrghg-admin.php';
}

/**
 * Core functionality callback functions.
 *
 * @see * Method and documentation.
 * @see * External stylesheets.
 * @see * Template parts.
 * @see * Code formatting.
 *
 * Note: The function `add_action()` is a wrapper of `add_filter()` and therefore
 * both can be used interchangeably. Using `add_action()` to call back an echoing
 * function is more intuitive than using `add_filter`, but beyond that it doesn’t
 * make any difference. The actual difference is in how a given filter is applied
 * and as a consequence, whether its callback has a parameter that it returns and
 * that is supported by `apply_filters()`, or not, as by `do_action()`.
 * @see wp-includes/plugin.php:403
 */

/**
 * Deactivates WordPress login, configures authentication duration.
 *
 * @since 0.24.0
 *
 * This security enhancement allows to deactivate the WordPress login dialog
 * and to activate it using the hosting platform file manager during narrow,
 * spaced-out windows. Unlike hosting platform login information, WordPress
 * website login information is transparent to SQL injection attacks.
 * @link https://resources.infosecinstitute.com/topic/hacking-a-wordpress-site/
 * @link https://secure.wphackedhelp.com/blog/hack-wordpress-website/
 * @link https://www.cloudways.com/blog/wordpress-sql-injection-protection/
 *
 * This solution relies on periodically editing the `wp-config.php` file,
 * as else it would require storing new login data in the DB again or offer
 * a new brute force target. This is added manually in `wp-config.php`:
 *
 *     // Turn login off by replacing true with false:
 *     define( 'ANRGHG_WP_LOGIN_ACTIVE', true );
 *
 * @see templates-for-additions-to-wp-config.php
 * @see details in the respective functions’ docblocks below.
 * Other documentation is moved to the filter templates file.
 * @see ./templates-for-configuration-filters.php
 */
if ( defined( 'ANRGHG_WP_LOGIN_ACTIVE' ) ) {

	/**
	 * Activates and switches profiles.
	 *
	 * @since 0.24.0
	 * @global array $g_a_anrghg_config
	 * @return void
	 */
	add_filter( 'plugins_loaded', function() {
		global $g_a_anrghg_config;
		$l_s_login_deactivation_profile = apply_filters( 'anrghg_login_deactivation_profile', $g_a_anrghg_config[ 'anrghg_login_deactivation_profile' ] );
		if ( ! ANRGHG_WP_LOGIN_ACTIVE ) {
			if ( $l_s_login_deactivation_profile ) {
				if ( '1' === $l_s_login_deactivation_profile ) {

					/**
					 * Edits the login page/screen. High profile option.
					 *
					 * @since 0.24.0
					 *
					 * The filter `wp_login_errors` is documented in `wp-login.php`.
					 * @see wp-login.php:1377..
					 *
					 * @param  object $p_o_errors  WP Error object.
					 * @return object $p_o_errors  WP Error object.
					 */
					add_filter( 'wp_login_errors', function( $p_o_errors ) {
						login_header( __( 'Log In' ), '', $p_o_errors );
						echo '</div>';
						echo '<div style="text-align: center; font-size: 18px; background: #FF8888; padding: 30px;">';
						echo '<p>' . __( 'Deactivated action: login.' ) . '</p></div>';
						do_action( 'login_footer' );
						echo "\r\n\t</body>\r\n</html>";
						exit;
						return $p_o_errors;
					} );
				} else {

					/**
					 * Deactivates sending authentication cookies.
					 *
					 * @since 0.24.0
					 *
					 * Alternative low-profile behavior only blocks cookie generation.
					 * The filter `send_auth_cookies` is documented in `wp-includes/pluggable.php`.
					 * @see wp-includes/pluggable.php:954
					 *
					 * @return false
					 */
					add_filter( 'send_auth_cookies', '__return_false' );
				}
			} else {

					/**
					 * Edits the login page/screen. Intermediate profile option.
					 *
					 * @since 0.24.0
					 *
					 * @courtesy Jonathan Daggerhart (@daggerhart on GitHub)
					 * @link https://gist.github.com/daggerhart/d19821ff8ce836a5fc68
					 *
					 * @param  object $p_o_errors  WP Error object.
					 * @return object $p_o_errors  WP Error object.
					 */
					add_filter( 'wp_login_errors', function( $p_o_errors ) {
					login_header( __( 'Log In' ), '', $p_o_errors );
					echo '</div>';
					do_action( 'login_footer' );
					echo "\r\n\t</body>\r\n</html>";
					exit;
					return $p_o_errors;
				} );
			}
		}
		if ( apply_filters( 'anrghg_auth_duration_edit', $g_a_anrghg_config[ 'anrghg_auth_duration_edit' ] ) ) {

			/**
			 * Edits the authentication cookie lifespan.
			 *
			 * @since 0.24.0
			 *
			 * Complementary optional authentication duration configuration.
			 * @courtesy Jenson Felsberg
			 * @link http://wpcodesnippet.com/keep-users-logged-in-longer-wordpress/
			 *
			 * @since 0.24.1
			 * More fine-grained duration configuration in days instead of weeks.
			 * @see wp-includes/user.php:2153
			 *
			 * @param  int $p_n_auth_expiration_seconds
			 * @return int $p_n_auth_expiration_seconds
			 */
			add_filter( 'auth_cookie_expiration', function( $p_n_auth_expiration_seconds ) {
				$l_n_auth_expiration_days    = ( int ) apply_filters( 'anrghg_auth_expiration_days', $g_a_anrghg_config[ 'anrghg_auth_expiration_days' ] );
				$p_n_auth_expiration_seconds = 86400 * $l_n_auth_expiration_days;
				return $p_n_auth_expiration_seconds;
			} );
		}
	} );
} // /Deactivates WordPress login, configures authentication duration.

/**
 * Filters all content-related hooks to append a message.
 *
 * @since 0.9.0
 * @reporter** @vitaefit
 * @link https://wordpress.org/support/topic/footnote-doesntwork-on-category-page/
 * @contributor** @misfist
 * @link https://wordpress.org/support/topic/footnote-doesntwork-on-category-page/#post-13864859
 * Category pages can have rich HTML content in a term description with article status
 * when WordPress’ built-in partial HTML blocker is deactivated.
 * @link https://docs.woocommerce.com/document/allow-html-in-term-category-tag-descriptions/
 *
 * The priority level integer is not constrained to be positive.
 * @link https://wordpress.org/support/topic/compatibility-with-the-official-amp-plugin/
 * @link https://wordpress.stackexchange.com/questions/78829/is-there-a-limit-to-hook-priority
 * @link https://core.trac.wordpress.org/ticket/30862
 * @link https://developer.wordpress.org/plugins/hooks/actions/
 *
 * @since 0.18.0
 * As the PHP_INT_MIN constant requires at least PHP 7.0, work around its use.
 *
 * @global array  $g_a_anrghg_config  Configuration keys and values.
 * @return void
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config;
	if ( apply_filters( 'anrghg_thank_you_active', $g_a_anrghg_config[ 'anrghg_thank_you_active' ] ) ) {
		switch ( apply_filters( 'anrghg_thank_you_priority_select', $g_a_anrghg_config[ 'anrghg_thank_you_priority_select' ] ) ) {
			case '1':
				$l_n_priority = PHP_INT_MAX;
				break;
			case '-1':
				$l_n_priority = 4 === PHP_INT_SIZE ? -2147483648 : -9223372036854775808;
				break;
			case '0':
			default:
				$l_n_priority = ( int ) apply_filters( 'anrghg_thank_you_priority_input', $g_a_anrghg_config[ 'anrghg_thank_you_priority_input' ] );
		}
		add_filter(      'the_content', 'anrghg_thank_you_text', $l_n_priority );
		add_filter( 'term_description', 'anrghg_thank_you_text', $l_n_priority );
	}
} );

/**
 * Appends a message to the post or page, or both.
 *
 * @since 0.9.0
 * @courtesy Justin Busa
 * @date 2014-06-09
 * @link https://www.wpbeaverbuilder.com/creating-wordpress-plugin-easier-think/
 * Originally designed as an example and targeting posts only.
 * Extended to optionally append another message to pages.
 *
 * @param  string $p_s_content        Content of the post or page.
 * @global array  $g_a_anrghg_config  Configuration keys and values.
 * @return string $p_s_content        Content of the post or page.
 */
function anrghg_thank_you_text( $p_s_content ) {
	global $g_a_anrghg_config;
	switch ( apply_filters( 'anrghg_thank_you_active', $g_a_anrghg_config[ 'anrghg_thank_you_active' ] ) ) {
		case '1': $l_b_posts =  true; $l_b_pages = false; break;
		case '2': $l_b_posts = false; $l_b_pages =  true; break;
		case '3': $l_b_posts =  true; $l_b_pages =  true; break;
		default : $l_b_posts = false; $l_b_pages = false;
	}
	if ( $l_b_posts && is_single() ) {
		$p_s_content .= '<div class="anrghg_post_thanks" tabindex="0"><div>';
		$p_s_content .= apply_filters( 'anrghg_thank_you_content', $g_a_anrghg_config[ 'anrghg_thank_you_content' ] );
		$p_s_content .= '</div></div><!--.anrghg_post_thanks-->';
	}
	if ( $l_b_pages && is_singular( 'page' ) ) {
		$p_s_content .= '<div class="anrghg_page_thanks" tabindex="0"><div>';
		$p_s_content .= apply_filters( 'anrghg_thank_you_content_page', $g_a_anrghg_config[ 'anrghg_thank_you_content_page' ] );
		$p_s_content .= '</div></div><!--.anrghg_page_thanks-->';
	}
	return $p_s_content;
}

/**
 * Filters all content-related hooks to prepend or append dates.
 *
 * @since 0.9.0
 * @see * Filters all content-related hooks to append a message.
 * @global array  $g_a_anrghg_config  Configuration keys and values.
 * @return void
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config;
	if ( apply_filters( 'anrghg_dates_active', $g_a_anrghg_config[ 'anrghg_dates_active' ] ) ) {
		switch ( apply_filters( 'anrghg_dates_priority_select', $g_a_anrghg_config[ 'anrghg_dates_priority_select' ] ) ) {
			case '1':
				$l_n_priority = PHP_INT_MAX;
				break;
			case '-1':
				$l_n_priority = 4 === PHP_INT_SIZE ? -2147483648 : -9223372036854775808;
				break;
			case '0':
			default:
				$l_n_priority = ( int ) apply_filters( 'anrghg_dates_priority_input', $g_a_anrghg_config[ 'anrghg_dates_priority_input' ] );
		}
		add_filter(      'the_content', 'anrghg_display_dates', $l_n_priority );
		add_filter( 'term_description', 'anrghg_display_dates', $l_n_priority );
	}
} );

/**
 * Appends or prepends published and/or last modified dates to the post or page.
 *
 * @since 0.9.0
 * @courtesy Shaun Quarton
 * @date 2019-04-07
 * @link https://pagely.com/blog/display-post-last-updated-wordpress/
 * Consistently with current practice, both dates are link text of
 * the permalink, and a tooltip shows the time.
 *
 * @param  string $p_s_content        Content of the post or page.
 * @global array  $g_a_anrghg_config  Configuration keys and values.
 * @return string $p_s_content        Content of the post or page.
 */
function anrghg_display_dates( $p_s_content ) {
	global $g_a_anrghg_config;
	if ( is_singular() ) {
		$l_n_id                 = get_the_id();
		$l_s_url                = get_permalink( $l_n_id );
		$l_s_type               = get_post_type( $l_n_id );
		$l_b_post               = 'post' === $l_s_type;
		$l_b_page               = 'page' === $l_s_type;
		$l_b_uni                = ( bool ) apply_filters( 'anrghg_dates_label_uni',      $g_a_anrghg_config[ 'anrghg_dates_label_uni' ] );
		$l_b_chrono             = ( bool ) apply_filters( 'anrghg_dates_chronological',  $g_a_anrghg_config[ 'anrghg_dates_chronological' ] );
		$l_b_post_top_mod       = ( bool ) apply_filters( 'anrghg_dates_post_top_modif', $g_a_anrghg_config[ 'anrghg_dates_post_top_modif' ] );
		$l_b_post_top_pub       = ( bool ) apply_filters( 'anrghg_dates_post_top_publi', $g_a_anrghg_config[ 'anrghg_dates_post_top_publi' ] );
		$l_b_post_end_mod       = ( bool ) apply_filters( 'anrghg_dates_post_end_modif', $g_a_anrghg_config[ 'anrghg_dates_post_end_modif' ] );
		$l_b_post_end_pub       = ( bool ) apply_filters( 'anrghg_dates_post_end_publi', $g_a_anrghg_config[ 'anrghg_dates_post_end_publi' ] );
		$l_b_page_top_mod       = ( bool ) apply_filters( 'anrghg_dates_page_top_modif', $g_a_anrghg_config[ 'anrghg_dates_page_top_modif' ] );
		$l_b_page_top_pub       = ( bool ) apply_filters( 'anrghg_dates_page_top_publi', $g_a_anrghg_config[ 'anrghg_dates_page_top_publi' ] );
		$l_b_page_end_mod       = ( bool ) apply_filters( 'anrghg_dates_page_end_modif', $g_a_anrghg_config[ 'anrghg_dates_page_end_modif' ] );
		$l_b_page_end_pub       = ( bool ) apply_filters( 'anrghg_dates_page_end_publi', $g_a_anrghg_config[ 'anrghg_dates_page_end_publi' ] );
		$l_s_label_modified     = apply_filters( 'anrghg_dates_label_modified',       $g_a_anrghg_config[ 'anrghg_dates_label_modified' ] );
		$l_s_label_published    = apply_filters( 'anrghg_dates_label_published',      $g_a_anrghg_config[ 'anrghg_dates_label_published' ] );
		$l_s_label_post_top_mod = apply_filters( 'anrghg_dates_label_post_top_modif', $g_a_anrghg_config[ 'anrghg_dates_label_post_top_modif' ] );
		$l_s_label_post_top_pub = apply_filters( 'anrghg_dates_label_post_top_publi', $g_a_anrghg_config[ 'anrghg_dates_label_post_top_publi' ] );
		$l_s_label_post_end_mod = apply_filters( 'anrghg_dates_label_post_end_modif', $g_a_anrghg_config[ 'anrghg_dates_label_post_end_modif' ] );
		$l_s_label_post_end_pub = apply_filters( 'anrghg_dates_label_post_end_publi', $g_a_anrghg_config[ 'anrghg_dates_label_post_end_publi' ] );
		$l_s_label_page_top_mod = apply_filters( 'anrghg_dates_label_page_top_modif', $g_a_anrghg_config[ 'anrghg_dates_label_page_top_modif' ] );
		$l_s_label_page_top_pub = apply_filters( 'anrghg_dates_label_page_top_publi', $g_a_anrghg_config[ 'anrghg_dates_label_page_top_publi' ] );
		$l_s_label_page_end_mod = apply_filters( 'anrghg_dates_label_page_end_modif', $g_a_anrghg_config[ 'anrghg_dates_label_page_end_modif' ] );
		$l_s_label_page_end_pub = apply_filters( 'anrghg_dates_label_page_end_publi', $g_a_anrghg_config[ 'anrghg_dates_label_page_end_publi' ] );

		if ( ( $l_b_post && ( $l_b_post_top_mod || $l_b_post_end_mod ) )
			|| ( $l_b_page && ( $l_b_page_top_mod || $l_b_page_end_mod ) )
		) {
			$l_s_updated_day       = get_the_modified_time( get_option( 'date_format' ) );
			$l_s_updated_time      = get_the_modified_time( 'H:iO' );
			$l_s_updated_iso8601   = get_the_modified_time( 'Y-m-d\TH:i:sO' );
			$l_s_updated    = " <time datetime=\"$l_s_updated_iso8601\"><a href=\"$l_s_url\"";
			$l_s_updated   .= " title=\"$l_s_updated_time\">$l_s_updated_day</a></time>";
		}
		if ( ( $l_b_post && ( $l_b_post_top_pub || $l_b_post_end_pub ) )
			|| ( $l_b_page && ( $l_b_page_top_pub || $l_b_page_end_pub ) )
		) {
			$l_s_published_day     = get_the_time( get_option( 'date_format' ) );
			$l_s_published_time    = get_the_time( 'H:iO' );
			$l_s_published_iso8601 = get_the_time( 'Y-m-d\TH:i:sO' );
			$l_s_published  = " <time datetime=\"$l_s_published_iso8601\"><a href=\"$l_s_url\"";
			$l_s_published .= " title=\"$l_s_published_time\">$l_s_published_day</a></time>";
		}
		if ( $l_b_post && ( $l_b_post_top_mod || $l_b_post_top_pub ) || $l_b_page && ( $l_b_page_top_mod || $l_b_page_top_pub ) ) {
			$l_s_add  = "\r\n\r\n<div class=\"anrghg_dates_$l_s_type" . '_top';
			$l_s_add .= '"><p tabindex="0"><span>';
			if ( $l_b_chrono && ( $l_b_post_top_pub || $l_b_page_top_pub ) ) {
				if ( $l_b_uni ) {
					$l_s_add .= $l_s_label_published;
				} else {
					if ( $l_b_post ) {
						$l_s_add .= $l_s_label_post_top_pub;
					} else {
						$l_s_add .= $l_s_label_page_top_pub;
					}
				}
				$l_s_add .= $l_s_published;
				if ( $l_b_post_top_mod || $l_b_page_top_mod ) {
					$l_s_add .= "</span><br />\r\n<span>";
				}
			}
			if ( $l_b_post_top_mod || $l_b_page_top_mod ) {
				if ( $l_b_uni ) {
					$l_s_add .= $l_s_label_modified;
				} else {
					if ( $l_b_post ) {
						$l_s_add .= $l_s_label_post_top_mod;
					} else {
						$l_s_add .= $l_s_label_page_top_mod;
					}
				}
				$l_s_add .= $l_s_updated;
			}
			if ( ! $l_b_chrono && ( $l_b_post_top_pub || $l_b_page_top_pub ) ) {
				if ( $l_b_post_top_mod || $l_b_page_top_mod ) {
					$l_s_add .= "</span><br />\r\n<span>";
				}
				if ( $l_b_uni ) {
					$l_s_add .= $l_s_label_published;
				} else {
					if ( $l_b_post ) {
						$l_s_add .= $l_s_label_post_top_pub;
					} else {
						$l_s_add .= $l_s_label_page_top_pub;
					}
				}
				$l_s_add .= $l_s_published;
			}
			$l_s_add .= "</span></p></div><!--.anrghg_dates_$l_s_type-->\r\n\r\n";
			$p_s_content = $l_s_add . $p_s_content;
		}
		if ( $l_b_post && ( $l_b_post_end_mod || $l_b_post_end_pub ) || $l_b_page && ( $l_b_page_end_mod || $l_b_page_end_pub ) ) {
			$l_s_add  = "\r\n\r\n<div class=\"anrghg_dates_$l_s_type" . '_end';
			$l_s_add .= '"><p tabindex="0"><span>';
			if ( $l_b_chrono && ( $l_b_post_end_pub || $l_b_page_end_pub ) ) {
				if ( $l_b_uni ) {
					$l_s_add .= $l_s_label_published;
				} else {
					if ( $l_b_post ) {
						$l_s_add .= $l_s_label_post_end_pub;
					} else {
						$l_s_add .= $l_s_label_page_end_pub;
					}
				}
				$l_s_add .= $l_s_published;
				if ( $l_b_post_end_mod || $l_b_page_end_mod ) {
					$l_s_add .= "</span><br />\r\n<span>";
				}
			}
			if ( ( $l_b_post && $l_b_post_end_mod ) || ( $l_b_page && $l_b_page_end_mod ) ) {
				if ( $l_b_uni ) {
					$l_s_add .= $l_s_label_modified;
				} else {
					if ( $l_b_post ) {
						$l_s_add .= $l_s_label_post_end_mod;
					} else {
						$l_s_add .= $l_s_label_page_end_mod;
					}
				}
				$l_s_add .= $l_s_updated;
			}
			if ( ! $l_b_chrono ) {
				if ( ( $l_b_post && $l_b_post_end_pub ) || ( $l_b_page && $l_b_page_end_pub ) ) {
					$l_s_add .= "</span><br />\r\n<span>";
					if ( $l_b_uni ) {
						$l_s_add .= $l_s_label_published;
					} else {
						if ( $l_b_post ) {
							$l_s_add .= $l_s_label_post_end_pub;
						} else {
							$l_s_add .= $l_s_label_page_end_pub;
						}
					}
					$l_s_add .= $l_s_published;
				}
			}
			$l_s_add .= "</span></p></div><!--.anrghg_dates_$l_s_type-->\r\n\r\n";
			$p_s_content .= $l_s_add;
		}
	}
	return $p_s_content;
} // /Appends or prepends published and/or last modified dates to the post or page.

/**
 * Hooks on head actions to add meta tags.
 *
 * @since 0.9.0
 * Compatibility with AMP legacy Reader theme by `amp_post_template_head` action.
 * @contributor** Milind More @milindmore22
 * @link https://github.com/ampproject/amp-wp/issues/6055#issuecomment-815691739
 *
 * @global array $g_a_anrghg_config  Configuration keys and values.
 * @return void
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config;
	if ( apply_filters( 'anrghg_date_meta_tags_active', $g_a_anrghg_config[ 'anrghg_date_meta_tags_active' ] ) ) {
		add_filter(                'wp_head', 'anrghg_add_date_meta_tags', 1 );
		add_filter( 'amp_post_template_head', 'anrghg_add_date_meta_tags', 1 );
	}
} );

/**
 * Adds published and last modified date meta tags.
 *
 * @since 0.9.0
 * @courtesy Aurovrata Venet
 * Code Reference WordPress.org, wp_head, Re, 2017.
 * @link https://developer.wordpress.org/reference/hooks/wp_head/
 * Code Reference WordPress.org, `the_modified_date()`, Source.
 * @link https://developer.wordpress.org/reference/functions/the_modified_date/#source
 *
 * WordPress doesn’t routinely insert the published and last modified dates,
 * with respect to authors who do not wish to disclose this information.
 * That backfires when it comes to bibliography software grabbing metadata from
 * the page head, and citation styles replacing missing dates with "(n.d.)".
 * @link https://wordpress.org/support/topic/display-meta-propertyarticlepublished_time/
 * @link https://wordpress.org/support/topic/remove-propertyarticlepublished_time/
 *
 * @global array $g_a_anrghg_config  Configuration keys and values.
 * @return void
 */
function anrghg_add_date_meta_tags() {
	global $g_a_anrghg_config;
	if ( '1' === apply_filters( 'anrghg_date_meta_tags_format', $g_a_anrghg_config[ 'anrghg_date_meta_tags_format' ] ) ) {
		?>

	<meta property="article:published_time" content="<?php printf( '%s', get_the_date( 'Y-m-d\TH:i:sO' ) ); ?>" />
	<meta property="article:modified_time" content="<?php printf( '%s', get_the_modified_date( 'Y-m-d\TH:i:sO' ) ); ?>" />
<?php
	} else {
		?>

	<meta name="date" content="<?php printf( '%s', get_the_date( 'Y-m-d\TH:i:sO' ) ); ?>" />
	<meta name="last-modified" content="<?php printf( '%s', get_the_modified_date( 'Y-m-d\TH:i:sO' ) ); ?>" />
<?php
	}
	return;
}

/**
 * Adds the URL slug as a class of the body element.
 *
 * @since 0.9.0
 * Based on code from wpbeginner.
 * @link https://www.wpbeginner.com/wp-themes/how-to-add-page-slug-in-body-class-of-your-wordpress-themes/
 *
 * @since 0.12.1
 * Get the slug by calling functions rather than from properties of the $post object.
 * @link https://wordpress.stackexchange.com/questions/42117/how-do-i-retrieve-the-slug-of-the-current-page
 *
 * @since 0.26.0
 * Ensure valid CSS class names by prepending an underscore to post or page slugs starting with a digit.
 *
 * @param  array $p_a_body_classes  Classes of the body element.
 * @return array $p_a_body_classes  Classes of the body element.
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config;
	if ( apply_filters( 'anrghg_slug_body_class_active', $g_a_anrghg_config[ 'anrghg_slug_body_class_active' ] ) ) {
		add_filter( 'body_class', function( $p_a_body_classes ) {
			$l_s_slug = basename( get_permalink() );
			if ( preg_match( '/[0-9]/', $l_s_slug[ 0 ] ) ) {
				$l_s_slug = '_' . $l_s_slug;
			}
			$p_a_body_classes[] = $l_s_slug;
			return $p_a_body_classes;
		} );
	}
} );

/**
 * Protects ASCII quotation marks against replacement with curly quotation marks.
 *
 * @since 0.9.0
 * @courtesy THE WEB FLASH
 * @link https://www.thewebflash.com/how-to-properly-disable-wptexturize-in-wordpress/
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config;
	if ( ! apply_filters( 'anrghg_wptexturize_active', $g_a_anrghg_config[ 'anrghg_wptexturize_active' ] ) ) {
		add_filter( 'run_wptexturize', '__return_false' );
	}
} );

/**
 * Filters all content-related hooks to add fragment IDs.
 *
 * @since 0.9.0
 * @see * Filters all content-related hooks to append a message.
 * @global array  $g_a_anrghg_config  Configuration keys and values.
 * @return void
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config;
	if ( apply_filters( 'anrghg_paragraph_links_active', $g_a_anrghg_config[ 'anrghg_paragraph_links_active' ] )
		|| apply_filters( 'anrghg_heading_links_active', $g_a_anrghg_config[ 'anrghg_heading_links_active' ] )
		|| apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] )
	) {
		$l_n_priority = 900;
		add_filter(      'the_content', 'anrghg_fragment_ids', $l_n_priority );
		add_filter( 'term_description', 'anrghg_fragment_ids', $l_n_priority );
	}
} );

/**
 * Purple Number substitutes, heading hyperlinks and table of contents.
 *
 * @since 0.9.0
 * @courtesy Douglas Engelbart
 * Based on an original idea from Douglas Engelbart and name-giving by his daughter.
 *
 * @contributor* Karolína Vyskočilová @vyskoczilova
 * Part of the heading ID algorithm is based on the Add Anchor Links plugin from Karolína Vyskočilová @vyskoczilova.
 * @link https://github.com/vyskoczilova/add-anchor-links
 * @link https://github.com/pewgeuges/add-anchor-links/compare/version-1.0.4...pewgeuges:extend-to-paragraphs
 * @link https://github.com/pewgeuges/add-anchor-links/blob/extend-to-paragraphs/include/class-add-anchor-links.php
 *
 * @param  string $p_s_content              Content of the post or page.
 * @global array  $g_a_anrghg_config        Configuration keys and values.
 * @global array  $g_a_anrghg_fragment_ids  Keeps track of the IDs added to the page.
 * @return string $p_s_content              Content of the post or page.
 */
function anrghg_fragment_ids( $p_s_content ) {
	global $g_a_anrghg_config, $g_a_anrghg_fragment_ids;
	$l_b_paragraphs = ( bool ) apply_filters( 'anrghg_paragraph_links_active', $g_a_anrghg_config[ 'anrghg_paragraph_links_active' ] );
	$l_b_headings   = ( bool ) (
		apply_filters( 'anrghg_heading_links_active', $g_a_anrghg_config[ 'anrghg_heading_links_active' ] )
		|| apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] )
	);
	if ( is_singular() || is_category() ) {
		$g_a_anrghg_fragment_ids = array();
		if ( $l_b_headings ) {
			$p_s_content = anrghg_process_fragment_ids( $p_s_content, 'headings' );
		}
		if ( $l_b_paragraphs ) {
			$p_s_content = anrghg_process_fragment_ids( $p_s_content, 'paragraphs' );
		}
		return $p_s_content;
	} else {
		return $p_s_content;
	}
}

/**
 * Adds fragment ID and anchor element to headings and/or paragraphs.
 *
 * @since 0.9.0
 * Generates and adds the associated table of contents as well.
 * Does not support blockquote, because WordPress consistently uses p elements inside.
 *
 * @param  string $p_s_content              Content of the post or page.
 * @param  string $p_s_scope                'headings' or 'paragraphs' (also list-items).
 * @global array  $g_a_anrghg_config        Configuration keys and values.
 * @global array  $g_a_anrghg_fragment_ids  Keeps track of the IDs added to the page.
 * @return string $p_s_content              Content of the post or page.
 */
function anrghg_process_fragment_ids( $p_s_content, $p_s_scope ) {
	global $g_a_anrghg_config, $g_a_anrghg_fragment_ids;
	if ( did_action( 'wp' ) && function_exists( 'amp_is_request' ) ) {
		$l_b_amp_active = amp_is_request();
	} else {
		$l_b_amp_active = false;
	}
	$l_n_post_id                 = get_the_id();
	$l_b_headings                = ( bool ) apply_filters( 'anrghg_heading_links_active', $g_a_anrghg_config[ 'anrghg_heading_links_active' ] );
	$l_s_h_number_pos            =          '1'; // -1,0,1 (prepended,none,appended)
	$l_s_contents_position       = apply_filters( 'anrghg_table_of_contents_position', $g_a_anrghg_config[ 'anrghg_table_of_contents_position' ] );
	$l_s_contents_positioner     = apply_filters( 'anrghg_table_of_contents_positioner', $g_a_anrghg_config[ 'anrghg_table_of_contents_positioner' ] );
	$l_s_contents_position_block = '[anrghg_toc]';
	$l_b_contents                = '1' == apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] );
	$l_b_force_contents          = false;
	$l_s_contents_post_meta      = get_post_meta( $l_n_post_id, 'anrghg_insert_contents', true ); // May be empty.
	if ( 'false' === $l_s_contents_post_meta ) $l_b_contents = false;
	if ( 'true'  === $l_s_contents_post_meta ) {
		$l_b_contents       = true;
		$l_b_force_contents = true;
	}
	if ( false !== strpos( $p_s_content, $l_s_contents_position_block )
		|| false !== strpos( $p_s_content, $l_s_contents_positioner )
	) {
		$l_b_contents       = true;
		$l_b_force_contents = true;
	}
	$l_s_contents_collapse_conf  = apply_filters( 'anrghg_table_of_contents_collapse', $g_a_anrghg_config[ 'anrghg_table_of_contents_collapse' ] );
	$l_b_collapsible             = ( bool ) $l_s_contents_collapse_conf;
	$l_b_collapsed               = '1' == $l_s_contents_collapse_conf;
	$l_s_contents_label          = apply_filters( 'anrghg_table_of_contents_label', $g_a_anrghg_config[ 'anrghg_table_of_contents_label' ] );
	$l_n_contents_depth          = ( int ) apply_filters( 'anrghg_table_of_contents_depth', $g_a_anrghg_config[ 'anrghg_table_of_contents_depth' ] );
	$l_n_contents_min_number     = ( int ) apply_filters( 'anrghg_table_of_contents_min_number', $g_a_anrghg_config[ 'anrghg_table_of_contents_min_number' ] );
	$l_s_contents_slug           =          't';
	$l_s_connector               =          '-';
	$l_b_top_level_font_weight   = ( bool ) '1'; // 1,0 (bold,normal)
	$l_s_top_level_font_weight   = $l_b_top_level_font_weight ? 'bold' : 'normal';
	$l_b_stepped                 = ( bool ) '1';
	$l_n_pad_step                = $l_b_stepped ? 22 : 0; // 12,22
	$l_s_contents_h2_pad         =   $l_n_pad_step       . 'px';
	$l_s_contents_h3_pad         = ( $l_n_pad_step * 2 ) . 'px';
	$l_s_contents_h4_pad         = ( $l_n_pad_step * 3 ) . 'px';
	$l_s_contents_h5_pad         = ( $l_n_pad_step * 4 ) . 'px';
	$l_s_contents_h6_pad         = ( $l_n_pad_step * 5 ) . 'px';
	$l_s_breakpoint_width        =  768  . 'px'; // 480,768
	$l_n_pad_step                = $l_b_stepped ? 7 : 0; // 4,7
	$l_s_contents_h2_pad_mobile  =   $l_n_pad_step       . 'px';
	$l_s_contents_h3_pad_mobile  = ( $l_n_pad_step * 2 ) . 'px';
	$l_s_contents_h4_pad_mobile  = ( $l_n_pad_step * 3 ) . 'px';
	$l_s_contents_h5_pad_mobile  = ( $l_n_pad_step * 4 ) . 'px';
	$l_s_contents_h6_pad_mobile  = ( $l_n_pad_step * 5 ) . 'px';

	/**
	 * Supplies `shortcode_unautop()` functionality for positioner.
	 *
	 * @since 0.16.1
	 * Needed because the Shortcode API is not used.
	 * @link https://wordpress.org/plugins/shortcode-empty-paragraph-fix/
	 * @link https://core.trac.wordpress.org/ticket/12061
	 * @link https://wordpress.org/plugins/wpautop-control/
	 * @link https://www.iceable.com/blog/stop-wpautop-messing-shortcodes/
	 * @link https://github.com/WordPress/gutenberg/issues/4446
	 */
	$p_s_content = str_replace( '<p>[/section]</p>', '[/section]', $p_s_content );
	$p_s_content = str_replace( '<p>[anrghg_section]</p>', '[anrghg_section]', $p_s_content );
	$p_s_content = str_replace( '<p>[TOC]</p>', '[TOC]', $p_s_content );
	$p_s_content = str_replace( '<p>[anrghg_toc]</p>', '[anrghg_toc]', $p_s_content );

	/**
	 * Updates counter rules of internal CSS for errand h1 support.
	 *
	 * @since 0.15.0
	 * Supports h1 because h1 heading level formatting is available in WP.
	 * WP inherits h1 formatting from TinyMCE where it makes actual sense.
	 * Support for h1 is also consistent with standard word processing UX.
	 * Users should be warned that stray h1 have a negative impact on SEO.
	 *
	 * @since 0.24.2
	 * Debug rules by moving them from the head to a body style element.
	 *
	 * @since 0.24.13
	 * Debug list item indentation when h1 is present in the article.
	 */
	if ( $l_b_contents ) {
		preg_match_all( '/<h1[\s>]/is', $p_s_content, $l_a_matches );
		if ( 0 < count( $l_a_matches[ 0 ] ) ) {
			$l_s_internal_css_complement = preg_replace( '/\s+/', ' ', "

				.anrghg_contents_h1 {
					font-weight: $l_s_top_level_font_weight;
				}

				.anrghg_contents_h2 {
					padding: 0 $l_s_contents_h2_pad;
					font-weight: $l_s_top_level_font_weight;
				}

				.anrghg_contents_h3 {
					padding: 0 $l_s_contents_h3_pad;
				}

				.anrghg_contents_h4 {
					padding: 0 $l_s_contents_h4_pad;
				}

				.anrghg_contents_h5 {
					padding: 0 $l_s_contents_h5_pad;
				}

				.anrghg_contents_h6 {
					padding: 0 $l_s_contents_h6_pad;
				}

				@media screen and (max-width: $l_s_breakpoint_width) {

					.anrghg_contents_h2 {
						padding: 0 $l_s_contents_h2_pad_mobile;
					}

					.anrghg_contents_h3 {
						padding: 0 $l_s_contents_h3_pad_mobile;
					}

					.anrghg_contents_h4 {
						padding: 0 $l_s_contents_h4_pad_mobile;
					}

					.anrghg_contents_h5 {
						padding: 0 $l_s_contents_h5_pad_mobile;
					}

					.anrghg_contents_h6 {
						padding: 0 $l_s_contents_h6_pad_mobile;
					}

				}

				body {counter-reset: h1}
				h1 {counter-reset: h2}
				h2 {counter-reset: h3}
				h3 {counter-reset: h4}
				h4 {counter-reset: h5}
				h5 {counter-reset: h6}

				h1 .anrghg_heading_number_after::before {counter-increment: h1; content: counter(h1) '.'}
				h2 .anrghg_heading_number_after::before {counter-increment: h2; content: counter(h1) '.' counter(h2) '.'}
				h3 .anrghg_heading_number_after::before {counter-increment: h3; content: counter(h1) '.' counter(h2) '.' counter(h3) '.'}
				h4 .anrghg_heading_number_after::before {counter-increment: h4; content: counter(h1) '.' counter(h2) '.' counter(h3) '.' counter(h4) '.'}
				h5 .anrghg_heading_number_after::before {counter-increment: h5; content: counter(h1) '.' counter(h2) '.' counter(h3) '.' counter(h4) '.' counter(h5) '.'}
				h6 .anrghg_heading_number_after::before {counter-increment: h6; content: counter(h1) '.' counter(h2) '.' counter(h3) '.' counter(h4) '.' counter(h5) '.' counter(h6) '.'}

				.anrghg_contents_list {counter-reset: toc_h1}
				.anrghg_contents_h1 {counter-reset: toc_h2}
				.anrghg_contents_h2 {counter-reset: toc_h3}
				.anrghg_contents_h3 {counter-reset: toc_h4}
				.anrghg_contents_h4 {counter-reset: toc_h5}
				.anrghg_contents_h5 {counter-reset: toc_h6}

				.anrghg_contents_h1 a::before {counter-increment: toc_h1; content: counter(toc_h1) '.'}
				.anrghg_contents_h2 a::before {counter-increment: toc_h2; content: counter(toc_h1) '.' counter(toc_h2) '.'}
				.anrghg_contents_h3 a::before {counter-increment: toc_h3; content: counter(toc_h1) '.' counter(toc_h2) '.' counter(toc_h3) '.'}
				.anrghg_contents_h4 a::before {counter-increment: toc_h4; content: counter(toc_h1) '.' counter(toc_h2) '.' counter(toc_h3) '.' counter(toc_h4) '.'}
				.anrghg_contents_h5 a::before {counter-increment: toc_h5; content: counter(toc_h1) '.' counter(toc_h2) '.' counter(toc_h3) '.' counter(toc_h4) '.' counter(toc_h5) '.'}
				.anrghg_contents_h6 a::before {counter-increment: toc_h6; content: counter(toc_h1) '.' counter(toc_h2) '.' counter(toc_h3) '.' counter(toc_h4) '.' counter(toc_h5) '.' counter(toc_h6) '.'}

			" );
			$p_s_content = '<style amp-custom>' . $l_s_internal_css_complement . '</style>' . $p_s_content;
		}
	}

	/**
	 * Collects raw strings and their position.
	 */
	if       ( 'headings'   === $p_s_scope ) {
		$l_s_pattern = '/<h([1-6])(?: [^>]+)?>(.+?)<\/h\g{1} *>/is';
	} elseif ( 'paragraphs' === $p_s_scope ) {
		$l_s_pattern = '/<(p|li)(?: [^>]+)?>(.+?)<\/\g{1} *>/is';
	}
	preg_match_all( $l_s_pattern, $p_s_content, $l_a_search_results, PREG_OFFSET_CAPTURE );
	$l_n_post_id = get_the_id();
	$l_n_offset  = 0;
	if ( $l_a_search_results ) {
		foreach ( $l_a_search_results[ 2 ] as $l_n_index => $l_s_item ) {
			list( $l_s_level, $l_n_element_position ) = $l_a_search_results[ 1 ][ $l_n_index ];
			list( $l_s_text, $l_n_text_position )     = $l_s_item;
			$l_s_text_length = strlen( $l_s_text );
			if ( $l_s_text_length ) {

				/**
				 * Derives shareable fragment identifiers.
				 *
				 * @since 0.9.0
				 * Eszett and pound are suboptimally supported (replaced with 's' and '').
				 * Punctuation apostrophes are deleted otherwise, resulting in mangled strings.
				 * Letter apostrophe U+02BC would be maintained, then URI-encoded otherwise.
				 * `remove_accents()` may be preferred for readability but must be deactivatable.
				 * Remove weird truncations caused by `sanitize_title_with_dashes()` maxing at 200 chars.
				 * U+202F is not removed by `sanitize_title_with_dashes()` because it wasn’t in use.
				 * U+200B ZERO WIDTH SPACE helps protect mobile layout against long unbreakable strings.
				 *
				 * @since 0.17.2
				 * Streamline `str_replace()` by calling `mb_strtolower()` before it’s called by `sanitize_title_with_dashes()`.
				 * @link https://developer.wordpress.org/reference/functions/sanitize_title_with_dashes/
				 * Add support for more alternative polygraphs like 'č' → 'cz', 'ř' → 'rz', 'ä' → 'ae'.
				 *
				 * @since 0.23.1
				 * Convert typographic dashes and hyphens (U+2010..U+2012, U+2015) skipped by `sanitize_title_with_dashes()`.
				 */
				if ( seems_utf8( $l_s_text ) ) {
					if ( function_exists( 'mb_strtolower' ) ) {
							$l_s_fragment_id = mb_strtolower( $l_s_text, 'UTF-8' );
					}
				}
				//                                                      NNBSP,                                       ZWSP
				$l_a_needle  = array( '\'', '’', 'ʼ', '‐', '‑', '‒', '―', ' ', '−', '£',  'č',  'ř',  'å',  'ä',  'ß', '​' );
				$l_a_replace = array(  '-', '-', '_', '-', '-', '-', '-',  '', '-', 'L', 'cz', 'rz', 'aa', 'ae', 'ss', '' );
				$l_a_lang = explode( '-', get_bloginfo( 'language' ) );
				$l_s_lang = $l_a_lang[ 0 ];
				if ( 'de' === $l_s_lang || 'sv' === $l_s_lang ) {
					array_push( $l_a_needle,   'ö',  'ü' );
					array_push( $l_a_replace, 'oe', 'ue' );
				}
				$l_s_fragment_id = str_replace( $l_a_needle, $l_a_replace, $l_s_fragment_id );
				if ( true ) {
					$l_s_fragment_id = remove_accents( $l_s_fragment_id );
				}
				$l_s_fragment_id = sanitize_title_with_dashes( $l_s_fragment_id, '', 'save' );
				if ( 200 < $l_s_text_length && strrpos( $l_s_fragment_id, '-' ) ) {
					$l_s_fragment_id = substr( $l_s_fragment_id, 0, strrpos( $l_s_fragment_id, '-' ) );
				}

				/**
				 * Disambiguates identical derived identifiers.
				 *
				 * @since 0.9.0
				 */
				$l_n_counter = 0;
				foreach ( $g_a_anrghg_fragment_ids as $l_s_existing ) {
					if (
						$l_s_fragment_id === $l_s_existing
						|| ( $l_s_fragment_id . $l_s_connector . $l_n_counter ) === $l_s_existing
					) {
						$l_n_counter++;
					}
				}
				if ( 0 !== $l_n_counter ) {
					$l_s_fragment_id .= $l_s_connector . $l_n_counter;
				}
				$g_a_anrghg_fragment_ids[] = $l_s_fragment_id;

				/**
				 * Collects data for table of contents.
				 *
				 * @since 0.9.0
				 */
				$l_s_unique_id  = $l_n_post_id . $l_s_connector . $l_s_fragment_id;
				if ( 'headings' === $p_s_scope ) {
					if ( $l_b_contents ) {
						$l_n_level = ( int ) $l_s_level;
						if ( $l_n_level <= $l_n_contents_depth ) {
							$l_a_contents[ 'level' ][] = 'h' . $l_s_level;
							$l_a_contents[ 'text'  ][] = $l_s_text;
							$l_a_contents[ 'id'    ][] = $l_s_unique_id;
						}
					}

					/**
					 * Adds markup to headings.
					 *
					 * @since 0.9.0
					 *
					 * @since 0.23.1
					 * Fully expand the table of contents on accessing it from a heading backlink, too (if JavaScript is not turned off).
					 *
					 * @since 0.25.0
					 * Use new AMP action `toggleChecked()` to make table of contents collapsible also when JavaScript is turned off.
					 * @link https://github.com/ampproject/amphtml/pull/35795
					 */
					$l_s_prepended  = '<span class="anrghg_offset_base"><span class="anrghg_offset_anchor" id="';
					$l_s_prepended .= $l_s_unique_id . '"></span></span>';

					/**
					 * Adds prepended heading number if configured accordingly.
					 */
					if ( $l_b_contents && '-1' === $l_s_h_number_pos ) {
						$l_s_prepended .= '<a class="anrghg_heading_number_before href="#' . $l_s_contents_slug;
						$l_s_prepended .= $l_s_connector . $l_s_unique_id . '"';
						if ( $l_b_amp_active ) {
							$l_s_prepended .= ' on="tap:anrghg_toggle_contents_' . $l_n_post_id . '.toggleChecked(force=true)"';
						} else {
							$l_s_prepended .= " onclick=\"anrghgExpandContents$l_n_post_id()\"";
						}
						$l_s_prepended .= '></a>';
					}

					/**
					 * Adds heading link opening tag if configured accordingly.
					 */
					if ( $l_b_headings ) {
						$l_s_prepended .= '<a tabindex="0" class="anrghg_heading_link" href="#';
						$l_s_prepended .= $l_s_unique_id . '">';
					}

					$p_s_content = substr_replace(
						$p_s_content,
						$l_s_prepended,
						$l_n_text_position + $l_n_offset,
						0
					);
					$l_n_offset += strlen( $l_s_prepended );

					$l_s_apppended  = '';
					if ( $l_b_headings ) {
						$l_s_apppended .= '</a>';
					}

					/**
					 * Adds appended heading numbers if configured accordingly.
					 */
					if ( $l_b_contents && '1' === $l_s_h_number_pos ) {
						$l_s_apppended .= '<a class="anrghg_heading_number_after" href="#' . $l_s_contents_slug;
						$l_s_apppended .= $l_s_connector . $l_s_unique_id . '"';
						if ( $l_b_amp_active ) {
							$l_s_apppended .= ' on="tap:anrghg_toggle_contents_' . $l_n_post_id . '.toggleChecked(force=true)"';
						} else {
							$l_s_apppended .= " onclick=\"anrghgExpandContents$l_n_post_id()\"";
						}
						$l_s_apppended .= '></a>';
					}

					$p_s_content = substr_replace(
						$p_s_content,
						$l_s_apppended,
						$l_n_text_position + $l_s_text_length + $l_n_offset,
						0
					);
					$l_n_offset += strlen( $l_s_apppended );

				/**
				 * Adds paragraph links sort of modern purple numbers.
				 */
				} elseif ( 'paragraphs' === $p_s_scope ) {
					$l_s_prepended  = '<span class="anrghg_offset_base"><span class="anrghg_offset_anchor" id="';
					$l_s_prepended .= $l_s_unique_id;
					$l_s_prepended .= '"></span></span><a tabindex="-1" class="anrghg_fragment_link" href="#';
					$l_s_prepended .= $l_s_unique_id;
					$l_s_prepended .= '"></a>';

					$p_s_content    = substr_replace(
						$p_s_content,
						$l_s_prepended,
						$l_n_text_position + $l_n_offset,
						0
					);
					$l_n_offset  += strlen( $l_s_prepended );
				}
			}
		}

		/**
		 * Table of contents.
		 *
		 * @since 0.9.0
		 *
		 * @since 0.13.0
		 * Expand and collapse, through pure HTML+CSS without JavaScript.
		 *
		 * @since 0.23.1
		 * Use JS to fully expand the table of contents on accessing it from a heading backlink.
		 * Use AMP action `toggleClass()` to do the same on AMP pages, provided JS is turned on.
		 *
		 * @since 0.25.0
		 * Use checkbox for AMP too, so the table of contents is collapsible also when JavaScript is turned off.
		 * @link https://github.com/ampproject/amphtml/pull/35795
		 *
		 * @since 0.27.0
		 * Check for minimum number of headings.
		 */
		if ( $l_b_contents && 'headings' === $p_s_scope && ! empty( $l_a_contents[ 'id' ][ 0 ] ) ) {
			if ( $l_b_force_contents || count( $l_a_contents[ 'id' ] ) >= $l_n_contents_min_number ) {
				$l_s_contents  = "\r\n";
				if ( ! $l_b_amp_active ) {
					$l_s_contents .= '<script>' . preg_replace( '/\s+/', ' ', "

						function anrghgExpandContents$l_n_post_id() {
								document.getElementById('anrghg_toggle_contents_$l_n_post_id').checked = true;
						}

					" ) . '</script>';
				}
				$l_s_contents .= "\r\n<div class=\"anrghg_p_tag_intersperse_preventer\">\r\n";
				$l_s_contents .= '<input type="checkbox" class="anrghg_display_toggle"';
				if ( $l_b_collapsible ) {
					$l_s_contents .= ' id="anrghg_toggle_contents_' . $l_n_post_id . '"';
				} else {
					$l_s_contents .= ' tabindex="-1"';
				}
				if ( ! $l_b_collapsed ) {
					$l_s_contents .= ' checked="checked"';
				}
				$l_s_contents .= ' /><div class="anrghg_contents">';
				$l_s_contents .= '<label for="anrghg_toggle_contents_' . $l_n_post_id . '">';
				if ( $l_b_collapsible ) {
					$l_s_contents .= '<a class="anrghg_contents_anchor"><div class="anrghg_pointer ';
				} else {
					$l_s_contents .= '<div tabindex="0" aria-label="' . $l_s_contents_label . '" class="';
				}
				$l_s_contents .= 'anrghg_contents_label">' . $l_s_contents_label . '</div><!--.anrghg_contents_label-->';
				if ( $l_b_collapsible ) {
					$l_s_contents .= '</a>';
				}
				$l_s_contents .= "</label>\r\n<div class=\"anrghg_contents_list\">";
				foreach ( $l_a_contents[ 'id' ] as $l_n_index => $l_s_id ) {
					if ( ! $l_b_collapsible ) {
						$l_s_contents .= '<span class="anrghg_offset_base"><span class="anrghg_offset_anchor" id="';
						$l_s_contents .= $l_s_contents_slug . $l_s_connector . $l_s_id . '"></span></span>';
					}
					$l_s_contents .= '<div class="anrghg_contents_' . $l_a_contents[ 'level' ][ $l_n_index ];
					if ( $l_b_collapsible ) {
						$l_s_contents .= ' anrghg_contents_heading" id="' . $l_s_contents_slug . $l_s_connector . $l_s_id;
					}
					$l_s_contents .= '"><a target="_top" href="#' . $l_s_id . '" title="';
					$l_s_contents .= $l_a_contents[ 'text' ][ $l_n_index ];
					$l_s_contents .= '"><span class="anrghg_inline_spacer"></span>';
					$l_s_contents .= $l_a_contents[ 'text' ][ $l_n_index ] . "</a>\r\n</div><!--.anrghg_contents_" . $l_a_contents[ 'level' ][ $l_n_index ] . "-->\r\n";
				}
				$l_s_contents .= "\r\n</div><!--.anrghg_contents_list-->\r\n</div><!--.anrghg_contents-->";
				$l_s_contents .= "\r\n</div><!--.anrghg_p_tag_intersperse_preventer-->\r\n";

				/**
				 * Inserts the table of contents in the document.
				 */
				$l_n_positioner_pos = strpos( $p_s_content, $l_s_contents_position_block );
				if ( $l_n_positioner_pos ) {
					$p_s_content = str_replace( $l_s_contents_position_block, $l_s_contents, $p_s_content );
				} else {
					$l_n_positioner_pos = strpos( $p_s_content, $l_s_contents_positioner );
					if ( $l_n_positioner_pos ) {
						$p_s_content = str_replace( $l_s_contents_positioner, $l_s_contents, $p_s_content );
					} elseif ( '1' === $l_s_contents_position ) {
						$p_s_content = $l_s_contents . $p_s_content;
					} elseif ( '-1' === $l_s_contents_position ) {
						$p_s_content .= $l_s_contents;
					} else {
						$p_s_content = preg_replace( '/(<h\d[^>]*>)/', $l_s_contents . '$1', $p_s_content, 1 );
					}
				}
			}
		}
	}
	$p_s_content = str_replace( array( $l_s_contents_position_block, $l_s_contents_positioner ), '', $p_s_content );
	return $p_s_content;
} // /Purple Number substitutes, heading hyperlinks and table of contents.

/**
 * AMP compatible complements as endnotes and references.
 *
 * @since 0.9.0
 *
 * AMP compatibility requested by users of the Footnotes plugin.
 * @contributor** @martinneumannat
 * @link https://wordpress.org/support/topic/making-it-amp-compatible/
 * @reporter** @psykonevro
 * @link https://wordpress.org/support/topic/footnotes-is-not-amp-compatible/
 * @reporter** keshavgupta
 * @link https://wordpress.org/support/topic/footnotes-plugin-is-not-working-on-amp/
 *
 * Implementation fostered by @milindmore22 and @westonruter from AMP project.
 * @contributor** @milindmore22
 * @link https://github.com/ampproject/amp-wp/issues/5913#issuecomment-785306933
 * @link https://github.com/markcheret/footnotes/issues/48#issuecomment-814193614
 * @contributor** @westonruter
 * @link https://github.com/ampproject/amp-wp/issues/5913#issuecomment-785419655
 *
 * @return void
 */
add_filter( 'plugins_loaded', function() {
	global $g_a_anrghg_config;
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] ) ) {
		if ( true ) {
			$l_n_priority = 1000;
			add_filter(      'the_content', 'anrghg_process_complements', $l_n_priority );
			add_filter( 'term_description', 'anrghg_process_complements', $l_n_priority );
		}
		if ( true ) {
			$l_n_priority = 1000;
			add_filter( 'widget_text', 'anrghg_process_complements', $l_n_priority );
		}
	}
} );

/**
 * Processes complements as AMP compatible endnotes and references.
 *
 * @since 0.9.0
 * Processes complements based on heuristics.
 *
 * This plugin does not rely on the WordPress Shortcodes API. Here’s why.
 *
 * Shortcodes cannot be used to reliably process endnotes and references.
 * E.g. this scheme is disabled in forms set up using the WPForms plugin.
 * Also, available options are too restricted and exclude preferred ones.
 * The shortcode API fails on Non-Latin shortcodes, and it cannot process
 * delimiters made of punctuation characters; it is neither versatile nor
 * internationalized.
 *
 * Additionally, registered shortcodes are removed from other usages such
 * as schema and meta descriptions. Appropriate for most shortcodes, this
 * policy may not be so for references. Stripping these off will convey a
 * misconception of the content as containing unsourced statements, which
 * may harm the website’s overall reputation.
 * @link https://codex.wordpress.org/Shortcode_API
 *
 * Source list in a hidden div (is:todo).
 * @reporter** @ahmadword
 * @link https://wordpress.org/support/topic/remove-the-blank-at-the-end-of-the-article-where-references-are-written/
 *
 * Custom post types (is:todo).
 * @reporter** @aenniss
 * @link https://wordpress.org/support/topic/doesnt-work-in-custon-post-types/
 *
 * Combining identical complements w/ identical # and w/o backlinks (is:todo).
 * @reporter** @eno812
 * @link https://wordpress.org/support/topic/multiple-references-for-the-same-citation/
 * @reporter** @jadjournal
 * @link https://wordpress.org/support/topic/footnote-is-duplicated-when-reference-is-used-for-multiple-times-in-an-article/
 * @reporter** @stef83
 * @link https://wordpress.org/support/topic/gleiche-referenz-ofters-auf-der-seite/
 * @reporter** @companydebt
 * @link https://wordpress.org/support/topic/duplicate-but-same-number/
 * @reporter** @drjkiel
 * @link https://wordpress.org/support/topic/duplicate-but-same-number/#post-14735105
 *
 * @param  string $p_s_content        Post/page with inline complements.
 * @global array  $g_a_anrghg_config  Configuration keys and values.
 * @global array  $g_a_anrghg_reuse   Reusable complements.
 * @global string $g_s_anrghg_footer  Stores lists for output in footer.
 * @return string $p_s_content        Post/page with referrers (at least).
 */
function anrghg_process_complements( $p_s_content ) {
	global $g_a_anrghg_config, $g_a_anrghg_reuse, $g_s_anrghg_footer;
	if ( did_action( 'wp' ) && function_exists( 'amp_is_request' ) ) {
		$l_b_amp_active = amp_is_request();
	} else {
		$l_b_amp_active = false;
	}
	$l_n_post_id                   = get_the_id();
	$l_s_start_note                = apply_filters( 'anrghg_note_start', $g_a_anrghg_config[ 'anrghg_note_start' ] );
	$l_s_end_note                  = apply_filters( 'anrghg_note_end', $g_a_anrghg_config[ 'anrghg_note_end' ] );
	$l_s_start_ref                 = apply_filters( 'anrghg_reference_start', $g_a_anrghg_config[ 'anrghg_reference_start' ] );
	$l_s_end_ref                   = apply_filters( 'anrghg_reference_end', $g_a_anrghg_config[ 'anrghg_reference_end' ] );
	$l_s_referrer_concatenator     = ',';
	$l_s_name_start                = '[name]';
	$l_s_name_end                  = '[/name]';
	$l_s_tooltip_end               = '[/excerpt]';
	$l_s_tooltip_more_link_text    = __( 'Read full note', 'anrghg' );
	$l_s_list_link_start           = '[listlink]';
	$l_s_list_link_end             = '[/listlink]';
	$l_s_section_end_code_name     = '/section';
	$l_s_position_code_note        = '[notes]';
	$l_s_position_code_ref         = '[refs]';
	$l_s_footer_deferral_config    = $g_a_anrghg_config[ 'anrghg_complement_list_footer_deferral' ];
	$l_b_footer_deferral           = ( bool ) $g_a_anrghg_config[ 'anrghg_complement_list_footer_deferral' ];
	if ( '-1' === $l_s_footer_deferral_config ) $l_s_footer_deferral = false; // Backcompat with pre-0.30.0 config.
	$l_s_footer_deferral_post_meta = get_post_meta( $l_n_post_id, 'anrghg_list_footer_deferral', true );
	if ( ! empty( $l_s_footer_deferral_post_meta ) ) {
		switch ( $l_s_footer_deferral_post_meta ) {
			case  'true': $l_b_footer_deferral =  true; break;
			case 'false': $l_b_footer_deferral = false; break;
		}
	}
	$l_s_footer_deferral_code      = '[complement_list_in_footer]';
	$l_s_referrer_prefix_note      = 'N';
	$l_s_referrer_affix_note       = '';
	$l_s_referrer_prefix_ref       = '';
	$l_s_referrer_affix_ref        = '';
	// Translators: Prepended screenreader text for note referrers.
	$l_s_referrer_aria_prefix_note = _x( 'Note', 'Screen reader prefix for note referrers', 'anrghg' );
	// Translators: Appended screenreader text for note referrers.
	$l_s_referrer_aria_affix_note  = _x( 'referrer', 'Screen reader affix for note referrers', 'anrghg' );
	// Translators: Prepended screenreader text for reference referrers.
	$l_s_referrer_aria_prefix_ref  = _x( 'Reference', 'Screen reader prefix for reference referrers', 'anrghg' );
	// Translators: Appended screenreader text for reference referrers.
	$l_s_referrer_aria_affix_ref   = _x( 'referrer', 'Screen reader affix for reference referrers', 'anrghg' );
	// Translators: Accessibility label for note list.
	$l_s_list_aria_label_note      = __( 'Note list', 'anrghg' );
	// Translators: List heading label for a single note.
	$l_s_1_list_label_note         = _x( 'Note', 'Singular', 'anrghg' );
	// Translators: List heading label for two notes.
	$l_s_2_list_label_note         = _x( 'Notes', 'Dual', 'anrghg' );
	$l_s_list_label_note           = apply_filters( 'anrghg_list_label_notes_plural', $g_a_anrghg_config[ 'anrghg_list_label_notes_plural' ] );
	$l_s_list_label_note_post_meta = get_post_meta( $l_n_post_id, 'anrghg_list_label_notes', true );
	if ( ! empty( $l_s_list_label_note_post_meta ) ) {
		$l_s_1_list_label_note        = $l_s_list_label_note_post_meta;
		$l_s_2_list_label_note        = $l_s_list_label_note_post_meta;
		$l_s_list_label_note          = $l_s_list_label_note_post_meta;
	}
	// Translators: Accessibility label for reference list.
	$l_s_list_aria_label_ref       = __( 'Reference list', 'anrghg' );
	// Translators: List heading label for a single reference.
	$l_s_1_list_label_ref          = _x( 'Reference', 'Singular', 'anrghg' );
	// Translators: List heading label for two references.
	$l_s_2_list_label_ref          = _x( 'References', 'Dual', 'anrghg' );
	$l_s_list_label_ref            = apply_filters( 'anrghg_list_label_references_plural', $g_a_anrghg_config[ 'anrghg_list_label_references_plural' ] );
	$l_s_list_label_ref_post_meta  = get_post_meta( $l_n_post_id, 'anrghg_list_label_refs', true );
	if ( ! empty( $l_s_list_label_ref_post_meta ) ) {
		$l_s_1_list_label_ref         = $l_s_list_label_ref_post_meta;
		$l_s_2_list_label_ref         = $l_s_list_label_ref_post_meta;
		$l_s_list_label_ref           = $l_s_list_label_ref_post_meta;
	}
	// Translators: Screenreader prefix for a note number in the list.
	$l_s_aria_number_note          = __( 'Note number', 'anrghg' );
	// Translators: Screenreader prefix for a reference number in the list.
	$l_s_aria_number_ref           = __( 'Reference number', 'anrghg' );
	$l_s_list_heading_element      = 'h3'; // h2,h3,h4,h5,h6,p
	$l_s_slug_note                 = 'note';
	$l_s_slug_ref                  = 'ref';
	$l_s_referrer_slug             = 'r';
	$l_s_connector                 = '-';
	$l_s_writing_direction_note    = get_post_meta( $l_n_post_id, 'anrghg_writing_dir_notes', true );
	$l_s_writing_direction_ref     = get_post_meta( $l_n_post_id, 'anrghg_writing_dir_refs', true );
	$l_s_gen_tip_content_note      = sprintf( __( 'Please click %shere%s to read the note.'     , 'anrghg' ), $l_s_list_link_start, $l_s_list_link_end );
	$l_s_gen_tip_content_ref       = sprintf( __( 'Please click %shere%s to read the reference.', 'anrghg' ), $l_s_list_link_start, $l_s_list_link_end );
	$l_s_gen_tip_active            = '0'; // 0,1,2,3 (0,notes,refs,both)
	switch ( $l_s_gen_tip_active ) {
		case '3': $l_b_gen_tip_active_note =  true; $l_b_gen_tip_active_ref =  true; break;
		case '2': $l_b_gen_tip_active_note = false; $l_b_gen_tip_active_ref =  true; break;
		case '1': $l_b_gen_tip_active_note =  true; $l_b_gen_tip_active_ref = false; break;
		case '0':
		default:  $l_b_gen_tip_active_note = false; $l_b_gen_tip_active_ref = false;
	}
	$l_s_collapsible              = '3'; // 0,1,2,3 (0,notes,refs,both)
	switch ( $l_s_collapsible ) {
		case '3': $l_b_collapsible_note =  true; $l_b_collapsible_ref =  true; break;
		case '2': $l_b_collapsible_note = false; $l_b_collapsible_ref =  true; break;
		case '1': $l_b_collapsible_note =  true; $l_b_collapsible_ref = false; break;
		case '0':
		default:  $l_b_collapsible_note = false; $l_b_collapsible_ref = false;
	}
	$l_s_backlink_tooltip_mode    = '0'; // 0,1[textual],2[symbolic]
	switch ( $l_s_backlink_tooltip_mode ) {
		case '2':
			$l_s_backlink_tooltip_text = 'Alt + ⬅';
			break;
		case '1':
			$l_s_backlink_tooltip_text = __( 'Hint: Hit the backbutton instead.', 'anrghg' );
			break;
		case '0':
		default:
			$l_s_backlink_tooltip_text = '';
	}

	/**
	 * Elementor integration.
	 *
	 * @since 0.30.0
	 * @reporter** @morsagmon
	 * @link https://wordpress.org/support/topic/references-section-not-expanding-with-elementor-accordion/
	 * @reporter** @drjkiel
	 * @link https://wordpress.org/support/topic/references-not-aligned-with-rest-of-page/
	 * Needed if lists are appended to a section with remainder status,
	 * not inserted at a final positioner which is the state-of-the-art
	 * workaround solution by lack of equivalent Elementor integration.
	 */
	if (  false ) { // Test markup.
		$p_s_content = "<div><div><section><div><div><div>
		<div
		class=\"elementor-element other-class\"></div>" . $p_s_content . "
		</div>
		</div>
		</div>
		</section>
		</div>
		</div>
		";
	}
	$l_b_elementor_moot = true;
	$l_b_elementor_mode = false;
	if ( 1 !== preg_match( '/<div\s+class="elementor-element/i', $p_s_content ) ) {
		$l_b_elementor_moot = false;
	}

	/**
	 * Assesses footer deferral.
	 */
	if ( $l_b_footer_deferral ) {
		$l_b_footer         = true;
		$l_b_elementor_moot = false;
		$p_s_content = str_replace( $l_s_footer_deferral_code, '', $p_s_content );
	} else {
		if ( false === strpos( $p_s_content, $l_s_footer_deferral_code ) ) {
			$l_b_footer = false;
		} else {
			$l_b_footer         = true;
			$l_b_elementor_moot = false;
			$p_s_content = str_replace( $l_s_footer_deferral_code, '', $p_s_content );
		}
	}

	/**
	 * Dissects the post into sections if applicable.
	 *
	 * @since 0.9.0
	 *
	 * Processing per section thanks to @grflukas issue report.
	 * @reporter** @grflukas
	 * @link https://wordpress.org/support/topic/multiple-reference-containers-in-single-post/
	 *
	 * @since 0.30.0
	 * Process section separators with arguments for special list labels.
	 * Arguments may be added manually, or automatically using the block.
	 * The argument order is constrained. Key names aim at ease of input.
	 * As this is not a registered shortcode, values are delimited by `|`
	 * not `"` to work around WPTexturize. `|` is escaped at block input.
	 */
	$l_a_sections = array(
		'content'    => array(),
		'label_note' => array(),
		'label_ref'  => array(),
		'dir_note'   => array(),
		'dir_ref'    => array(),
		'remainder'  => false,
	);
	$section_end_regex = str_replace( '/', '\/', $l_s_section_end_code_name );
	if ( false === preg_match( '/\[(anrghg_section|' . $section_end_regex . ')/i', $p_s_content ) ) {
		$l_a_sections[ 'content'   ][] = $p_s_content;
		$l_a_sections[ 'remainder' ][] = true;
	} else {
		preg_match_all(
			'/\[(?:anrghg_section|' . $section_end_regex . ')(?:\s+_1=\|([^|]+?)\|)?(?:\s+_2=\|([^|]+?)\|)?(?:\s+_3=\|([^|]+?)\|)?(?:\s+_4=\|([^|]+?)\|)?(?:\s+_5=\|([^|]+?)\|)?\]/si',
			$p_s_content,
			$l_a_matches,
			PREG_OFFSET_CAPTURE
		);
		$l_n_offset = 0;
		foreach ( $l_a_matches[ 0 ] as $l_n_index => $l_a_match ) {
			list( $l_s_delim, $l_n_pos ) = $l_a_match;
			$l_a_sections[ 'content'    ][] = substr( $p_s_content, $l_n_offset, $l_n_pos - $l_n_offset );
			$l_a_sections[ 'label_note' ][] = $l_a_matches[ 1 ][ $l_n_index ][ 0 ];
			$l_a_sections[ 'label_ref'  ][] = $l_a_matches[ 2 ][ $l_n_index ][ 0 ];
			$l_a_sections[ 'dir_note'   ][] = $l_a_matches[ 3 ][ $l_n_index ][ 0 ];
			$l_a_sections[ 'dir_ref'    ][] = $l_a_matches[ 4 ][ $l_n_index ][ 0 ];
			$l_a_sections[ 'in_footer'  ][] = $l_a_matches[ 5 ][ $l_n_index ][ 0 ];
			$l_a_sections[ 'remainder'  ][] = false;
			$l_n_len_delim = strlen( $l_s_delim );
			$l_n_offset = $l_n_pos + $l_n_len_delim;
		}
		if ( $l_n_offset < strlen( $p_s_content ) ) {
			$l_a_sections[ 'content'    ][] = substr( $p_s_content, $l_n_offset );
			$l_a_sections[ 'label_note' ][] = '';
			$l_a_sections[ 'label_ref'  ][] = '';
			$l_a_sections[ 'dir_note'   ][] = '';
			$l_a_sections[ 'dir_ref'    ][] = '';
			$l_a_sections[ 'in_footer'  ][] = '';
			$l_a_sections[ 'remainder'  ][] = true;
		}
	}

	/**
	 * Processes sections or whole post.
	 *
	 * For references to nest in notes, they are processed first.
	 */
	$l_n_list_id = 1;
	foreach ( $l_a_sections[ 'content' ] as $l_n_section_index => $l_s_section ) {
		$l_a_note = array();
		$l_a_ref  = array();
		for ( $l_n_type = 1; $l_n_type < 3; $l_n_type++ ) {
			$l_s_type = 1 === $l_n_type ? 'ref' : 'note';
			$l_s_opening_delimiter = ${"l_s_start_$l_s_type"};
			$l_s_closing_delimiter = ${"l_s_end_$l_s_type"};

			/**
			 * Unifies delimiters’ HTML escapement schemas.
			 *
			 * @since 0.9.0
			 * @reporter** @patrick_here
			 * @link https://wordpress.org/support/topic/how-to-add-footnotes-shortcode-in-elementor/
			 * @reporter** @alifarahani8000
			 * @link https://wordpress.org/support/topic/after-version-2-5-10-the-ref-or-tags-are-not-longer-working/
			 * @reporter** @gova
			 * @link https://wordpress.org/support/topic/footnotes-content-number-not-sequential/
			 * While the Classic Editor (visual mode) escapes both pointy brackets,
			 * the Block Editor enforces balanced escapement only in code editor mode
			 * when the opening tag is already escaped. In visual mode, the Block Editor
			 * does not escape the greater-than sign.
			 */
			if ( preg_match( '/[&"\'<>]/', $l_s_opening_delimiter . $l_s_closing_delimiter ) ) {
				$l_s_unified_opening_delimiter = htmlspecialchars( htmlspecialchars_decode( $l_s_opening_delimiter ) );
				$l_s_unified_closing_delimiter = htmlspecialchars( htmlspecialchars_decode( $l_s_closing_delimiter ) );
				$l_s_section = str_replace( $l_s_opening_delimiter, $l_s_unified_opening_delimiter, $l_s_section );
				$l_s_section = str_replace( $l_s_closing_delimiter, $l_s_unified_closing_delimiter, $l_s_section );
				$l_s_section = str_replace( htmlspecialchars( $l_s_opening_delimiter ), $l_s_unified_opening_delimiter, $l_s_section );
				$l_s_section = str_replace( htmlspecialchars( $l_s_closing_delimiter ), $l_s_unified_closing_delimiter, $l_s_section );
				$l_s_section = str_replace( str_replace( '&gt;', '>', htmlspecialchars( $l_s_opening_delimiter ) ), $l_s_unified_opening_delimiter, $l_s_section );
				$l_s_section = str_replace( str_replace( '&gt;', '>', htmlspecialchars( $l_s_closing_delimiter ) ), $l_s_unified_closing_delimiter, $l_s_section );
				$l_s_opening_delimiter = $l_s_unified_opening_delimiter;
				$l_s_closing_delimiter = $l_s_unified_closing_delimiter;
			}

			/**
			 * Converts delimiters to regex.
			 */
			$l_s_pattern                   = '/([\/\[\]\(\)\{\}\*\+\?\|\^\$\.\\\\])/';
			$l_s_regex_opening_delimiter   = preg_replace( $l_s_pattern, '\\\\$1', $l_s_opening_delimiter );
			$l_s_regex_closing_delimiter   = preg_replace( $l_s_pattern, '\\\\$1', $l_s_closing_delimiter );
			$l_s_pattern_opening_delimiter = '/' . $l_s_regex_opening_delimiter . '/';
			$l_s_pattern_closing_delimiter = '/' . $l_s_regex_closing_delimiter . '/';

			/**
			 * Restricts processing to readable inner HTML.
			 *
			 * @since 0.9.0
			 * @reporter** @kristencahill
			 * @link https://wordpress.org/support/topic/conflict-with-accordians-plugin/
			 * In outer HTML and schema, complements are kept, but they mustn’t be processed.
			 * Plus, some strings used as delimiters may occur in JavaScript and be mistaken.
			 * Note: The lengths of both processing-ready delimiters are assumed to be equal.
			 */
			$l_n_random = random_int( 100000, PHP_INT_MAX );
			$l_s_process_opening_delimiter = "@@opening$l_n_random@@";
			$l_s_process_closing_delimiter = "@@closing$l_n_random@@";
			$l_a_tokens = array();
			preg_match_all(
				'/(<(s(cript|tyle|vg))[^>]*? *>.*?<\/\g{2} *>|<[^>]+>|[^<>]+)/is',
				$l_s_section,
				$l_a_tokens
			);
			$l_b_complement_is_open = false;
			$l_s_section = '';
			foreach ( $l_a_tokens[ 0 ] as $l_s_token ) {
				if ( '<' === $l_s_token[ 0 ] ) {
					$l_s_section .= $l_s_token;
				} else {
					do {
						if ( $l_b_complement_is_open ) {
							$l_n_delimiter_position = strpos( $l_s_token, $l_s_closing_delimiter );
						} else {
							$l_n_delimiter_position = strpos( $l_s_token, $l_s_opening_delimiter );
						}
						if ( false === $l_n_delimiter_position ) {
							break;
						} else {
							if ( $l_b_complement_is_open ) {
								$l_s_token = preg_replace(
									$l_s_pattern_closing_delimiter,
									$l_s_process_closing_delimiter,
									$l_s_token,
									1
								);
								$l_b_complement_is_open = false;
							} else {
								$l_s_token = preg_replace(
									$l_s_pattern_opening_delimiter,
									$l_s_process_opening_delimiter,
									$l_s_token,
									1
								);
								$l_b_complement_is_open = true;
							}
						}
					} while ( true );
					$l_s_section .= $l_s_token;
				}
			}

			/**
			 * Removes complements from automatic excerpts.
			 *
			 * @since 0.27.4
			 * @reporter** @martinneumannat
			 * @link https://wordpress.org/support/topic/problem-with-footnotes-in-excerpts-of-the-blog-page/
			 * In manual excerpts, complements are converted to referrers and tooltips.
			 */
			global $post;
			if ( empty( $post->post_excerpt ) && ( is_archive() || is_category() || is_tag() || is_author() || is_date() || is_post_type_archive() || is_tax() ) ) {
				$l_s_section = preg_replace(
					'/' . $l_s_process_opening_delimiter . '.+?' . $l_s_process_closing_delimiter . '/s',
					'',
					$l_s_section
				);
			}

			/**
			 * Intersperses referrer concatenators.
			 *
			 * @since 0.29.0
			 */
			$l_s_section = preg_replace(
				'/(' . $l_s_process_closing_delimiter . ')(' . $l_s_process_opening_delimiter . ')/s',
				'$1<span class="anrghg_referrer_concatenator">' . $l_s_referrer_concatenator . '</span>$2',
				$l_s_section
			);

			/**
			 * Collects complements and resolves reusables.
			 */
			if ( false !== strpos( $l_s_section, $l_s_process_opening_delimiter ) ) {
				preg_match_all(
					'/' . $l_s_process_opening_delimiter . '(.+?)' . $l_s_process_closing_delimiter . '/s',
					$l_s_section,
					$l_a_matches,
					PREG_OFFSET_CAPTURE
				);
				$l_n_offset = 0;
				$l_n_length_process_delimiter = strlen( $l_s_process_opening_delimiter );
				$l_n_number = 1;
				$l_a_complements = array();

				/**
				 * Resolves reusables at post level.
				 *
				 * @since 0.23.0
				 */
				$l_a_post_reusables  = array( 'key' => array(), 'val' => array() );
				foreach ( $l_a_matches[ 1 ] as $l_a_match ) {
					list( $l_s_complement, $l_n_position ) = $l_a_match;
					$l_n_length_complement = strlen( $l_s_complement );
					if ( $l_n_length_complement ) {
						$l_n_length_new_name = strpos( $l_s_complement, $l_s_name_end );
						if ( $l_n_length_new_name ) {
							$l_s_new_name  = substr( $l_s_complement, 0, $l_n_length_new_name );
							$l_s_complement = substr(
								$l_s_complement,
								( $l_n_length_new_name + strlen( $l_s_name_end ) )
							);
							$l_a_post_reusables[ 'key' ][] = $l_s_new_name;
							$l_a_post_reusables[ 'val' ][] = $l_s_complement;
						}
						$l_s_probe0 = '';
						$l_s_probe1 = '';
						$l_s_clip = strpos( $l_s_complement, ' ' );
						if ( $l_s_clip ) {
							$l_s_probe0 = substr( $l_s_complement, 0, $l_s_clip );
							$l_s_probe1 = substr( $l_s_complement, ( strrpos( $l_s_complement, ' ' ) + 1 ) );
						} else {
							$l_s_probe0 = $l_s_complement;
						}
						$l_b_go = true;
						foreach ( $l_a_post_reusables[ 'key' ] as $l_n_index => $l_s_name ) {
							if ( $l_s_name === $l_s_probe0 ) {
								$l_s_add = $l_s_clip ? substr( $l_s_complement, $l_s_clip + 1 ) : '';
								$l_s_complement = $l_a_post_reusables[ 'val' ][ $l_n_index ] . $l_s_add;
								$l_b_go = false;
							}
							if ( $l_b_go && ! empty( $l_s_probe1 && $l_s_name === $l_s_probe1 ) ) {
								$l_s_add = substr( $l_s_complement, strrpos( $l_s_complement, ' ' ) );
								$l_s_complement = $l_s_add . $l_a_post_reusables[ 'val' ][ $l_n_index ];
								$l_b_go = false;
							}
						}

						/**
						 * Resolves reusables at site level.
						 *
						 * @since 0.9.0
						 * @reporter** @skeeterz71
						 * @link https://wordpress.org/support/topic/possible-to-reuse-footnotes/
						 *
						 * Reusing footnotes is a widely requested yet hard-to-implement feature.
						 */
						if ( $l_b_go ) {
							foreach ( $g_a_anrghg_reuse[ 'key' ] as $l_n_index => $l_s_name ) {
								if ( $l_s_name === $l_s_probe0 ) {
									$l_s_add = $l_s_clip ? substr( $l_s_complement, $l_s_clip + 1 ) : '';
									$l_s_complement = $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] . $l_s_add;
									$l_b_go = false;
								}
								if ( $l_b_go && ! empty( $l_s_probe1 && $l_s_name === $l_s_probe1 ) ) {
									$l_s_add = substr( $l_s_complement, 0, strrpos( $l_s_complement, ' ' ) );
									$l_s_complement = $l_s_add . ' ' . $g_a_anrghg_reuse[ 'val' ][ $l_n_index ];
								}
							}
						}

						/**
						 * Line-wraps visible URLs for Unicode-non-conformant browsers.
						 *
						 * @since 0.12.0
						 * Browsers implementing Unicode UAX#14 are able to line-wrap URLs at slashes.
						 * This algorithm has previously been contributed to another plugin.
						 * @reporter** @andreasra
						 * @link https://wordpress.org/support/topic/footnotes-appearing-in-header/page/3/#post-13657398
						 * @reporter** @karolszakiel
						 * @link https://wordpress.org/support/topic/footnotes-on-mobile-phones/
						 * @reporter** @bjrnet21
						 * @link https://wordpress.org/support/topic/2-1-4-breaks-on-my-site-images-dont-show/
						 * @reporter** @a223123131
						 * @link https://wordpress.org/support/topic/broken-layout-starting-version-2-1-4/
						 * @reporter** @spiralofhope2
						 * @link https://wordpress.org/support/topic/two-links-now-breaks-footnotes-with-blogtext/
						 * @reporter** @rjl20
						 * @link https://wordpress.org/support/topic/two-links-now-breaks-footnotes-with-blogtext/#post-13825479
						 * @reporter** @spaceling
						 * @link https://wordpress.org/support/topic/two-links-now-breaks-footnotes-with-blogtext/#post-13825532
						 * @reporter** @lukashuggenberg
						 * @link https://wordpress.org/support/topic/2-2-6-breaks-all-footnotes/
						 * @reporter** @klusik
						 * @link https://wordpress.org/support/topic/2-2-6-breaks-all-footnotes/#post-13825885
						 * @reporter** @friedrichnorth
						 * @link https://wordpress.org/support/topic/footnotes-dont-show-after-update-to-2-2-6/
						 * @reporter** @bernardzit
						 * @link https://wordpress.org/support/topic/footnotes-dont-show-after-update-to-2-2-6/#post-13826029
						 * @reporter** @rumperuu
						 * @link https://wordpress.org/support/topic/line-wrap-href-regex-bug/
						 * @link https://stackoverflow.com/questions/814700/http-url-allowed-characters-in-parameter-names
						 * @link https://webmasters.stackexchange.com/questions/93540/are-spaces-in-href-valid
						 */
						if ( true ) {
							$l_s_complement = preg_replace(
								'/(?<![-\w\.!~\*\'\(\);]=[\'"])(?<![-\w\.!~\*\'\(\);]=[\'"] )(?<![-\w\.!~\*\'\(\);]=[\'"]  )(?<![-\w\.!~\*\'\(\);]=)(?<!\/)((ht|f)tps?:\/\/[^\\s<]+)/',
								'<span class="anrghg_url_wrap">$1</span>',
								$l_s_complement
							);
						}

						/**
						 * Stores the data for the lists, prepares the slug.
						 */
						$l_a_complements[ 'number' ][] = $l_n_number;
						$l_a_complements[ 'text'   ][] = $l_s_complement;
						$l_s_id_endpoint  = $l_s_connector . $l_n_post_id . $l_s_connector . $l_n_list_id;
						$l_s_id_endpoint .= $l_s_connector . $l_n_number;
						$l_s_slug_endpoint = ${"l_s_slug_$l_s_type"} . $l_s_id_endpoint;

						/**
						 * Sets up generic tooltips.
						 *
						 * @since 0.22.0
						 * @reporter** @mushlih
						 * @link https://wordpress.org/support/topic/make-all-classes-the-same/
						 */
						if ( ${"l_b_gen_tip_active_$l_s_type"} ) {
							$l_s_tooltip_text = ${"l_s_gen_tip_content_$l_s_type"};
						} else {

							/**
							 * Sets up dedicated tooltip text.
							 *
							 * @since 0.9.0
							 * @reporter** @jbj2199
							 * @link https://wordpress.org/support/topic/change-tooltip-text/
							 */
							$l_n_length_tooltip_text = strpos( $l_s_complement, $l_s_tooltip_end );
							if ( $l_n_length_tooltip_text ) {
								$l_s_tooltip_text  = substr( $l_s_complement, 0, $l_n_length_tooltip_text );
								if ( true ) {
									$l_s_tooltip_text .= " <a class=\"anrghg_tooltip_more\" href='#$l_s_slug_endpoint'>";
									$l_s_tooltip_text .= $l_s_tooltip_more_link_text . '</a>';
								}
							} else {
								$l_s_tooltip_text = $l_s_complement;
							}
						}
						$l_s_tooltip_text = str_replace( $l_s_list_link_start, "<a href='#$l_s_slug_endpoint'>", $l_s_tooltip_text );
						$l_s_tooltip_text = str_replace( $l_s_list_link_end, '</a>', $l_s_tooltip_text );

						/**
						 * Handles newlines and paragraph breaks in complements.
						 *
						 * @reporter** @droodman
						 * @link https://wordpress.org/support/topic/eats-text-when-there-are-linebreaks-in-footnotes-in-lists/
						 */
						$l_s_tooltip_text = str_replace( array( '</p>', '<p>' ), '<br />', $l_s_tooltip_text );

						/**
						 * Generates referrers and tooltips.
						 */
						$l_s_processed_complement  = '<span class="anrghg_complement_referrer">';
						$l_s_processed_complement .= '<a target="_top" href="#' . $l_s_slug_endpoint;
						$l_s_processed_complement .= "\" class=\"anrghg_referrer_$l_s_type\"";
						$l_s_processed_complement .= ' aria-label="' . ${"l_s_referrer_aria_prefix_$l_s_type"};
						$l_s_processed_complement .= ' ' . $l_n_number . ' ';
						$l_s_processed_complement .= ${"l_s_referrer_aria_affix_$l_s_type"} . '">';
						$l_s_processed_complement .= '<span class="screenreadertext">';
						$l_s_processed_complement .= ${"l_s_referrer_aria_prefix_$l_s_type"} . ' </span>';
						$l_s_processed_complement .= ${"l_s_referrer_prefix_$l_s_type"} . $l_n_number;
						$l_s_processed_complement .= ${"l_s_referrer_affix_$l_s_type"};
						$l_s_processed_complement .= '<span class="screenreadertext"> ';
						$l_s_processed_complement .= ${"l_s_referrer_aria_affix_$l_s_type"} . '</span>';
						$l_s_processed_complement .= '</a><span class="anrghg_offset_base">';
						$l_s_processed_complement .= '<span class="anrghg_offset_anchor" id="';
						$l_s_processed_complement .= $l_s_referrer_slug . $l_s_id_endpoint . '"></span></span>';

						/**
						 * Adds directionality sensitive tooltips.
						 *
						 * @since 0.9.0
						 * Support only writing direction set by page language.
						 * @since 0.29.0
						 * Support writing direction tailored to the post and the complement type.
						 * @since 0.30.0
						 * Support writing direction tailored to the instance from section divider arguments.
						 */
						if ( true ) {
							$l_s_processed_complement .= '<span class="anrghg_complement_tooltip';
							$l_s_dir = ${"l_s_writing_direction_$l_s_type"};
							$l_s_block_dir = $l_a_sections[ "dir_$l_s_type" ][ $l_n_section_index ];
							switch ( $l_s_block_dir ) {
								case '-1': $l_s_dir = 'rtl'; break;
								case  '1': $l_s_dir = 'ltr'; break;
							}
							if ( ! empty( $l_s_dir ) ) {
								$l_s_processed_complement .= '" dir="' . $l_s_dir;
							}
							$l_s_processed_complement .= '">' . $l_s_tooltip_text . '</span>';
						}

						$l_s_processed_complement .= '</span>';
						$l_s_section = substr_replace(
							$l_s_section,
							$l_s_processed_complement,
							$l_n_position - $l_n_length_process_delimiter + $l_n_offset,
							$l_n_length_complement + ( $l_n_length_process_delimiter * 2 )
						);
						$l_n_offset -= $l_n_length_complement + ( $l_n_length_process_delimiter * 2 );
						$l_n_offset += strlen( $l_s_processed_complement );
						$l_n_number++;
					}
				}

				/**
				 * Prepares data for lists.
				 */
				${"l_a_$l_s_type"} = $l_a_complements;
				${"l_n_list_id_$l_s_type"} = $l_n_list_id;
				$l_n_list_id++;
			}
		}

		/**
		 * Displays collected complements in lists.
		 *
		 * Although tabular, the data is not displayed in a table but in divs
		 * for a more fluid layout, supporting a multi column display option,
		 * and for accessing complements by opening a direct link. This needs
		 * hard links and use of the `:target` selector (requires hard links)
		 * as documented in the CSS section.
		 * @reporter** @1a-spielwiese
		 * @link https://wordpress.org/support/topic/making-footnotes-unhidden-when-a-someone-tries-to-address-them-via-hyperlink/
		 * @see * Rules for complement lists.
		 */
		if ( ! ( is_archive() || is_category() || is_tag() || is_author() || is_date() || is_post_type_archive() || is_tax() ) ) {

			for ( $l_n_type = 1; $l_n_type < 3; $l_n_type++ ) {
				$l_s_type = 1 === $l_n_type ? 'note' : 'ref';

				/**
				 * 1: Generates notes list and appends it to section.
				 * 2: Appends references list to notes list.
				 */
				if ( ! empty( ${"l_a_$l_s_type"} ) ) {
					$l_s_list_id      = $l_n_post_id . '_' . ${"l_n_list_id_$l_s_type"};
					$l_s_list  = "\r\n" . '<div class="anrghg_p_tag_intersperse_preventer">';
					$l_s_list .= '<input type="checkbox" class="anrghg_display_toggle"';
					$l_s_list .= ' aria-label="' . ${"l_s_list_aria_label_$l_s_type"} . '"';
					if ( ${"l_b_collapsible_$l_s_type"} ) {
						$l_s_list .= ' id="anrghg_toggle_' . $l_s_list_id . '"';
					} else {
						$l_s_list .= ' checked="checked" tabindex="-1"';
					}
					$l_s_list .= ' /><div role="table" class="anrghg_complement_list';
					$l_s_list .= " anrghg_$l_s_type" . 's';

					/**
					 * Sets the writing direction.
					 *
					 * @since 0.29.0
					 * Support writing direction tailored to the post and the complement type.
					 * @since 0.30.0
					 * Support writing direction tailored to the instance from section divider arguments.
					 */
					$l_s_dir = ${"l_s_writing_direction_$l_s_type"};
					$l_s_block_dir = $l_a_sections[ "dir_$l_s_type" ][ $l_n_section_index ];
					switch ( $l_s_block_dir ) {
						case '-1': $l_s_dir = 'rtl'; break;
						case  '1': $l_s_dir = 'ltr'; break;
					}
					if ( ! empty( $l_s_dir ) ) {
						$l_s_list .= '" dir="' . $l_s_dir;
					}

					if ( ${"l_b_collapsible_$l_s_type"} ) {
						$l_s_list .= '"><label for="anrghg_toggle_' . $l_s_list_id . '"><a tabindex="-1';
					}
					$l_s_list .= '"><' . $l_s_list_heading_element . ' class="anrghg_complement_heading';
					$l_s_list .= " anrghg_$l_s_type" . 's';
					if ( ${"l_b_collapsible_$l_s_type"} ) {
						$l_s_list .= ' anrghg_pointer">';
					} else {
						$l_s_list .= '" tabindex="0" aria-label="' . ${"l_s_list_aria_label_$l_s_type"} . '">';
					}
					if ( ! empty(  $l_a_sections[ "label_$l_s_type" ][ $l_n_section_index ] ) ) {
						$l_s_list .= $l_a_sections[ "label_$l_s_type" ][ $l_n_section_index ];
					} else {

						/**
						 * Accounts for singular and dual forms of list labels.
						 *
						 * @since 0.10.0
						 * The number sensitive `ngettext()` (or `_n()`) function does not seem to handle
						 * configurable strings since the number is required and strings cannot be passed
						 * by reference. The only way to do this seems to be to configure a set of cases,
						 * then handle programmatically a widely working scheme like singular/dual/plural
						 * and use `_x()` to disambiguate the dual and plural strings in English.
						 */
						if ( ! empty( ${"l_s_1_list_label_$l_s_type"} ) ) {
							if ( 1 === count( ${"l_a_$l_s_type"}[ 'number' ] ) ) {
								$l_s_list .= ${"l_s_1_list_label_$l_s_type"};
							}
							if ( ! empty( ${"l_s_2_list_label_$l_s_type"} ) ) {
								if ( 2 === count( ${"l_a_$l_s_type"}[ 'number' ] ) ) {
									$l_s_list .= ${"l_s_2_list_label_$l_s_type"};
								}
							}
						} else {
							$l_s_list .= ${"l_s_list_label_$l_s_type"};
						}
					}
					$l_s_list .= "</$l_s_list_heading_element>";
					if ( ${"l_b_collapsible_$l_s_type"} ) {
						$l_s_list .= '</a></label>';
					}
					$l_s_list .= '<div class="anrghg_complement_list_body">';
					foreach ( ${"l_a_$l_s_type"}[ 'number' ] as $l_n_index => $l_n_number ) {
						$l_s_item_endpoint  = $l_s_connector . $l_n_post_id;
						$l_s_item_endpoint .= $l_s_connector . ${"l_n_list_id_$l_s_type"};
						$l_s_item_endpoint .= $l_s_connector . $l_n_number;
						$l_s_fragment_id = ${"l_s_slug_$l_s_type"} . $l_s_item_endpoint;
						$l_s_list .= "\r\n" . '<div class="anrghg_complement_row" role="row" id="';
						$l_s_list .= $l_s_fragment_id . '"><a class="anrghg_complement_backlink"';
						$l_s_list .= ' target="_top" href="#' . $l_s_referrer_slug . $l_s_item_endpoint;
						$l_s_list .= '" title="' . $l_s_backlink_tooltip_text . '">';
						$l_s_list .= '<div class="anrghg_complement_number" role="rowheader" aria-label="';
						$l_s_list .= ${"l_s_aria_number_$l_s_type"} . ' ' . $l_n_number . '">';
						$l_s_list .= $l_n_number . '</div><!--.anrghg_complement_number--></a>';
						$l_s_list .= '<div class="anrghg_complement_list_content" role="cell" tabindex="0">';
						$l_s_complement = ${"l_a_$l_s_type"}[ 'text' ][ $l_n_index ];
						$l_n_length_tooltip_text = strpos( $l_s_complement, $l_s_tooltip_end );
						if ( $l_n_length_tooltip_text ) {
							$l_s_complement_text = substr(
								$l_s_complement,
								( $l_n_length_tooltip_text + strlen( $l_s_tooltip_end ) )
							);
						} else {
							$l_s_complement_text = $l_s_complement;
						}

						/**
						 * Prints parenthesized URL after anchor.
						 *
						 * @since 0.30.0
						 * @courtesy https://www.investopedia.com/terms/s/stagflation.asp#citation-3
						 * Investopedia contributed in that it shows a similar feature.
						 * @reporter** @companydebt
						 * @link https://wordpress.org/support/topic/duplicate-but-same-number/
						 * @companydebt shared Investopedia’s homepage in an unrelated context.
						 */
						if ( ( 'note' === $l_s_type && apply_filters( 'anrghg_display_urls_note_list', $g_a_anrghg_config[ 'anrghg_display_urls_note_list' ] ) )
							|| ( 'ref'  === $l_s_type && apply_filters( 'anrghg_display_urls_reference_list', $g_a_anrghg_config[ 'anrghg_display_urls_reference_list' ] ) )
						) {
							$l_s_complement_text = preg_replace(
								'/(<a\s[^>]*?href\s*=\s*(["\'])\s*([^"\']+)\s*\g{2}.+?<\/a\s*>)/',
								'$1 (<span class="anrghg_url_wrap">$3</span>) ',
								$l_s_complement_text
							);
						}

						$l_s_list .= $l_s_complement_text;
						$l_s_list .= "\r\n</div><!--.anrghg_complement_list_content-->\r\n</div><!--.anrghg_complement_row-->";
					}
					$l_s_list .= "\r\n</div><!--.anrghg_complement_list_body-->";
					$l_s_list .= "\r\n</div><!--.anrghg_complement_list-->";
					$l_s_list .= "\r\n</div><!--.anrghg_p_tag_intersperse_preventer-->\r\n";

					/**
					 * Inserts the list.
					 *
					 * @see * Elementor integration.
					 * Based on the assumption that six last closing tags are stable.
					 * Enters fallback mode in case unexpected markup is encountered.
					 */
					$l_s_position_code = ${"l_s_position_code_$l_s_type"};
					if ( $l_b_footer || '1' == $l_a_sections[ 'in_footer' ][ $l_n_section_index ] ) {
						$g_s_anrghg_footer .= $l_s_list;
					} else {
						if ( strpos( $l_s_section, $l_s_position_code ) ) {
							$l_s_section = str_replace( $l_s_position_code, $l_s_list, $l_s_section );
						} else {
							if ( $l_b_elementor_moot
								&& $l_a_sections[ 'remainder' ][ $l_n_section_index ] 
							) {
								if ( $l_b_elementor_mode ) {
									$l_s_section .= $l_s_list;
								} else {
									$l_a_tag_names = array( 'div', 'div', 'section', 'div', 'div', 'div' );
									$l_s_trimmed_section = $l_s_section;
									$l_b_done = true;
									foreach ( $l_a_tag_names as $l_s_tag_name ) {
										$l_s_end_tag = '</' . $l_s_tag_name . '>';
										if ( $l_s_end_tag === substr( trim( $l_s_trimmed_section ), -( strlen( $l_s_end_tag ) ) ) ) {
											$l_s_trimmed_section = substr( trim( $l_s_trimmed_section ), 0, -( strlen( $l_s_end_tag ) ) );
										} else {
											$l_b_done = false;
											break;
										}
									}
									if ( $l_b_done ) {
										$l_b_elementor_mode = true;
										$l_s_section = $l_s_trimmed_section . $l_s_list;
									} else {
										$l_s_section .= "\r\n<!-- ANRGHG Publishing Helper- Elementor integration:";
										$l_s_section .= " Entered fallback mode, due to unexpected markup. -->\r\n";
										$l_s_section .= $l_s_list;
									}
								}
							} else {
								$l_s_section .= $l_s_list;
							}
						}
					}
				}
				$l_s_section = str_replace( $l_s_position_code, '', $l_s_section );
			}
			if ( $l_b_elementor_mode ) {
				$l_s_section .= "\r\n\t\t\t\t\t\t\t</div><!-- .elementor-widget-wrap -->";
				$l_s_section .= "\r\n\t\t\t\t\t\t</div><!-- .elementor-column -->";
				$l_s_section .= "\r\n\t\t\t\t\t</div><!-- .elementor-container -->";
				$l_s_section .= "\r\n\t\t\t\t</section><!-- .elementor-section -->";
				$l_s_section .= "\r\n\t\t\t</div><!-- .elementor-section-wrap -->";
				$l_s_section .= "\r\n\t\t</div><!-- .elementor -->\r\n";
			}

			/**
			 * Appends spacing at section end.
			 *
			 * @since 0.27.0
			 * Add bottom margin below reference lists between sections.
			 */
			$l_s_section .= "\r\n<div class=\"anrghg_complements_section_spacing\"></div>\r\n";
		}

		/**
		 * Reassembles the sections.
		 */
		$l_a_processed_sections[] = $l_s_section;
	}
	$p_s_content = implode( $l_a_processed_sections );
	return $p_s_content;
} // /Processes complements as AMP compatible endnotes and references.

/**
 * Outputs complement lists in the footer.
 *
 * @since 0.11.0
 * Hooking this on the 'wp_footer' action outputs below the footer.
 *
 * @since 0.12.2
 * Anonymous function called by output buffer hooked on an early action.
 *
 * @since 0.13.0
 * Insert lists beneath the last footer start tag, as some themes have
 * multiple footer elements. Only in the absence of a footer element,
 * the initial position before the body end tag is maintained.
 *
 * @since 0.13.1
 * Because the good solution uses an output buffer, the block is first
 * inserted at the fallback position, for the CSS selectors to be there
 * when the official AMP plugin shakes the tree.
 * @link https://stackoverflow.com/questions/772510/wordpress-filter-to-modify-final-html-output
 * @link https://stackoverflow.com/questions/38693992/notice-ob-end-flush-failed-to-send-buffer-of-zlib-output-compression-1-in
 * @link to answer (share) https://stackoverflow.com/a/60665230
 *
 * @since 0.17.3
 * Not start output buffer by default, to avoid notice about missing zlib extension in PHP 7.3.
 *
 * @since 0.26.0
 * Setting to activate output buffering with notice that PHP 7.3 requires zlib extension.
 * Support moving lists into footers of very old themes repurposing any element as footer.
 *
 * @global $g_s_anrghg_footer
 */
if ( $g_a_anrghg_config[ 'anrghg_complement_list_footer_deferral' ] ) {
	add_filter( 'wp_footer', function() {
		global $g_s_anrghg_footer;
		if ( ! empty( $g_s_anrghg_footer ) ) {
			echo "\r\n\r\n" . '<div class="anrghg_footer_complement_list">';
			echo $g_s_anrghg_footer;
			echo "\r\n</div><!--.anrghg_footer_complement_list-->\r\n";
		}
	}, ( 4 === PHP_INT_SIZE ? -2147483648 : -9223372036854775808 ) );
	if ( $g_a_anrghg_config[ 'anrghg_complement_list_output_buffering' ] ) {
		add_filter( 'after_setup_theme', function() {
			ob_start( function( $p_s_html ) {
				global $g_s_anrghg_footer;
				if ( ! empty( $g_s_anrghg_footer ) ) {
					if ( false !== strpos( $p_s_html, '<footer' ) || false !== preg_match( '/<[^>]+ id=(["\'])footer\g{1}/is', $p_s_html ) ) {
						$l_n_start = strpos( $p_s_html, '<div class="anrghg_footer_complement_list">' );
						$l_n_end   = strpos( $p_s_html, '</div><!--.anrghg_footer_complement_list-->' ) + 43;
						$l_s_move  = "\r\n\r\n" . substr( $p_s_html, $l_n_start, $l_n_end - $l_n_start ) . "\r\n";
						$p_s_html  = substr_replace( $p_s_html, '', $l_n_start, $l_n_end - $l_n_start );
						if ( false !== strpos( $p_s_html, '<footer' ) ) {
							preg_match_all( '/<footer[^>]*>/is', $p_s_html, $l_a_matches, PREG_OFFSET_CAPTURE );
						} else {
							preg_match_all( '/<[^>]+ id=(["\'])footer\g{1}[^>]*>/is', $p_s_html, $l_a_matches, PREG_OFFSET_CAPTURE );
						}
						foreach ( $l_a_matches[ 0 ] as $l_a_match ) {
							list( $l_s_match, $l_n_position ) = $l_a_match;
							$l_a_positions[] = ( $l_n_position + strlen( $l_s_match ) );
						}
						$l_n_last_index = ( count( $l_a_positions ) - 1 );
						$l_n_target     = $l_a_positions[ $l_n_last_index ];
						$p_s_html       = substr_replace( $p_s_html, $l_s_move, $l_n_target, 0 );
					}
				}
				return $p_s_html;
			} );
		}, PHP_INT_MAX );
	}
} // /Outputs complement lists in the footer.

/**
 * Internal CSS.
 *
 * @since 0.9.0
 * Compatibility with AMP legacy Reader theme.
 * @contributor** @milindmore22
 * @link https://github.com/ampproject/amp-wp/issues/6055#issuecomment-815691739
 * To work around the many downsides of external stylesheets, all CSS is internal,
 * and most of it is configurable. For the code to be both the WPCS compliant and
 * streamlined, it is minified on the fly—although incompletely to keep the regex
 * simple.
 * @see * External stylesheets.
 *
 * @since 0.22.0
 * Streamline CSS selectors to reduce impact on CSS budget for AMP compatibility.
 * The list given below was slightly revised at reversal.
 * Abbr class name:   Full class name:
 * anrghg_cpl_blnk  = anrghg_complement_backlink
 * anrghg_cpl_h     = anrghg_complement_heading
 * anrghg_cpl_li    = anrghg_complement_list
 * anrghg_cpl_nr    = anrghg_complement_number
 * anrghg_cpl_row   = anrghg_complement_row
 * anrghg_cpl_rr    = anrghg_complement_referrer
 * anrghg_cpl_tt    = anrghg_complement_tooltip
 * anrghg_cpli_ct   = anrghg_complement_list_content
 * anrghg_dpl_tg    = anrghg_display_toggle
 * anrghg_fgm_lnk   = anrghg_fragment_link
 * anrghg_ft_cpl_li = anrghg_footer_complement_list
 * anrghg_h_lnk     = anrghg_heading_link
 * anrghg_h_nr_a    = anrghg_heading_number_after
 * anrghg_h_nr_b    = anrghg_heading_number_before
 * anrghg_off_a     = anrghg_offset_anchor
 * anrghg_off_b     = anrghg_offset_base
 * anrghg_ptr       = anrghg_pointer
 * anrghg_rr_nte    = anrghg_referrer_note
 * anrghg_rr_ref    = anrghg_referrer_ref
 * anrghg_tc        = anrghg_contents
 * anrghg_tc_h      = anrghg_contents_heading
 * anrghg_tc_h1     = anrghg_contents_h1
 * anrghg_tc_h2     = anrghg_contents_h2
 * anrghg_tc_h3     = anrghg_contents_h3
 * anrghg_tc_h4     = anrghg_contents_h4
 * anrghg_tc_h5     = anrghg_contents_h5
 * anrghg_tc_h6     = anrghg_contents_h6
 * anrghg_tc_lb     = anrghg_contents_label
 * anrghg_tc_lst    = anrghg_contents_list
 * anrghg_tt_mo     = anrghg_tooltip_more
 *
 * Raise priority level from PHP_INT_MAX to 100 so Theme Custom CSS overrides plugin settings.
 * Priority matters as Theme Custom CSS is expected to override plugins.
 * However, Theme Custom CSS is output at priority level 101.
 * @see wp-includes/default-filters.php:299
 *
 * @since 0.24.9
 * Revert class name abbreviations as almost insignificant for CSS budget control for AMP, while screwing up customization.
 */
$l_n_priority = 100;
add_filter(                'wp_head', 'anrghg_internal_styles', $l_n_priority );
add_filter( 'amp_post_template_head', 'anrghg_internal_styles', $l_n_priority );

/**
 * Outputs internal CSS.
 *
 * @since 0.9.0
 * @see * External stylesheets.
 *
 * @since 0.22.0
 * Opening style tag compatible with HTML < 5.
 * @see wp-includes/theme.php:1822
 *
 * @global array $g_a_anrghg_config  Configuration keys and values.
 * @return void
 */
function anrghg_internal_styles() {
	global $g_a_anrghg_config;
	if ( did_action( 'wp' ) && function_exists( 'amp_is_request' ) ) {
		$l_b_amp_active = amp_is_request();
	} else {
		$l_b_amp_active = false;
	}
	if ( $l_b_amp_active ) {
		echo "\r\n<style amp-custom>\r\n";
	} else {
		echo "\r\n<style" . ( current_theme_supports( 'html5', 'style' ) ? '' : ' type="text/css"' ) . " media=\"all\">\r\n";
	}

	/**
	 * Activate CSS smooth scrolling.
	 *
	 * @since 0.9.0
	 * @reporter** @paulgpetty
	 * @link https://wordpress.org/support/topic/functionally-great/#post-13607795
	 * @reporter** @bogosavljev
	 * @link https://wordpress.org/support/topic/compatibility-issue-with-wpforms/#post-14214720
	 * Native CSS-based smooth scrolling only works in recent browsers.
	 */
	if ( true ) {
		echo 'html {scroll-behavior: smooth;}';
	}

	/**
	 * Rules for scroll offset to fragment identifiers on public pages.
	 *
	 * @since 0.9.0
	 * The scroll-padding property cannot be used yet.
	 * Only its shorthand notation is supported only by modern browsers.
	 * @link https://css-tricks.com/almanac/properties/s/scroll-padding/
	 *
	 * @since 0.24.11
	 * Need to have scroll offset in px, not (15) vh.
	 * And to have its scalar be a distinct variable.
	 */
	$l_n_scroll_offset_scalar =  0;
	$l_s_scroll_offset        = '0';
	if ( true ) {
		$l_n_scroll_offset_scalar = 115;
		$l_s_scroll_offset        = $l_n_scroll_offset_scalar . 'px';
		if ( ! true ) {
			echo "body {scroll-padding: $l_s_scroll_offset 0 0 0;}\r\n";
		} else {
			echo "\r\n/*scroll offset*/";
			echo preg_replace( '/\s+/', ' ', "

				.anrghg_offset_base {
					position: relative;
				}

				.anrghg_offset_anchor {
					position: absolute;
					bottom: $l_s_scroll_offset;
				}

			" );
		}
	}

	/**
	 * Rules for appended Thank-you messages.
	 *
	 * @since 0.9.0
	 * Based on Justin Busa’s example feature.
	 * @link https://www.wpbeaverbuilder.com/creating-wordpress-plugin-easier-think/
	 */
	if ( apply_filters( 'anrghg_thank_you_active', $g_a_anrghg_config[ 'anrghg_thank_you_active' ] ) ) {
		$l_s_align           =         '-1'; // -1[start],0[center],1[end]
		switch ( $l_s_align ) {
			case '-1': $l_s_align_start =    '0'; $l_s_align_end = 'auto'; break;
			case  '1': $l_s_align_start = 'auto'; $l_s_align_end =    '0'; break;
			case  '0':
			default:   $l_s_align_start = 'auto'; $l_s_align_end = 'auto';
		}
		$l_s_margin_top      =   0   . 'px';
		$l_s_margin_bottom   =   0   . 'px';
		$l_s_color           =   '#000000;';
		$l_s_font_size       =  15   . 'px';
		$l_s_font_style      =     'normal'; // normal,italic
		$l_s_font_weight     =     'normal'; // normal,bold,bolder,lighter
		$l_s_text_align      =      'start'; // start,center,end
		$l_s_background      =   '#FFFFFF;';
		$l_s_max_width       = 400   . 'px';
		$l_s_margin_top      =  50   . 'px';
		$l_s_margin_end      =  16   . 'px';
		$l_s_margin_bottom   =  50   . 'px';
		$l_s_margin_start    =  16   . 'px';
		$l_s_padding_top     =  20   . 'px';
		$l_s_padding_end     =  24   . 'px';
		$l_s_padding_bottom  =  20   . 'px';
		$l_s_padding_start   =  24   . 'px';
		$l_s_border_width    =   1   . 'px';
		$l_s_border_style    = 'solid'; // solid,dotted,dashed,double,inset,outset,ridge,groove
		$l_s_border_color    = '#00FF00;';
		$l_s_radius_top      =   2   . 'px'; // px,%
		$l_s_radius_right    =   2   . 'px'; // px,%
		$l_s_radius_right    =           '';
		$l_s_radius_bottom   =   2   . 'px'; // px,%
		$l_s_radius_bottom   =           '';
		$l_s_radius_left     =   2   . 'px'; // px,%
		$l_s_radius_left     =           '';
		$l_s_shadow_x_offset =  -5   . 'px';
		$l_s_shadow_y_offset =   7   . 'px';
		$l_s_shadow_blur     =   5   . 'px';
		$l_s_shadow_spread   =   2   . 'px';
		$l_s_shadow_color    =   '#484848;';

		/**
		 * Optionally configuring pages separately.
		 */
		$l_s_page_align           =         '0'; // -1[start],0[center],1[end]
		switch ( $l_s_page_align ) {
			case '-1': $l_s_page_align_start =    '0'; $l_s_page_align_end = 'auto'; break;
			case  '1': $l_s_page_align_start = 'auto'; $l_s_page_align_end =    '0'; break;
			case  '0':
			default:   $l_s_page_align_start = 'auto'; $l_s_page_align_end = 'auto';
		}
		$l_s_page_margin_top      =   0   . 'px';
		$l_s_page_margin_bottom   =   0   . 'px';
		$l_s_page_color           =   '#000000;';
		$l_s_page_font_size       =  15   . 'px';
		$l_s_page_font_style      =     'normal'; // normal,italic
		$l_s_page_font_weight     =     'normal'; // normal,bold,bolder,lighter
		$l_s_page_text_align      =      'start'; // start,center,end
		$l_s_page_background      =  '#FFFFFF;' ;
		$l_s_page_max_width       = 400   . 'px';
		$l_s_page_margin_top      =  50   . 'px';
		$l_s_page_margin_end      =  16   . 'px';
		$l_s_page_margin_bottom   =  50   . 'px';
		$l_s_page_margin_start    =  16   . 'px';
		$l_s_page_padding_top     =  20   . 'px';
		$l_s_page_padding_end     =  24   . 'px';
		$l_s_page_padding_bottom  =  20   . 'px';
		$l_s_page_padding_start   =  24   . 'px';
		$l_s_page_border_width    =   1   . 'px';
		$l_s_page_border_style    =      'solid'; // solid,dotted,dashed,double,inset,outset,ridge,groove
		$l_s_page_border_color    =   '#00FF00;';
		$l_s_page_radius_top      =   2   . 'px'; // px,%
		$l_s_page_radius_right    =   2   . 'px'; // px,%
		$l_s_page_radius_right    =           '';
		$l_s_page_radius_bottom   =   2   . 'px'; // px,%
		$l_s_page_radius_bottom   =           '';
		$l_s_page_radius_left     =   2   . 'px'; // px,%
		$l_s_page_radius_left     =           '';
		$l_s_page_shadow_x_offset =  -5   . 'px';
		$l_s_page_shadow_y_offset =   7   . 'px';
		$l_s_page_shadow_blur     =   5   . 'px';
		$l_s_page_shadow_spread   =   2   . 'px';
		$l_s_page_shadow_color    =   '#484848;';
		if ( true ) {
			echo "\r\n/*appended message*/";
			echo preg_replace( '/\s+/', ' ', "

				body.home div.anrghg_page_thanks,
				body.home div.anrghg_post_thanks {
					display: none;
				}

			" );
		}
		if ( true ) {
			echo 'div.anrghg_page_thanks,';
		}
		echo preg_replace( '/\s+/', ' ', "

			div.anrghg_post_thanks {
				display: block;
				width: fit-content;
				margin-top: 0;
				margin-inline-end: $l_s_align_end;
				margin-bottom: 0;
				margin-inline-start: $l_s_align_start;
			}

		" );
		if ( true ) {
			echo 'div.anrghg_page_thanks div,';
		}
		echo preg_replace( '/\s+/', ' ', "

		div.anrghg_post_thanks div {
				color: $l_s_color;
				font-size: $l_s_font_size;
				font-style: $l_s_font_style;
				font-weight: $l_s_font_weight;
				text-align: $l_s_text_align;
				background-color: $l_s_background;
				max-width: $l_s_max_width;
				padding-top: $l_s_padding_top;
				padding-inline-end: $l_s_padding_end;
				padding-bottom: $l_s_padding_bottom;
				padding-inline-start: $l_s_padding_start;
				margin-top: $l_s_margin_top;
				margin-inline-end: $l_s_margin_end;
				margin-bottom: $l_s_margin_bottom;
				margin-inline-start: $l_s_margin_start;
				border: $l_s_border_width $l_s_border_style $l_s_border_color;
				border-radius: $l_s_radius_top $l_s_radius_right $l_s_radius_bottom $l_s_radius_left;
				box-shadow: $l_s_shadow_x_offset $l_s_shadow_y_offset $l_s_shadow_blur $l_s_shadow_spread $l_s_shadow_color;
			}

		" );
		if ( true ) {
			echo preg_replace( '/\s+/', ' ', "

				div.anrghg_page_thanks {
					display: block;
					width: fit-content;
					margin-top: 0;
					margin-inline-end: $l_s_page_align_end;
					margin-bottom: 0;
					margin-inline-start: $l_s_page_align_start;
					}

				div.anrghg_page_thanks div {
					color: $l_s_page_color;
					font-size: $l_s_page_font_size;
					font-style: $l_s_page_font_style;
					font-weight: $l_s_page_font_weight;
					text-align: $l_s_page_text_align;
					background-color: $l_s_page_background;
					max-width: $l_s_page_max_width;
					padding-top: $l_s_page_padding_top;
					padding-inline-end: $l_s_page_padding_end;
					padding-bottom: $l_s_page_padding_bottom;
					padding-inline-start: $l_s_page_padding_start;
					margin-top: $l_s_page_margin_top;
					margin-inline-end: $l_s_page_margin_end;
					margin-bottom: $l_s_page_margin_bottom;
					margin-inline-start: $l_s_page_margin_start;
					border: $l_s_page_border_width $l_s_page_border_style $l_s_page_border_color;
					border-radius: $l_s_page_radius_top $l_s_page_radius_right $l_s_page_radius_bottom $l_s_page_radius_left;
					box-shadow: $l_s_page_shadow_x_offset $l_s_page_shadow_y_offset $l_s_page_shadow_blur $l_s_page_shadow_spread $l_s_page_shadow_color;
				}

			" );
		}
	}

	/**
	 * Rules for date metadata.
	 *
	 * @since 0.9.0
	 *
	 * @since 0.22.0
	 * Configure styles for posts and pages separately.
	 */
	if ( true ) {
		$l_s_metadata_post_top_text_align    =    'start';
		$l_s_metadata_post_top_margin_top    = -30 . 'px';
		$l_s_metadata_post_top_margin_bottom =  50 . 'px';
		$l_s_metadata_post_top_font_size     =  14 . 'px';
		$l_s_metadata_post_top_text_color    =  '#595959';

		$l_s_metadata_post_end_text_align    =      'end';
		$l_s_metadata_post_end_margin_top    =  50 . 'px';
		$l_s_metadata_post_end_margin_bottom =   0 . 'px';
		$l_s_metadata_post_end_font_size     =  14 . 'px';
		$l_s_metadata_post_end_text_color    =    'black';

		$l_s_metadata_page_top_text_align    =    'start';
		$l_s_metadata_page_top_margin_top    = -30 . 'px';
		$l_s_metadata_page_top_margin_bottom =  50 . 'px';
		$l_s_metadata_page_top_font_size     =  14 . 'px';
		$l_s_metadata_page_top_text_color    =  '#595959';

		$l_s_metadata_page_end_text_align    =      'end';
		$l_s_metadata_page_end_margin_top    =  50 . 'px';
		$l_s_metadata_page_end_margin_bottom =   0 . 'px';
		$l_s_metadata_page_end_font_size     =  14 . 'px';
		$l_s_metadata_page_end_text_color    =    'black';
		echo "\r\n/*date metadata*/";
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_dates_post_top {
				text-align: $l_s_metadata_post_top_text_align;
				margin: $l_s_metadata_post_top_margin_top 0 $l_s_metadata_post_top_margin_bottom;
				font-size: $l_s_metadata_post_top_font_size;
				color: $l_s_metadata_post_top_text_color;
			}

			.anrghg_dates_page_top {
				text-align: $l_s_metadata_page_top_text_align;
				margin: $l_s_metadata_page_top_margin_top 0 $l_s_metadata_page_top_margin_bottom;
				font-size: $l_s_metadata_page_top_font_size;
				color: $l_s_metadata_page_top_text_color;
			}

			.anrghg_dates_post_end {
				text-align: $l_s_metadata_post_end_text_align;
				margin: $l_s_metadata_post_end_margin_top 0 $l_s_metadata_post_end_margin_bottom;
				font-size: $l_s_metadata_post_end_font_size;
				color: $l_s_metadata_post_end_text_color;
			}

			.anrghg_dates_page_end {
				text-align: $l_s_metadata_page_end_text_align;
				margin: $l_s_metadata_page_end_margin_top 0 $l_s_metadata_page_end_margin_bottom;
				font-size: $l_s_metadata_page_end_font_size;
				color: $l_s_metadata_page_end_text_color;
			}

		" );
	}

	/**
	 * Rules for scriptless display toggle.
	 *
	 * @since 0.13.0
	 * Invisible checkbox as adjacent sibling replaces JavaScript class toggle
	 * and AMP classToggle action on tap event, and is actioned by clicking an
	 * unlimited number of labels.
	 * @usedby Purple Number substitutes, heading hyperlinks and table of contents.
	 * @usedby AMP compatible processing of complements as endnotes and references.
	 */
	if ( true ) {
		echo "\r\n/*display toggle*/";
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_display_toggle {
				opacity: 0;
				width: 0;
				height: 0;
				position: absolute;
				left: -999px;
			}

			.anrghg_pointer {
				cursor: pointer;
			}

		" );
	}

	/**
	 * Rules for anchor elements in headings with transitioning multiline underline.
	 *
	 * @since 0.9.0
	 * @contributor** user2046
	 * @link https://stackoverflow.com/a/58895717
	 * @link https://stackoverflow.com/questions/30352431/css-transition-not-working-with-underline
	 */
	if ( apply_filters( 'anrghg_heading_links_active', $g_a_anrghg_config[ 'anrghg_heading_links_active' ] ) ) {
		$l_s_text_color_link              =     'inherit';
		$l_s_text_color_hover             =     'inherit';
		$l_s_text_color_active            =     '#008000';
		$l_s_text_decoration_color_link   = 'transparent';
		$l_s_text_decoration_color_hover  =     'inherit';
		$l_s_text_decoration_color_active = 'transparent';
		$l_s_text_decoration_style_link   =       'solid'; // solid,double,dotted,wavy
		$l_s_text_decoration_style_hover  =       'solid'; // solid,double,dotted,wavy
		$l_s_text_decoration_style_active =       'solid'; // solid,double,dotted,wavy
		$l_s_transition_delay_link        =   3000 . 'ms';
		$l_s_transition_delay_hover       =      0 . 'ms';
		$l_s_transition_delay_active      =    200 . 'ms';
		$l_s_transition_duration_link     =   4000 . 'ms';
		$l_s_transition_duration_hover    =    400 . 'ms';
		$l_s_transition_duration_active   =    300 . 'ms';
		$l_s_transition_function_link     =    'ease-out';
		$l_s_transition_function_hover    =     'ease-in';
		$l_s_transition_function_active   = 'ease-in-out';
		echo "\r\n/*heading anchors*/";
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_heading_link {
				border-bottom: none !important;
				color: $l_s_text_color_link;
				text-decoration-line: underline;
				text-decoration-color: $l_s_text_decoration_color_link;
				text-decoration-style: $l_s_text_decoration_style_link;
				transition: $l_s_transition_duration_link $l_s_transition_function_link $l_s_transition_delay_link;
			}

			.anrghg_heading_link:hover {
				color: $l_s_text_color_hover;
				text-decoration-line: underline;
				text-decoration-color: $l_s_text_decoration_color_hover;
				text-decoration-style: $l_s_text_decoration_style_hover;
				transition: $l_s_transition_duration_hover $l_s_transition_function_hover $l_s_transition_delay_hover;
			}

			.anrghg_heading_link:active {
				color: $l_s_text_color_active;
				text-decoration-line: underline;
				text-decoration-color: $l_s_text_decoration_color_active;
				text-decoration-style: $l_s_text_decoration_style_active;
				transition: $l_s_transition_duration_active $l_s_transition_function_active $l_s_transition_delay_active;
			}

		" );
	}

	/**
	 * Rules for anchor elements in paragraphs.
	 *
	 * @since 0.9.0
	 */
	if ( apply_filters( 'anrghg_paragraph_links_active', $g_a_anrghg_config[ 'anrghg_paragraph_links_active' ] ) ) {
		$l_s_fragment_link_character      = apply_filters( 'anrghg_paragraph_link_character', $g_a_anrghg_config[ 'anrghg_paragraph_link_character' ] );
		$l_s_fragment_link_position       =  -2.7 . 'em';
		$l_s_fragment_link_padding_before =   1.2 . 'em';
		$l_s_fragment_link_padding_after  =    .8 . 'em';
		$l_s_fade_in_delay                =   0   . 'ms';
		$l_s_fade_in_duration             = 500   . 'ms';
		$l_s_fade_out_delay               =   0   . 'ms';
		$l_s_fade_out_duration            = 500   . 'ms';
		$l_s_fade_in_delay                =   0   . 'ms';
		echo "\r\n/*paragraph anchors*/";
		echo preg_replace( '/\s+/', ' ', "

			p,
			li {
				position: relative;
			}

			.anrghg_fragment_link::before {
				content: '$l_s_fragment_link_character';
				position: absolute;
				left: $l_s_fragment_link_position;
				padding: 0 $l_s_fragment_link_padding_after 0 $l_s_fragment_link_padding_before;
				opacity: 0;
				transition: opacity $l_s_fade_out_duration $l_s_fade_out_delay;
			}

			html[dir=rtl] .anrghg_fragment_link::before {
				right: $l_s_fragment_link_position;
				padding: 0 $l_s_fragment_link_padding_before 0 $l_s_fragment_link_padding_after;
			}

			p:hover .anrghg_fragment_link::before,
			li:hover .anrghg_fragment_link::before {
				opacity: 1;
				transition: opacity $l_s_fade_in_duration $l_s_fade_in_delay;
			}

		" );
	}

	/**
	 * Rules for table of contents and heading numbers.
	 *
	 * @since 0.9.0
	 * Mobile breakpoint is 480, or 768 is often used.
	 * @link https://www.geeksforgeeks.org/how-to-target-desktop-tablet-and-mobile-using-media-query/
	 *
	 * @since 0.24.11
	 * Animation of expand/collapse with CSS in sync with twistie rotation.
	 * Debugged keyboard actionability for a11y.
	 *
	 * @since 0.24.14
	 * Debug line height in collapsible table of contents for almost all themes.
	 * Debug line height in collapsible table of contents for themes where the general fix does not work.
	 * Set a (soon optional) collapsed width of the table of contents for use with most themes.
	 *
	 * @since 0.24.15
	 * Debug collapsed height of the table of contents.
	 *
	 * @since 0.29.1
	 * Apply the 0.29.0 writing direction fix to table of contents twistie.
	 * @link https://github.com/mdn/content/pull/8691
	 */
	if ( apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] ) ) {
		$l_s_contents_align            =      '-1'; // -1[start],0[center],1[end]
		switch ( $l_s_contents_align ) {
			case '-1': $l_s_contents_margin_start =    '0'; $l_s_contents_margin_end = 'auto'; break;
			case  '1': $l_s_contents_margin_start = 'auto'; $l_s_contents_margin_end =    '0'; break;
			case  '0':
			default:   $l_s_contents_margin_start = 'auto'; $l_s_contents_margin_end = 'auto';
		}
		$l_s_contents_margin_top       =   20   . 'px';
		$l_s_contents_margin_bottom    =   20   . 'px';
		$l_s_contents_padding_y        =   14   . 'px';
		$l_s_contents_padding_start    =   45   . 'px';
		$l_s_contents_padding_end      =    0   . 'px';
		$l_s_contents_border_width     =    1   . 'px'; // 0 for borderless.
		$l_s_contents_border_style     =       'solid';
		$l_s_contents_border_color     =     '#ABABAB';
		$l_s_collapsed_border          =           '0'; // 0,fix,normal
		$l_s_contents_collapsed_width  =            '';
		if ( $l_s_collapsed_border ) {
			$l_s_contents_border_collapsed =          '';
			if ( 'fix' === $l_s_collapsed_border ) {
				$l_s_contents_collapsed_width = 'width: ' . 340 . 'px;';
			}
		} else {
			$l_s_contents_border_collapsed = 'border-color: ' . $l_s_contents_border_color  . '00;';
		}
		$l_s_label_twistie_char        =           '❰'; // ⟨,❰,❮,‹,«
		$l_s_twistie_color_collapsed   =     'inherit'; // inherit (link color), or #6A6A6A;
		$l_s_twistie_color_expanded    =     '#CBCBCB'; // inherit (link color), or #CBCBCB
		$l_s_label_margin_bottom       =   10   . 'px';
		$l_s_contents_numbering_pad    =   14   . 'px';
		$l_b_top_level_font_weight     = ( bool ) '1'; // 1,0 (bold,normal)
		$l_s_top_level_font_weight     = $l_b_top_level_font_weight ? 'bold' : 'normal';
		$l_b_stepped                   =  ( bool ) '1';
		$l_n_pad_step                  = $l_b_stepped ? 22 : 0; // 12,22
		$l_s_contents_h3_pad           =   $l_n_pad_step       . 'px';
		$l_s_contents_h4_pad           = ( $l_n_pad_step * 2 ) . 'px';
		$l_s_contents_h5_pad           = ( $l_n_pad_step * 3 ) . 'px';
		$l_s_contents_h6_pad           = ( $l_n_pad_step * 4 ) . 'px';
		$l_s_breakpoint_width          =  768   . 'px'; // 480,768
		$l_n_pad_step                  = $l_b_stepped ? 7 : 0; // 4,7
		$l_s_contents_h3_pad_mobile    =   $l_n_pad_step       . 'px';
		$l_s_contents_h4_pad_mobile    = ( $l_n_pad_step * 2 ) . 'px';
		$l_s_contents_h5_pad_mobile    = ( $l_n_pad_step * 3 ) . 'px';
		$l_s_contents_h6_pad_mobile    = ( $l_n_pad_step * 4 ) . 'px';
		$l_s_generic_backlink          =           '⬆';
		$l_s_backlink_pad_scalar       =   22;
		$l_s_backlink_pad_delta        =    9;
		$l_s_backlink_pad_before       =   $l_s_backlink_pad_scalar                             . 'px';
		$l_s_backlink_pad_hover_before = ( $l_s_backlink_pad_scalar - $l_s_backlink_pad_delta ) . 'px';
		$l_s_backlink_pad_hover_after  =                              $l_s_backlink_pad_delta   . 'px';
		$l_f_backlink_opacity          =    0.3;
		$l_s_headings_move_duration    =  500   . 'ms';
		$l_s_contents_move_duration    =  300   . 'ms';
		$l_n_item_font_size_scalar     =   16;
		$l_s_item_font_size            = $l_n_item_font_size_scalar . 'px';
		$l_f_item_line_height          =    1.4;
		$l_n_row_spacing_scalar        =    6;
		$l_s_row_spacing               = $l_n_row_spacing_scalar . 'px';
		$l_f_net_row_height_scalar     = ( $l_n_item_font_size_scalar * $l_f_item_line_height + $l_n_row_spacing_scalar );
		$l_b_calc_item_row_height      = ( bool ) '1';
		if ( $l_b_calc_item_row_height ) {
			$l_s_overall_row_height       = ( $l_f_net_row_height_scalar + $l_n_scroll_offset_scalar ) . 'px';
		} else {
			$l_s_overall_row_height       =  28 . 'px';
		}
		echo "\r\n/*table of contents*/";
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_contents {
				width: fit-content;
				max-width: 100%;
				margin-top: $l_s_contents_margin_top;
				margin-inline-end: $l_s_contents_margin_end;
				margin-bottom: $l_s_contents_margin_bottom;
				margin-inline-start: $l_s_contents_margin_start;
				border: $l_s_contents_border_width $l_s_contents_border_style $l_s_contents_border_color;
				padding-top: $l_s_contents_padding_y;
				padding-inline-end: $l_s_contents_padding_end;
				padding-bottom: $l_s_contents_padding_y;
				padding-inline-start: $l_s_contents_padding_start;
			}

			.anrghg_contents_label {
				width: fit-content;
				margin-bottom: $l_s_label_margin_bottom;
			}

			a.anrghg_contents_anchor .anrghg_contents_label {
				text-decoration: none;
				border-bottom: none;
			}

			.anrghg_contents_label:focus {
				outline: 2px solid #0606066B;
			}

			a.anrghg_contents_anchor .anrghg_contents_label::after {
				display: inline-block;
				content: '$l_s_label_twistie_char';
				padding: 0 20px;
			}

			.anrghg_display_toggle:not(:checked) + .anrghg_contents {
				$l_s_contents_border_collapsed
				$l_s_contents_collapsed_width
				transition: all $l_s_contents_move_duration;
			}

			.anrghg_display_toggle:focus + .anrghg_contents a .anrghg_contents_label,
			.anrghg_display_toggle + .anrghg_contents a:active .anrghg_contents_label {
				border: 2px solid #ABABAB;
			}

			a.anrghg_contents_anchor .anrghg_contents_label::after {
				color: $l_s_twistie_color_collapsed;
				transform: rotate(-.25turn) translateY(-.15em);
				transition: all $l_s_contents_move_duration ease-in-out;
			}

			html[dir=rtl] a.anrghg_contents_anchor .anrghg_contents_label::after {
				transform: rotate(.25turn) translateY(-.15em);
			}

			.anrghg_display_toggle:checked + .anrghg_contents a .anrghg_contents_label::after {
				color: $l_s_twistie_color_expanded;
				transform: rotate(.25turn) translateY(-.15em);
				transition: all $l_s_contents_move_duration ease-in-out;
			}

			html[dir=rtl] .anrghg_display_toggle:checked + .anrghg_contents a .anrghg_contents_label::after {
				transform: rotate(-.25turn) translateY(-.15em);
			}

			.anrghg_display_toggle + .anrghg_contents .anrghg_contents_list .anrghg_contents_heading {
				opacity: 0;
				padding-top: 0;
				margin-top: 0;
				margin-bottom: 0;
				height: 0;
				transition: all $l_s_contents_move_duration ease-in-out;
				white-space: nowrap;
				overflow: hidden;
				visibility: hidden;
			}

			.anrghg_display_toggle:checked + .anrghg_contents .anrghg_contents_list .anrghg_contents_heading {
				opacity: 1;
				padding-top: $l_s_scroll_offset;
				margin-top: -$l_s_scroll_offset;
				margin-bottom: $l_s_row_spacing;
				height: $l_s_overall_row_height;
				transition: all $l_s_contents_move_duration ease-in-out;
				visibility: visible;
			}

			.anrghg_contents_list .anrghg_contents_heading {
				font-size: $l_s_item_font_size;
				line-height: $l_f_item_line_height;
				padding-bottom: 0;
				text-align: end;
			}

			.anrghg_contents_list div:first-line {
				text-align: start;
			}

			.anrghg_inline_spacer {
				display: inline-block;
				width: $l_s_contents_numbering_pad;
			}

			.anrghg_contents_h2 {
				font-weight: $l_s_top_level_font_weight;
			}

			.anrghg_contents_h3 {
				padding: 0 $l_s_contents_h3_pad;
			}

			.anrghg_contents_h4 {
				padding: 0 $l_s_contents_h4_pad;
			}

			.anrghg_contents_h5 {
				padding: 0 $l_s_contents_h5_pad;
			}

			.anrghg_contents_h6 {
				padding: 0 $l_s_contents_h6_pad;
			}

			@media screen and (max-width: $l_s_breakpoint_width) {

				.anrghg_contents {
					padding-inline-end: 0;
					padding-inline-start: 3px;
				}

				.anrghg_display_toggle:focus + .anrghg_contents a .anrghg_contents_label,
				.anrghg_display_toggle + .anrghg_contents a:active .anrghg_contents_label {
					padding-inline-start: 3px;
					margin-inline-start: -3px;
				}

				.anrghg_contents_h3 {
					padding: 0 $l_s_contents_h3_pad_mobile;
				}

				.anrghg_contents_h4 {
					padding: 0 $l_s_contents_h4_pad_mobile;
				}

				.anrghg_contents_h5 {
					padding: 0 $l_s_contents_h5_pad_mobile;
				}

				.anrghg_contents_h6 {
					padding: 0 $l_s_contents_h6_pad_mobile;
				}

			}

			.anrghg_heading_number_after::before {
				content: '$l_s_generic_backlink';
				text-decoration: none !important;
				border-bottom: none !important;
				border-bottom-color: transparent !important;
				padding-inline-start: $l_s_backlink_pad_before;
				opacity: $l_f_backlink_opacity;
				transition: all $l_s_headings_move_duration ease-in-out;
			}

			.anrghg_heading_number_after:hover::before {
				text-decoration: underline;
				padding-inline-start: $l_s_backlink_pad_hover_before;
				padding-inline-end: $l_s_backlink_pad_hover_after;
				opacity: 1;
				transition: all $l_s_headings_move_duration ease-in-out;
			}

			body {counter-reset: h2}
			h2 {counter-reset: h3}
			h3 {counter-reset: h4}
			h4 {counter-reset: h5}
			h5 {counter-reset: h6}

			h2 .anrghg_heading_number_after::before {counter-increment: h2; content: counter(h2) '.'}
			h3 .anrghg_heading_number_after::before {counter-increment: h3; content: counter(h2) '.' counter(h3) '.'}
			h4 .anrghg_heading_number_after::before {counter-increment: h4; content: counter(h2) '.' counter(h3) '.' counter(h4) '.'}
			h5 .anrghg_heading_number_after::before {counter-increment: h5; content: counter(h2) '.' counter(h3) '.' counter(h4) '.' counter(h5) '.'}
			h6 .anrghg_heading_number_after::before {counter-increment: h6; content: counter(h2) '.' counter(h3) '.' counter(h4) '.' counter(h5) '.' counter(h6) '.'}

			.anrghg_contents_list {counter-reset: toc_h2}
			.anrghg_contents_h2 {counter-reset: toc_h3}
			.anrghg_contents_h3 {counter-reset: toc_h4}
			.anrghg_contents_h4 {counter-reset: toc_h5}
			.anrghg_contents_h5 {counter-reset: toc_h6}

			.anrghg_contents_h2 a::before {counter-increment: toc_h2; content: counter(toc_h2) '.'}
			.anrghg_contents_h3 a::before {counter-increment: toc_h3; content: counter(toc_h2) '.' counter(toc_h3) '.'}
			.anrghg_contents_h4 a::before {counter-increment: toc_h4; content: counter(toc_h2) '.' counter(toc_h3) '.' counter(toc_h4) '.'}
			.anrghg_contents_h5 a::before {counter-increment: toc_h5; content: counter(toc_h2) '.' counter(toc_h3) '.' counter(toc_h4) '.' counter(toc_h5) '.'}
			.anrghg_contents_h6 a::before {counter-increment: toc_h6; content: counter(toc_h2) '.' counter(toc_h3) '.' counter(toc_h4) '.' counter(toc_h5) '.' counter(toc_h6) '.'}

		" );
	}

	/**
	 * Rules for complement referrers.
	 */
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] ) ) {
		$l_s_vertical_align          =    'text-top'; // baseline,middle,text-top,super,top
		$l_s_vertical_align          =    0.4 . 'em';
		$l_s_referrer_font_size      =     'smaller'; // smaller,normal
		$l_s_referrer_font_size      =    0.8 . 'em';
		$l_s_referrer_font_weight    =      'normal';
		$l_s_referrer_letter_spacing =      'normal';
		$l_s_referrer_color          =     'inherit'; // Optionally not stand out.
		$l_s_referrer_color          =     '#008000'; // Optional contrast with other links.
		$l_s_referrer_color_rule     =            ''; // For link color, color must be empty i.e. no rule.
		if ( ! true ) {
			$l_s_referrer_color_rule    = "color: $l_s_referrer_color;";
		}
		echo "\r\n/*complement referrers*/";
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_complement_referrer {
				position: relative;
				white-space: nowrap;
			}

			.anrghg_referrer_ref,
			.anrghg_referrer_note,
			.anrghg_referrer_concatenator {
				$l_s_referrer_color_rule
				vertical-align: $l_s_vertical_align;
				font-size: $l_s_referrer_font_size !important;
				font-weight: $l_s_referrer_font_weight;
				letter-spacing: $l_s_referrer_letter_spacing;
				line-height: 0;
				text-decoration: none;
				border-bottom: none;
			}

			span.screenreadertext {
				font-size: 0;
				width: 0;
				height: 0;
				overflow: hidden;
				white-space: nowrap;
				opacity: 0;
			}

		" );
	}

	/**
	 * Rules for complement tooltips.
	 *
	 * Display triggered using `:hover` and `:focus-within` pseudo-classes,
	 * a script-free solution benefitting AMP compatibility.
	 * @contributor** @milindmore22
	 * @link https://github.com/ampproject/amp-wp/issues/5913#issuecomment-785306933
	 * @contributor** @westonruter
	 * @link https://github.com/ampproject/amp-wp/issues/5913#issuecomment-785419655
	 */
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] ) ) {
		$l_s_tooltip_x_property      =        'left'; // left,right
		$l_s_tooltip_y_property      =      'bottom'; // top,bottom
		$l_s_tooltip_offset_x        =  -27   . 'px';
		$l_s_tooltip_offset_y        =   34   . 'px';
		$l_s_breakpoint_width        =  768   . 'px';
		$l_s_tooltip_line_height     =    1.5 . 'em';
		$l_s_tooltip_font_size       =   15   . 'px';
		$l_s_tooltip_color           =     '#000000';
		$l_s_tooltip_background      =     '#FFFFFF';
		$l_s_padding_y               =   10   . 'px';
		$l_s_padding_start           =   20   . 'px';
		$l_s_padding_end             =   18   . 'px';
		$l_s_border_width            =    2   . 'px';
		$l_s_border_style            =       'solid';
		$l_s_border_color            =     '#8FC27A';
		$l_s_border_radius           =    0   . 'px';
		$l_s_shadow_x_offset         =   -2   . 'px';
		$l_s_shadow_y_offset         =    4   . 'px';
		$l_s_shadow_blur             =   11   . 'px';
		$l_s_shadow_spread           =    5   . 'px';
		$l_s_shadow_color            =     '#5B5B5B';
		$l_s_max_width               =  400   . 'px';
		$l_s_fade_in_delay           =    0   . 'ms';
		$l_s_fade_in_duration        =  200   . 'ms';
		$l_s_fade_out_delay          =  500   . 'ms';
		$l_s_fade_out_duration       = 1000   . 'ms';
		echo "\r\n/*complement tooltips*/";
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_complement_tooltip {
				display: block;
				position: absolute;
				$l_s_tooltip_x_property: $l_s_tooltip_offset_x;
				$l_s_tooltip_y_property: $l_s_tooltip_offset_y;
				z-index: 2147483647;
				width: max-content;
				max-width: $l_s_max_width;
				visibility: hidden;
				opacity: 0;
				transition-property: visibility, opacity;
				transition-delay: $l_s_fade_out_delay;
				transition-duration: $l_s_fade_out_duration;
				padding-top: $l_s_padding_y;
				padding-inline-end: $l_s_padding_end;
				padding-bottom: $l_s_padding_y;
				padding-inline-start: $l_s_padding_start;
				line-height: $l_s_tooltip_line_height;
				font-size: $l_s_tooltip_font_size;
				color: $l_s_tooltip_color;
				background-color: $l_s_tooltip_background;
				border: $l_s_border_width $l_s_border_style $l_s_border_color;
				border-radius: $l_s_border_radius;
				box-shadow: $l_s_shadow_x_offset $l_s_shadow_y_offset $l_s_shadow_blur $l_s_shadow_spread $l_s_shadow_color;
				white-space: normal;
			}

			.anrghg_complement_referrer:hover > .anrghg_complement_tooltip,
			.anrghg_complement_referrer:focus-within > .anrghg_complement_tooltip {
				visibility: visible;
				opacity: 1;
				transition-property: visibility, opacity;
				transition-delay: $l_s_fade_in_delay;
				transition-duration: $l_s_fade_in_duration;
			}

			.anrghg_tooltip_more {
				white-space: nowrap;
				font-style: italic;
				text-decoration: none;
				border-bottom: none;
			}

			@media screen and (max-width: $l_s_breakpoint_width) {

				.anrghg_complement_referrer {
					position: static;
				}

				.anrghg_complement_tooltip {
					right: 0;
					left: auto;
					top: auto;
					bottom: auto;
					width: fit-content;
				}

			}

		" );
	}

	/**
	 * Rules for complement lists.
	 *
	 * @since 0.9.0
	 * The CSS `:target` selector is used thanks to @westonruter code contribution.
	 * @contributor** @westonruter
	 * @link https://github.com/markcheret/footnotes/issues/48#issuecomment-799582394
	 * CSS flexbox layout:
	 * @link https://coder-coder.com/display-divs-side-by-side/
	 * Padding between number and backlink symbol matters.
	 * @link https://wordpress.org/support/topic/hyperlink-symbol-in-reference-container/
	 *
	 * @since 0.7.0
	 * Append rotating twisties in pseudo-elements to labels of collapsible lists.
	 * @link https://help.hcltechsw.com/dom_designer/9.0.1/appdev/H_CUSTOMIZING_THE_EXPAND_AND_COLLAPSE_ICONS_OVER.html
	 *
	 * @since 0.28.0
	 * Optional three-column layout (available along with a two-column option).
	 * @reporter** @spaceling
	 * @link https://wordpress.org/support/topic/change-the-position-5/page/2/#post-13671138
	 *
	 * @since 0.29.0
	 * Fix bug in twisties of collapsible lists due to mirrored CSS rotation in right-to-left writing direction.
	 * @see * Rules for table of contents and heading numbers.
	 */
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] ) ) {
		$l_s_list_heading_margin_top =  20   . 'px';
		$l_s_heading_twistie_char    =          '❰'; // ⟨,❰,❮,‹,«
		$l_s_twistie_color_collapsed =    'inherit'; // inherit (link color), or #6A6A6A;
		$l_s_twistie_color_expanded  =    '#CBCBCB'; // inherit (link color), or #CBCBCB
		$l_s_transition_duration     = 300   . 'ms';
		$l_n_list_column_number_note = apply_filters( 'anrghg_note_list_layout', $g_a_anrghg_config[ 'anrghg_note_list_layout' ] );
		$l_n_list_column_number_ref  = apply_filters( 'anrghg_reference_list_layout', $g_a_anrghg_config[ 'anrghg_reference_list_layout' ] );
		$l_s_list_row_pad_bottom     =  20   . 'px';
		$l_s_list_number_symbol      =          '⬆'; // ↑,⇑,⬆
		$l_s_list_symbol_position    =          '1'; // -1[before],0[none],1[after]
		$l_s_list_number_symbol      = $l_s_list_symbol_position ? $l_s_list_number_symbol : '';
		switch ( $l_s_list_symbol_position ) {
			case '-1':
				$l_s_list_symbol_position = 'before';
				break;
			case  '1':
			case  '0':
			default:
				$l_s_list_symbol_position = 'after';
		}
		$l_s_list_number_min_width   =   3.3 . 'em';
		$l_s_list_number_pad_after   =  12   . 'px';
		$l_b_list_number_font_weight = ( bool ) '1'; // 1,0 (bold,normal)
		$l_s_list_number_font_weight = $l_b_list_number_font_weight ? 'bold' : 'normal';
		$l_s_list_symbol_pad_before  =   4   . 'px';
		$l_s_list_symbol_pad_after   =   0   . 'px';
		$l_s_footer_list_pad_top     =  50   . 'px';
		$l_s_footer_list_width       =  80   .  '%';
		$l_s_footer_list_color       =    '#237AF3';
		$l_s_list_end_margin_bottom  =  30   . 'px';
		echo "\r\n/*complement lists*/";
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_complement_heading {
				margin-top: $l_s_list_heading_margin_top;
			}

			a .anrghg_complement_heading {
				text-decoration: none;
				border-bottom: none;
			}

			.anrghg_complement_heading:focus {
				outline: 2px solid #0606066B;
			}

			.anrghg_display_toggle:not(:checked) + .anrghg_complement_list .anrghg_complement_heading:focus {
				outline: 2px solid #060606E5;
			}

			a .anrghg_complement_heading::after {
				display: inline-block;
				content: '$l_s_heading_twistie_char';
				padding: 0 20px;
				color: $l_s_twistie_color_expanded;
				transform: rotate(.25turn) translateY(-.15em);
				transition: all $l_s_transition_duration ease-in-out;
			}

			div[dir=rtl] a .anrghg_complement_heading::after {
				transform: rotate(-.25turn) translateY(-.15em);
			}

			.anrghg_display_toggle:not(:checked) + .anrghg_complement_list a .anrghg_complement_heading::after {
				color: $l_s_twistie_color_collapsed;
				transform: rotate(-.25turn) translateY(-.15em);
				transition: all $l_s_transition_duration ease-in-out;
			}

			.anrghg_display_toggle:not(:checked) + .anrghg_complement_list[dir=rtl] a .anrghg_complement_heading::after {
				transform: rotate(.25turn) translateY(-.15em);
			}

			.anrghg_display_toggle:checked + .anrghg_complement_list.anrghg_notes .anrghg_complement_list_body {
				display: block;
				columns: $l_n_list_column_number_note;
			}

			.anrghg_display_toggle:checked + .anrghg_complement_list.anrghg_refs .anrghg_complement_list_body {
				display: block;
				columns: $l_n_list_column_number_ref;
			}

			.anrghg_display_toggle:not(:checked) + .anrghg_complement_list .anrghg_complement_list_body {
				columns: 1;
			}

			@media screen and (max-width: $l_s_breakpoint_width) {
				.anrghg_display_toggle:not(:checked) + .anrghg_complement_list .anrghg_complement_list_body {
					columns: 1;
				}
			}

			.anrghg_complement_row {
				display: flex;
				padding-top: $l_s_scroll_offset;
				margin-top: -$l_s_scroll_offset;
				padding-bottom: $l_s_list_row_pad_bottom;
			}

			.anrghg_display_toggle:not(:checked) + .anrghg_complement_list .anrghg_complement_row {
				display: none;
			}

			.anrghg_display_toggle:not(:checked) + .anrghg_complement_list .anrghg_complement_row:target {
				display: flex;
			}

			.anrghg_complement_number {
				min-width: $l_s_list_number_min_width;
				padding-inline-end: $l_s_list_number_pad_after;
				font-weight: $l_s_list_number_font_weight;
				text-align: end;
			}

			.anrghg_complement_number::$l_s_list_symbol_position {
				content: '$l_s_list_number_symbol';
				padding-inline-start: $l_s_list_symbol_pad_before;
				padding-inline-end: $l_s_list_symbol_pad_after;
			}

			.anrghg_complement_backlink {
				text-decoration: none;
				border-bottom: none;
			}

			.anrghg_complements_section_spacing {
				margin-bottom: $l_s_list_end_margin_bottom;
			}

			.anrghg_url_wrap {
				word-wrap: anywhere;
				overflow-wrap: anywhere;
				word-break: break-all;
			}

			.anrghg_footer_complement_list {
				width: $l_s_footer_list_width;
				margin: 0 auto 0;
				padding-top: $l_s_footer_list_pad_top;
				text-align: start;
				text-transform: initial;
				z-index: 2147483647;
				color: $l_s_footer_list_color;
			}

			.anrghg_footer_complement_list h2,
			.anrghg_footer_complement_list h3,
			.anrghg_footer_complement_list h4,
			.anrghg_footer_complement_list h5,
			.anrghg_footer_complement_list h6,
			.anrghg_footer_complement_list p,
			.anrghg_footer_complement_list a {
				color: $l_s_footer_list_color;
			}

		" );
	}
	echo "\r\n/*Custom CSS (overrides the above)*/\r\n\r\n";
	echo $g_a_anrghg_config[ 'anrghg_additional_css' ];
	echo "\r\n</style>\r\n";

	/**
	 * Noscript styles.
	 *
	 * @since 0.24.11
	 */
	echo "\r\n<noscript><style";
	if ( $l_b_amp_active ) {
		echo " amp-custom>\r\n";
	} else {
		echo ( current_theme_supports( 'html5', 'style' ) ? '' : ' type="text/css"' ) . " media=\"all\">\r\n";
	}

	/**
	 * For table of contents.
	 */
	if ( apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] ) ) {
		echo preg_replace( '/\s+/', ' ', "

			.anrghg_display_toggle:not(:checked) + .anrghg_contents .anrghg_contents_list .anrghg_contents_heading:target {
				opacity: 1;
				padding-top: $l_s_scroll_offset;
				margin-top: -$l_s_scroll_offset;
				margin-bottom: $l_s_row_spacing;
				height: $l_s_overall_row_height;
				visibility: visible;
			}

		" );
	}
	echo "\r\n</style></noscript>\r\n";
	return;
} // /Outputs internal CSS.
