<?php
/**
 * Administration pages.
 *
 * @since 0.14.1
 * Split off the main file.
 * @package WordPress
 * @subpackage ANRGHG Publishing Helper
 */
defined( 'ABSPATH' ) or exit( nl2br( "\r\n\r\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;This PHP file cannot be displayed in the browser." ) );

/**
 * Adds two dedicated Gutenberg blocks.
 *
 * @since 0.27.0
 * Only on the Admin frontend, no JavaScript added to public pages.
 * Block contents are processed server-side, for AMP compatibility.
 * @link https://weston.ruter.net/2018/12/18/creating-gutenberg-blocks-without-a-build-step-via-htm/
 * @see admin/anrghg-contents-block.js
 * @see admin/anrghg-section-block.js
 *
 * @since 0.30.0
 * Due to technical limitations while using the Node Package Manager (NPM)
 * but not Docker, the newly recommended configuration files bearing names
 * that must be equal to `block.json`, or be the endpoint of a path ending
 * in that string, cannot be processed, compromising internationalization.
 * Therefore, these files are present in the folder as a disclaimer, while
 * the ‘A White Pixel’ block tutorial based on NPM without Docker applies.
 * @link https://awhitepixel.com/blog/wordpress-gutenberg-create-custom-block-tutorial/
 * @link https://make.wordpress.org/core/2021/06/23/block-api-enhancements-in-wordpress-5-8/
 * @link https://developer.wordpress.org/block-editor/handbook/tutorials/create-block/wp-plugin/
 *
 * Tip: Prevent duplicate '-js-js' by not appending '-js' to the handle.
 */
add_filter( 'enqueue_block_editor_assets', function() {
	global $g_a_anrghg_config;
	$l_n_post_id = get_the_id();
	if ( apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] ) ) {
		wp_register_script(
			'anrghg-contents-block',
			plugin_dir_url( __FILE__ ) . 'anrghg-contents-block.min.js',
			array( 'wp-blocks', 'wp-element', 'wp-block-editor' ),
			C_S_ANRGHG_VER,
			false
		);
		wp_add_inline_script(
			'anrghg-contents-block',
			'const anrghgContents = ' . json_encode( array(
				'showEscaPipeAlert'     => ( bool ) $g_a_anrghg_config[ 'anrghg_block_escape_pipe_alert' ],
				'tableOfContentsActive' => $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ],
			) ),
			'before'
		);
		register_block_type( 'anrghg/table-of-contents', array(
			'editor_script' => 'anrghg-contents-block',
		) );
	}
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] ) ) {
		wp_register_script(
			'anrghg-section-block',
			plugin_dir_url( __FILE__ ) . 'anrghg-section-block.min.js',
			array( 'wp-blocks', 'wp-element', 'wp-block-editor' ),
			C_S_ANRGHG_VER,
			false
		);
		$l_s_default_dir = is_rtl() ? '-1' : '1';
		$default_dir_type = $g_a_anrghg_config[ 'anrghg_section_block_default_dir_notes' ];
		switch ( $default_dir_type ) {
			case '-11': $l_s_default_dir_notes = $l_s_default_dir; break;
			case  '-1': $l_s_default_dir_notes = '-1';             break;
			case   '1': $l_s_default_dir_notes =  '1';             break;
			case   '0': $l_s_default_dir_notes =  '0';             break;
		}
		$default_dir_type = $g_a_anrghg_config[ 'anrghg_section_block_default_dir_refs' ];
		switch ( $default_dir_type ) {
			case '-11': $l_s_default_dir_refs  = $l_s_default_dir; break;
			case  '-1': $l_s_default_dir_refs  = '-1';             break;
			case   '1': $l_s_default_dir_refs  =  '1';             break;
			case   '0': $l_s_default_dir_refs  =  '0';             break;
		}
		$l_s_conf = apply_filters( 'anrghg_list_label_notes_plural', $g_a_anrghg_config[ 'anrghg_list_label_notes_plural' ] );
		$l_s_meta = get_post_meta( $l_n_post_id, 'anrghg_list_label_notes', true );
		$l_s_config_list_label_notes = empty( $l_s_meta ) ? $l_s_conf : $l_s_meta;
		$l_s_conf = apply_filters( 'anrghg_list_label_references_plural', $g_a_anrghg_config[ 'anrghg_list_label_references_plural' ] );
		$l_s_meta = get_post_meta( $l_n_post_id, 'anrghg_list_label_refs', true );
		$l_s_config_list_label_refs = empty( $l_s_meta ) ? $l_s_conf : $l_s_meta;
		wp_add_inline_script(
			'anrghg-section-block',
			'const anrghgSection = ' . json_encode( array(
				'defaultDirNotes'      => $l_s_default_dir_notes,
				'defaultDirRefs'       => $l_s_default_dir_refs,
				'configListLabelNotes' => $l_s_config_list_label_notes,
				'configListLabelRefs'  => $l_s_config_list_label_refs,
				'helpListLabelNotes'   => sprintf( __( 'Configured as “%s” — leave empty to keep.', 'anrghg' ), $l_s_config_list_label_notes ),
				'helpListLabelRefs'    => sprintf( __( 'Configured as “%s” — leave empty to keep.', 'anrghg' ), $l_s_config_list_label_refs ),
				'showEscaPipeAlert'    => ( bool ) $g_a_anrghg_config[ 'anrghg_block_escape_pipe_alert' ],
				'sectionBlockSettings' => $g_a_anrghg_config[ 'anrghg_section_block_settings' ],
			) ),
			'before'
		);
		register_block_type( 'anrghg/notes-and-references', array(
			'editor_script' => 'anrghg-section-block',
		) );
	}
} );

/**
 * Adds a meta box in editor inspector sidepane.
 *
 * @since 0.27.4 (provisional)
 * @since 0.29.0
 */
add_filter( 'add_meta_boxes', function() {
	global $g_a_anrghg_config;
	$l_a_options = get_option( 'anrghg' );
	if (
		$g_a_anrghg_config[ 'anrghg_meta_box_active' ]
		&& (
			( apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] )
			&& $g_a_anrghg_config[ 'anrghg_meta_box_contents_insert' ]
			)
			|| ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] )
			&& $g_a_anrghg_config[ 'anrghg_meta_box_complements_dir' ]
			)
			|| ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] )
			&& $g_a_anrghg_config[ 'anrghg_meta_box_list_labels' ]
			)
			|| ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] )
			&& $g_a_anrghg_config[ 'anrghg_meta_box_list_output_footer' ]
			)
		)
	) {
		$l_a_editors = array( 'post' );
		foreach ( $l_a_editors as $l_s_editor ) {
			add_meta_box(
				'anrghg_meta_box',
				_x( 'Complements', 'Meta box title', 'anrghg' ),
				'anrghg_meta_box',
				$l_s_editor,
				'side', // Position: 'side', 'advanced', 'normal'.
				'low'   // Priority: 'high', 'core', 'sorted', 'default', 'low'.
			);
		}
	}
} );

/**
 * Added post meta key names full list.
 *
 * @since 0.29.1
 * @global array $g_a_anrghg_meta_keys
 */
$g_a_anrghg_meta_keys = array(
	'anrghg_insert_contents',
	'anrghg_writing_dir_notes',
	'anrghg_writing_dir_refs' ,
	'anrghg_list_label_notes',
	'anrghg_list_label_refs',
	'anrghg_list_footer_deferral',
);

/**
 * Meta box content.
 *
 * @since 0.29.0
 * These settings override site-wide configuration,
 * and are overridden by block-level configuration.
 *
 * While in Options pages we can use the value '1' for checkboxes,
 * in the editor metaboxes we cannot, or the box will stay checked
 * regardless whether the saved value in the DB is '1' or not. So,
 * the reason for using strings other than '1' and '0' as checkbox
 * and associated hidden input values in checkboxes in meta boxes,
 * be it 'true' and 'false' as in WordPress Core, or '01' and '00'
 * as here, is that even with `autocomplete="off"`, there is a bug
 * in meta box checkboxes with value '1'. Given that this bug does
 * not affect Admin Options pages, plugins do have the opportunity
 * to yield user-friendly settings values optimized for easy input
 * in configuration filters, provided as an alternative to the UI.
 * Note: Zero-padding the values seems like the least-worst option
 * with user-friendliness in mind with respect to eventually using
 * the scheme in publicly visible setups. Another option is to use
 * the (less intuitive) '!0' and '!1' shorthands.
 * @since 0.30.0
 * Use 'true', 'false' and 'null' to fix a bug with radio buttons,
 * and for clarity with respect to website and plugin maintenance.
 */
function anrghg_meta_box( $post ) {
	global $g_a_anrghg_config, $g_a_anrghg_meta_keys;
	wp_nonce_field( basename( __FILE__ ), 'anrghg_nonce' );

	/**
	 * Default values determined after becoming available.
	 */
	$l_s_default_dir = is_rtl() ? 'rtl' : 'ltr';
	$g_a_anrghg_meta_keys[ 'anrghg_insert_contents'      ] = 'null';
	$g_a_anrghg_meta_keys[ 'anrghg_writing_dir_notes'    ] = $l_s_default_dir;
	$g_a_anrghg_meta_keys[ 'anrghg_writing_dir_refs'     ] = $l_s_default_dir;
	$g_a_anrghg_meta_keys[ 'anrghg_list_label_notes'     ] = '';
	$g_a_anrghg_meta_keys[ 'anrghg_list_label_refs'      ] = '';
	$g_a_anrghg_meta_keys[ 'anrghg_list_footer_deferral' ] = 'null';

	/**
	 * Takes in the saved values as the post ID has become available.
	 */
	$l_n_post_id = $post->ID;
	foreach ( $g_a_anrghg_meta_keys as $l_s_key => $l_s_val ) {
		$l_s_registered = get_post_meta( $l_n_post_id, $l_s_key, true );
		$l_a_post_meta[ $l_s_key ] = empty( $l_s_registered ) ? $l_s_val : $l_s_registered;
	}

	/**
	 * Table of contents.
	 *
	 * @since 0.30.0
	 */
	if ( apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] )
		&& $g_a_anrghg_config[ 'anrghg_meta_box_contents_insert' ]
	) {
		$l_s_key0 = 'anrghg_insert_contents';
		echo '<table style="width: 100%"><caption style="text-align: start;"><strong>' . __( 'Table of contents', 'anrghg' ) . '</strong></caption><tbody><tr><td>';
		if ( has_block( 'anrghg/table-of-contents', $l_n_post_id ) ) {
			echo '<label style="cursor: default;"><input type="checkbox" checked="checked" disabled="disabled" style="cursor: default;" />';
			echo __( 'inserted as positioned by block', 'anrghg' ) . '</label></p>';
		} else {
			$l_s_conf = apply_filters( 'anrghg_table_of_contents_active', $g_a_anrghg_config[ 'anrghg_table_of_contents_active' ] );
			switch ( $l_s_conf ) {
				case  '1': $l_s_status = __( 'Insert the table of contents by default if applicable', 'anrghg' );
					break;
				case '-1':
				default:   $l_s_status = __( 'Insert a table of contents if specified accordingly', 'anrghg' );
			}
			// Translators: The placeholder is the currently selected settings label.
			echo '<fieldset><legend>' . sprintf( __( 'Configured as “%s”.', 'anrghg' ), $l_s_status ) . '</legend>';
			echo '<label><input type="radio" id="' . $l_s_key0 . '_0" name="' . $l_s_key0 . '" value="null"'  . checked( $l_a_post_meta[ $l_s_key0 ],  'null', false ) . ' /> ' . __( 'Leave as configured', 'anrghg' ) . '</label><br />';
			echo '<label><input type="radio" id="' . $l_s_key0 . '_1" name="' . $l_s_key0 . '" value="true"'  . checked( $l_a_post_meta[ $l_s_key0 ],  'true', false ) . ' /> ' . __( 'Insert the table of contents', 'anrghg' ) . '</label><br />';
			echo '<label><input type="radio" id="' . $l_s_key0 . '_2" name="' . $l_s_key0 . '" value="false"' . checked( $l_a_post_meta[ $l_s_key0 ], 'false', false ) . ' /> ' . __( 'Do not insert this table', 'anrghg' ) . '</label><br />';
			echo '</fieldset><div style="width: 100%; height: 10px;"></div>';
		}
		echo '</td></tr></tbody></table>';
	}

	/**
	 * Complement lists and referrer tooltips: Writing direction.
	 *
	 * @since 0.29.0
	 * @reporter** @nujuminstitute
	 * @link https://wordpress.org/support/topic/alignment-71/
	 */
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] )
		&& $g_a_anrghg_config[ 'anrghg_meta_box_complements_dir' ]
	) {
		$l_s_key0 = 'anrghg_writing_dir_notes';
		$l_s_key1 = 'anrghg_writing_dir_refs';
		$l_s_val_field0 = 'ltr';
		$l_s_val_field1 = 'rtl';
		$l_s_val_field2 = 'null';
		echo '<table style="width: 100%"><caption style="text-align: start;"><strong>' . __( 'Complement lists and referrer tooltips: Writing direction', 'anrghg' ) . '</strong></caption><tbody>';
		echo '<tr><td><fieldset><div style="display: flex;"><div style="flex-grow: 2;"><legend>' . __( 'Notes:', 'anrghg' ) . '</legend></div><div style="flex-shrink: 1;"><label for="' . $l_s_key0 . '2">' . __( '(Unset)', 'anrghg' ) . '</label></div></div><table style="width: 100%"><tbody><tr>';
		echo '<td><label><input type="radio" id="' . $l_s_key0 . '0" name="' . $l_s_key0 . '" value="' . $l_s_val_field0 . '"' . checked( $l_a_post_meta[ $l_s_key0 ], $l_s_val_field0, false ) . ' /> ' . __( 'Left to right', 'anrghg' ) . '</label></td>';
		echo '<td><label><input type="radio" id="' . $l_s_key0 . '1" name="' . $l_s_key0 . '" value="' . $l_s_val_field1 . '"' . checked( $l_a_post_meta[ $l_s_key0 ], $l_s_val_field1, false ) . ' /> ' . __( 'Right to left', 'anrghg' ) . '</label></td>';
		echo '<td width="10"><input type="radio" id="' . $l_s_key0 . '2" name="' . $l_s_key0 . '" value="' . $l_s_val_field2 . '"' . checked( $l_a_post_meta[ $l_s_key0 ], $l_s_val_field2, false ) . ' /></td>';
		echo '</tr></tbody></table></fieldset></td></tr>';
		echo '<tr><td><fieldset><div style="display: flex;"><div style="flex-grow: 2;"><legend>' . __( 'References:', 'anrghg' ) . '</legend></div><div style="flex-shrink: 1;"><label for="' . $l_s_key1 . '2">' . __( '(Unset)', 'anrghg' ) . '</label></div></div><table style="width: 100%"><tbody><tr>';
		echo '<td><label><input type="radio" id="' . $l_s_key1 . '0" name="' . $l_s_key1 . '" value="' . $l_s_val_field0 . '"' . checked( $l_a_post_meta[ $l_s_key1 ], $l_s_val_field0, false ) . ' /> ' . __( 'Left to right', 'anrghg' ) . '</label></td>';
		echo '<td><label><input type="radio" id="' . $l_s_key1 . '1" name="' . $l_s_key1 . '" value="' . $l_s_val_field1 . '"' . checked( $l_a_post_meta[ $l_s_key1 ], $l_s_val_field1, false ) . ' /> ' . __( 'Right to left', 'anrghg' ) . '</label></td>';
		echo '<td width="10"><input type="radio" id="' . $l_s_key1 . '2" name="' . $l_s_key1 . '" value="' . $l_s_val_field2 . '"' . checked( $l_a_post_meta[ $l_s_key1 ], $l_s_val_field2, false ) . ' /></td>';
		echo '</tr></tbody></table></fieldset></td></tr></tbody></table><div style="width: 100%; height: 7px;"></div>';
	}

	/**
	 * Labels of complement lists.
	 *
	 * @since 0.30.0
	 */
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] )
		&& $g_a_anrghg_config[ 'anrghg_meta_box_list_labels' ]
	) {
		$l_s_key0 = 'anrghg_list_label_notes';
		$l_s_key1 = 'anrghg_list_label_refs';
		echo '<fieldset><legend><strong>' . __( 'Complement list labels', 'anrghg' ) . '</strong></legend><table style="width: 100%"><tbody>';
		echo '<tr><td><label>' . __( 'Notes'     , 'anrghg' ) . ' ' . __( '(leave empty to keep)', 'anrghg' ) . '<br /><input type="text" id="' . $l_s_key0 . '" name="' . $l_s_key0 . '" value="' . esc_attr( $l_a_post_meta[ $l_s_key0 ] ) . '" placeholder="' . apply_filters( 'anrghg_list_label_notes_plural'     , $g_a_anrghg_config[ 'anrghg_list_label_notes_plural'      ] ) . '" style="width: 100%" /></label></td></tr>';
		echo '<tr><td><label>' . __( 'References', 'anrghg' ) . ' ' . __( '(leave empty to keep)', 'anrghg' ) . '<br /><input type="text" id="' . $l_s_key1 . '" name="' . $l_s_key1 . '" value="' . esc_attr( $l_a_post_meta[ $l_s_key1 ] ) . '" placeholder="' . apply_filters( 'anrghg_list_label_references_plural', $g_a_anrghg_config[ 'anrghg_list_label_references_plural' ] ) . '" style="width: 100%" /></label></td></tr>';
		echo '</tbody></table></fieldset><div style="width: 100%; height: 12px;"></div>';
	}

	/**
	 * Footer deferral of complement lists.
	 *
	 * @since 0.30.0
	 */
	if ( apply_filters( 'anrghg_complements_active', $g_a_anrghg_config[ 'anrghg_complements_active' ] )
		&& $g_a_anrghg_config[ 'anrghg_meta_box_list_output_footer' ]
	) {
		$l_s_key0 = 'anrghg_list_footer_deferral';
		$l_s_val_field0 = 'null';
		$l_s_val_field1 = 'true';
		$l_s_val_field2 = 'false';
		echo '<table style="width: 100%"><caption style="text-align: start;"><strong>' . __( 'Complement lists: Footer deferral', 'anrghg' ) . '</strong></caption><tbody><tr><td>';
		$l_s_conf = $g_a_anrghg_config[ 'anrghg_complement_list_footer_deferral' ];
		switch ( $l_s_conf ) {
			case  '1': $l_s_status = __( 'Defer complement lists to the footer', 'anrghg' );
				break;
			case '0':
			default:   $l_s_status = __( 'Keep complement lists in the article', 'anrghg' );
		}
		// Translators: The placeholder is the currently selected settings label.
		echo '<fieldset><legend>' . sprintf( __( 'Configured as “%s”.', 'anrghg' ), $l_s_status ) . '</legend>';
	echo '<label><input type="radio" id="' . $l_s_key0 . '0" name="' . $l_s_key0 . '" value="' . $l_s_val_field0 . '"' . checked( $l_a_post_meta[ $l_s_key0 ], $l_s_val_field0, false ) . ' /> ' . __( 'Leave as configured', 'anrghg' ) . '</label><br />';
		echo '<label><input type="radio" id="' . $l_s_key0 . '1" name="' . $l_s_key0 . '" value="' . $l_s_val_field1 . '"' . checked( $l_a_post_meta[ $l_s_key0 ], $l_s_val_field1, false ) . ' /> ' . __( 'Defer to footer', 'anrghg' ) . '</label><br />';
		echo '<label><input type="radio" id="' . $l_s_key0 . '2" name="' . $l_s_key0 . '" value="' . $l_s_val_field2 . '"' . checked( $l_a_post_meta[ $l_s_key0 ], $l_s_val_field2, false ) . ' /> ' . __( 'Keep in article', 'anrghg' ) . '</label>';
		echo '</fieldset></td></tr></tbody></table>';
	}

} // /Meta box content.

/**
 * Saves the form fields of this meta box.
 */
add_filter( 'save_post', function( $p_n_post_id ) {
	$l_b_autosave    = wp_is_post_autosave( $p_n_post_id );
	$l_b_revision    = wp_is_post_revision( $p_n_post_id );
	$l_b_nonce_valid = ( bool ) ( isset( $_POST[ 'anrghg_nonce' ] ) && wp_verify_nonce( $_POST[ 'anrghg_nonce' ], basename( __FILE__ ) ) );
	if ( $l_b_autosave || $l_b_revision || ! $l_b_nonce_valid ) {
		return;
	}
	global $g_a_anrghg_meta_keys;
	foreach ( $g_a_anrghg_meta_keys as $l_s_key ) {
		if( array_key_exists( $l_s_key, $_POST ) ) {
			update_post_meta(
				$p_n_post_id,
				$l_s_key,
				$_POST[ $l_s_key ]
			);
		}
	}
} );

/**
 * Adds HTTP headers for downloading exported data (provisional).
 *
 * @since 0.8.0
 */
/*
add_filter( 'init', function () {
} );
*/

/**
 * Adds a menu item and/or submenu items to the admin_menu hook.
 *
 * @since 0.5.0
 * @link https://developer.wordpress.org/plugins/settings/custom-settings-page/
 * @return void
 *
 * @since 0.24.6
 * Experimental 3rd admin page to test format conversion.
 */
function anrghg_menu_entries() {
	global $g_a_anrghg_config;
	switch ( $g_a_anrghg_config[ 'anrghg_menu_position' ] ) {
		case 'top': $l_n_item_position =  25; break;
		case 'mid': $l_n_item_position =  80; break;
		case 'low': $l_n_item_position = 100; break;
	}
	if ( 'top' === $g_a_anrghg_config[ 'anrghg_menu_level' ] ) {
		add_menu_page( // Main menu (original template).
			'',
			// Translators: This is the main menu label of the plugin.
			__( 'ANRGHG Helper', 'anrghg' ),
			'manage_options',
			'anrghg-reuse',
			'',
			'dashicons-editor-justify',
			$l_n_item_position
		);
		add_submenu_page(
			'anrghg-reuse',
			// Translators: The placeholder is the plugin name.
			sprintf( __( 'Reusable complements — %s', 'anrghg' ), __( 'ANRGHG Publishing Helper', 'anrghg' ) ),
			// Translators: This is a submenu label.
			__( 'Reusables', 'anrghg' ),
			'manage_options',
			'anrghg-reuse',
			'anrghg_reusables_page'
		);
		add_submenu_page(
			'anrghg-reuse',
			// Translators: The 1st placeholder is the plugin name, the 2nd is the version number string.
			sprintf( __( 'Settings — %s %s', 'anrghg' ), __( 'ANRGHG Publishing Helper', 'anrghg' ), C_S_ANRGHG_VER ),
			// Translators: This is a submenu label.
			__( 'Settings', 'anrghg' ),
			'manage_options',
			'anrghg',
			'anrghg_settings_page'
		);
		if ( apply_filters( 'anrghg_format_conversion_active', $g_a_anrghg_config[ 'anrghg_format_conversion_active' ] ) ) {
			add_submenu_page(
				'anrghg-reuse',
				// Translators: The placeholder is the plugin name.
				sprintf( __( 'Format conversion — %s', 'anrghg' ), __( 'ANRGHG Publishing Helper', 'anrghg' ) ),
				// Translators: This is a submenu label.
				__( 'Conversion', 'anrghg' ),
				'manage_options',
				'anrghg-convert',
				'anrghg_conversion_page'
			);
		}
	}
	if ( 'sub' === $g_a_anrghg_config[ 'anrghg_menu_level' ]
		|| ( 'top' === $g_a_anrghg_config[ 'anrghg_menu_level' ]
		&& ! apply_filters( 'anrghg_menu_remove_duplicate', $g_a_anrghg_config[ 'anrghg_menu_remove_duplicate' ] )
		)
	) {
		add_submenu_page(
			'tools.php',
			// Translators: The placeholder is the plugin name.
			sprintf( __( 'Reusable complements — %s', 'anrghg' ), __( 'ANRGHG Publishing Helper', 'anrghg' ) ),
			// Translators: This is a submenu label.
			__( 'Reusables', 'anrghg' ),
			'manage_options', // Capability.
			'anrghg-reuse', // Slug.
			'anrghg_reusables_page' // Callback.
		);
		add_options_page( // Submenu of Settings.
			// Translators: The 1st placeholder is the plugin name, the 2nd is the version number string.
			sprintf( __( 'Settings — %s %s', 'anrghg' ), __( 'ANRGHG Publishing Helper', 'anrghg' ), C_S_ANRGHG_VER ),
			// Translators: This is a submenu label.
			__( 'Complements', 'anrghg' ),
			'manage_options',
			'anrghg',
			'anrghg_settings_page'
		);
		if ( apply_filters( 'anrghg_format_conversion_active', $g_a_anrghg_config[ 'anrghg_format_conversion_active' ] ) ) {
			add_submenu_page(
				'tools.php',
				// Translators: The placeholder is the plugin name.
				sprintf( __( 'Format conversion — %s', 'anrghg' ), __( 'ANRGHG Publishing Helper', 'anrghg' ) ),
				// Translators: This is a submenu label.
				__( 'Conversion', 'anrghg' ),
				'manage_options',
				'anrghg-convert',
				'anrghg_conversion_page'
			);
		}
	}
	return;
}
add_filter( 'admin_menu', 'anrghg_menu_entries' );

/**
 * ANRGHG conversion page callback function.
 * Options page 3.
 *
 * @since 0.24.6
 * Initially added as a testing playground to try implementing a concept from @erling007.
 * @contributor** @erling007
 * @link https://wordpress.org/support/topic/transforming-older-source-list-to-footnotes-plugin/
 * Functional (java)script may require 1000+ SLOC (is:todo).
 *
 * Conversion tool Word ➔ WordPress with plugin-compatible footnotes (is:todo).
 * @link https://wordpress.org/support/topic/counter-styles-not-working/#post-13765309
 * @link https://wordpress.org/support/topic/counter-styles-not-working/page/2/#post-13771755
 *
 * @return void
 */
function anrghg_conversion_page() {
	global $g_a_anrghg_config, $g_s_anrghg_convert;
	if ( ! current_user_can( 'manage_options' ) ) {
		return;
	}
	if ( 'top' === $g_a_anrghg_config[ 'anrghg_menu_level' ] ) {
		if ( isset( $_GET[ 'settings-updated' ] ) ) {
			add_settings_error(
				'anrghg_convert_messages',
				'anrghg_convert_message',
				__( 'Content saved.', 'anrghg' ),
				'success'
			);
		}
	}
	settings_errors( 'anrghg_convert_messages' );

	/**
	 * Conversion page internal CSS.
	 *
	 * @todo Change $l_s_empty_editor to 'Content' when operational.
	 */
	$l_s_scroll_offset       =  15 . 'vh';
	$l_s_name_font_family    = "sans-serif"; // monospace,sans-serif,serif,'Courier New'
	$l_s_content_font_family = "sans-serif"; // monospace,sans-serif,serif
	$l_n_name_width_value    = 200;
	$l_s_name_field_width    = ( $l_n_name_width_value + 7 ) . 'px';
	$l_s_name_column_width   =   $l_n_name_width_value       . 'px';
	$l_s_empty_editor        = __( 'The conversion script is not functional yet; action is just demo while under construction', 'anrghg' );
	echo "\r\n<style>";
	echo preg_replace( '/\s+/', ' ', "

		h2 {
			margin: 10px 2px;
			font-size: 16px;
		}

		td {
			vertical-align: top;
			color: black;
		}

		td:last-child {
			width: 100%;
		}

		div.edit,
		textarea,
		input[type=text] {
			color: black;
			font-family: $l_s_content_font_family;
			font-size: 15px;
			margin-inline-start: -4px;
		}

		input[type=text] {
			font-family: $l_s_name_font_family;
			width: $l_s_name_field_width;
			margin: 0;
			padding-inline-end: 0;
		}

		div.edit,
		textarea {
			width: 100%;
			resize: vertical;
		}

		textarea {
			height: 80px;
		}

		#input textarea {
			height: 362px;
			margin-bottom: -4px;
			padding: 6px 6px 2px;
		}

		div.edit {
			min-height: 362px;
			max-width: 100%;
			padding: 6px 6px 2px;
			background: white;
			border: 1px solid #7E8993;
			border-radius: 4px;
			box-sizing: border-box;
			resize: vertical;
			overflow: auto;
			line-height: 1.3em;
		}

		div.edit:focus,
		textarea:focus,
		input[type=text]:focus {
			border: 2px solid #09D260;
		}

		div.edit:empty::before {
			content: '$l_s_empty_editor';
			color: #72767D;
		}


		div.submit {
			position: fixed;
			bottom: -8px;
			width: 0;
			margin-top: -30px;
		}

		button.command {
			margin-inline-start: 5px;
			cursor: pointer;
		}

		button.view {
			width: 85px;
			height: 30px;
			border-radius: 0;
			margin: 10px 4px -5px;
			cursor: pointer;
			outline: none;
		}

		div#text {
			display: none;
		}

		#visual_tab {
			background: white;
			border-width: 4px;
		}

		#text_tab {
			background: #E7E7E7;
			border-width: 1px;
		}

		div.bottom_spacer {
			height: 42px;
		}

	" );
	echo "\r\n</style>\r\n";

	/**
	 * Start of form.
	 */
	echo '<div class="wrap"><h1>' . esc_html( get_admin_page_title() ) . '</h1>';
	echo '<form action="options.php" method="post"';
	echo ' onsubmit="return copyEditableDiv()">';
	settings_fields( 'anrghg_convert' ); // Security.

	/**
	 * Form body.
	 */
	do_settings_sections( 'anrghg_convert' );
	echo '<div  class="bottom_spacer"></div>';
	echo '<div class="submit">';
	submit_button( __( 'Save', 'anrghg' ) );
	echo "</div></form></div><!--.wrap-->\r\n";
	return;
}

/**
 * Adds the conversion page.
 *
 * @since 0.24.6
 * @return void
 */
add_filter( 'admin_init', function () {
	$l_s_option_name  = 'anrghg_convert';
	$l_s_option_group = $l_s_option_name;
	register_setting( $l_s_option_group, $l_s_option_name );
	$l_s_section_name = 'anrghg_conversion';
	add_settings_section(
		$l_s_section_name,
		__( 'Make hard-coded complements automation-ready', 'anrghg' ),
		$l_s_section_name . '_cb',
		'anrghg_convert'
	);
	return;
} );

/**
 * Conversion page section callback function.
 *
 * @since 0.24.6
 * @return void
 */
function anrghg_conversion_cb() {
	global $g_a_anrghg_config, $g_s_anrghg_convert;

	/**
	 * Conversion form internal JavaScript.
	 *
	 * Stored in another file (not external though).
	 */
	$l_s_script = file_get_contents( plugin_dir_url( __FILE__ ) . 'js/convert.js' );
	$l_s_script = preg_replace( '/\/\*.+?\*\//', '', $l_s_script );
	$l_s_script = preg_replace( '/\s+/', ' ', $l_s_script );
	echo "\r\n<!--convert--><script>\r\n" . $l_s_script . "\r\n</script>\r\n";

	/**
	 * Delimiters.
	 */
	echo '<div hidden id="delimiter_start">' . $g_a_anrghg_config[ 'anrghg_reference_start' ] . '</div>';
	echo '<div hidden id="delimiter_end">' . $g_a_anrghg_config[ 'anrghg_reference_end' ] . '</div>';

	/**
	 * Conversion editor.
	 */
	echo '<button type="button" class="view" id="visual_tab" onclick="viewVisual()">';
	echo __( 'Visual', 'anrghg' ) . '</button>';
	echo '<button type="button" class="view" id="text_tab" onclick="viewText()">';
	echo __( 'Text', 'anrghg' ) . '</button>';
	echo '<div hidden id="mode">visual</div>';
	echo "\r\n<table class='conversion'><tbody>";
	echo '<tr id="input">';
	echo "<td></td><td>\r\n";
	echo '<div id="visual">';
	echo '<div contenteditable autofocus class="edit" id="content">';
	echo $g_s_anrghg_convert . '</div>';
	echo '<input type="hidden" id="save" />';
	echo '</div><div id="text">';
	echo '<textarea class="edit" id="gethtml" name="anrghg_convert" placeholder="';
	echo __( 'Content', 'anrghg' ) . '">' . $g_s_anrghg_convert . '</textarea>';
	echo '</div></td></tr></tbody></table>';
	echo '<button type="button" class="command" onclick="convert()">';
	echo __( 'Convert', 'anrghg' ) . '</button>';

	/**
	 * Conversion form internal JavaScript.
	 *
	 * Editor scripts.
	 * Toggle views, and manage getting content saved out of the editable div.
	 * (The condition is only to get rid of a pointless console error message.)
	 */
	echo "\r\n<!--editable div--><script>" . preg_replace( '/\s+/', ' ', "

		function viewVisual() {
			if ( document.getElementById('mode').innerHTML === 'text' ) {
				let content = document.getElementById('gethtml').value;
				content = content.replace(/&lt;/g, '<');
				content = content.replace(/&gt;/g, '>');
				content = content.replace(/&amp;/g, '&');
				document.getElementById('content').innerHTML = content;
			}
			document.getElementById('visual').style.display = 'block';
			document.getElementById('text').style.display = 'none';
			document.getElementById('mode').textContent = 'visual';
			document.getElementById('visual_tab').style.background = 'white';
			document.getElementById('visual_tab').style.borderWidth = '4px';
			document.getElementById('text_tab').style.background = '#E7E7E7';
			document.getElementById('text_tab').style.borderWidth = '1px';
		}

		function viewText() {
			if ( document.getElementById('mode').textContent === 'visual' ) {
				document.getElementById('gethtml').value = document.getElementById('content').innerHTML;
			}
			document.getElementById('visual').style.display = 'none';
			document.getElementById('text').style.display = 'block';
			document.getElementById('mode').textContent = 'text';
			document.getElementById('visual_tab').style.background = '#E7E7E7';
			document.getElementById('visual_tab').style.borderWidth = '1px';
			document.getElementById('text_tab').style.background = 'white';
			document.getElementById('text_tab').style.borderWidth = '4px';
		}

		function copyEditableDiv() {
			if ( document.getElementById('content') !== null ) {
				let mode = document.getElementById('mode').textContent;
				content = '';
				if ( mode = 'visual' ) {
					viewText();
					viewVisual();
					content = document.getElementById('content').innerHTML;
				} else {
					content = document.getElementById('gethtml').value;
				}
				content = content.replace(/&lt;/g, '<');
				content = content.replace(/&gt;/g, '>');
				content = content.replace(/&amp;/g, '&');
				document.getElementById('save').value = content;
				document.getElementById('gethtml').removeAttribute('name');
				document.getElementById('save').setAttribute('name', 'anrghg_convert');
			}
			return true;
		}

	" ) . "\r\n</script>";
	return;
}

/**
 * ANRGHG reusables page callback function.
 * Options page 1.
 *
 * @since 0.5.0
 * @return void
 */
function anrghg_reusables_page() {
	global $g_a_anrghg_config, $g_a_anrghg_reuse;
	if ( ! current_user_can( 'manage_options' ) ) {
		return;
	}
	if ( 'top' === $g_a_anrghg_config[ 'anrghg_menu_level' ] ) {
		if ( isset( $_GET[ 'settings-updated' ] ) ) {
			add_settings_error(
				'anrghg_reuse_messages',
				'anrghg_reuse_message',
				__( 'Your Complements List has been updated.', 'anrghg' ),
				'success'
			);
		}
	}
	settings_errors( 'anrghg_reuse_messages' );

	/**
	 * Reusable complements form internal CSS.
	 *
	 * @since 0.5.0
	 */
	echo "\r\n<style>";
	$l_s_scroll_offset       =  15 . 'vh';
	$l_s_name_font_family    = "sans-serif"; // monospace,sans-serif,serif,'Courier New'
	$l_s_content_font_family = "sans-serif"; // monospace,sans-serif,serif
	$l_n_name_width_value    = 200;
	$l_s_name_field_width    = ( $l_n_name_width_value + 7 ) . 'px';
	$l_s_name_column_width   =   $l_n_name_width_value       . 'px';
	$l_s_empty_editor        = __( 'Content', 'anrghg' );
	echo preg_replace( '/\s+/', ' ', "

		div.json {
			width: 97%;
			height: 87px;
			overflow: scroll;
			padding: 0 10px;
			margin: 3px 1px;
			background: white;
			border: 2px dashed #00B798;
			font-family: monospace;
			white-space: pre;
			user-select: all;
			resize: both;
		}

		h2 {
			margin: 10px 2px;
			font-size: 16px;
		}

		.offset.base {
			position: relative;
		}

		.offset .anchor {
			position: absolute;
			bottom: $l_s_scroll_offset;
		}

		td {
			vertical-align: top;
			color: black;
		}

		td:last-child {
			width: 100%;
		}

		div.edit,
		textarea,
		input[type=text] {
			color: black;
			font-family: $l_s_content_font_family;
			font-size: 15px;
		}

		input[type=text] {
			font-family: $l_s_name_font_family;
			width: $l_s_name_field_width;
			margin: 0;
			padding-inline-end: 0;
		}

		div.edit,
		textarea {
			width: 100%;
			resize: vertical;
		}

		textarea {
			height: 80px;
		}

		textarea#editor {
			height: 273px;
		}

		#new textarea {
			height: 220px;
		}

		div.edit {
			min-height: 62px;
			max-width: 100%;
			padding: 6px 6px 2px;
			background: white;
			border: 1px solid #7E8993;
			border-radius: 4px;
			box-sizing: border-box;
			resize: vertical;
			overflow: auto;
			line-height: 1.3em;
		}

		div.edit:focus,
		textarea:focus,
		input[type=text]:focus {
			border: 2px solid #09D260;
		}

		div.edit:empty::before {
			content: '$l_s_empty_editor';
			color: #72767D;
		}

		#use {
			height: 280px;
			width: $l_s_name_column_width;
			overflow: auto;
			resize: vertical;
			padding-left: 5px;
			border: 1px solid #5E97E9;
			line-height: 1.1em;
		}

		#caption {
			margin-top: 5px;
			color: #F24400;
			font-size: 88%;
			font-style: italic;
			white-space: nowrap;
		}

		#init,
		#len {
			visibility: collapse;
			height: 0;
		}

		button.edit {
			cursor: pointer;
		}

		div.submit {
			position: fixed;
			bottom: -8px;
			width: 0;
			margin-top: -30px;
		}

		div.bottom_spacer {
			height: 42px;
		}

	" );
	echo "\r\n</style>\r\n";

	/**
	 * Start of form.
	 */
	echo '<div class="wrap"><h1>' . esc_html( get_admin_page_title() ) . '</h1>';
	echo '<form action="options.php" method="post"';
	if ( true ) {
		echo ' onsubmit="return copyEditableDivs()"';
	}
	echo '>';
	settings_fields( 'anrghg_reuse' ); // Security.

	/**
	 * Backup files.
	 *
	 * @since 0.5.0
	 */
	$l_a_convert = array();
	foreach ( $g_a_anrghg_reuse[ 'key' ] as $l_n_index => $l_s_name ) {
		if ( ! empty( $l_s_name ) || ! empty( $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] ) ) {
			$l_a_convert[] = array( $l_s_name => $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] );
		}
	}
	$l_s_json = json_encode( $l_a_convert, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT );
	$l_s_uploads_path = wp_upload_dir();
	if ( ! is_dir( $l_s_uploads_path[ 'basedir' ] . '/anrghg' ) ) {
		mkdir( $l_s_uploads_path[ 'basedir' ] . '/anrghg' );
	}
	$l_o_json = fopen( $l_s_uploads_path[ 'basedir' ] . '/anrghg/anrghg-reusable-complements_latest.json', 'w' );
	fwrite( $l_o_json, $l_s_json );
	$l_s_scope = __( 'Last revision:', 'anrghg' );
	$l_s_stamp = '';
	if ( $g_a_anrghg_config[ 'anrghg_keep_reusables_history' ] ) {
		$l_s_datetime = current_time( 'Y-m-d\TH:i:sO' );
		$l_o_json = fopen( $l_s_uploads_path[ 'basedir' ] . "/anrghg/anrghg-reusable-complements_$l_s_datetime.json", 'w' );
		fwrite( $l_o_json, $l_s_json );
		$l_s_scope = __( 'Full revision history:', 'anrghg' );
		$l_s_stamp = '_[datetime]';
	}
	echo '<div>' . __( 'Your data is stored in the DB and backed up in JSON as in this frame. ', 'anrghg' );
	echo $l_s_scope . ' ';
	echo trailingslashit( wp_basename( content_url() ) ) . wp_basename( $l_s_uploads_path[ 'basedir' ] );
	echo "/anrghg/anrghg-reusable-complements$l_s_stamp.json.</div>";
	echo '<div class="json">' . esc_attr( $l_s_json ) . '</div><div id="scroll"></div>';

	/**
	 * Form body.
	 */
	do_settings_sections( 'anrghg_reuse' );
	echo '<div  class="bottom_spacer"></div>';
	echo '<div class="submit">';
	submit_button( __( 'Save Data', 'anrghg' ) );
	echo "</div></form></div><!--.wrap-->\r\n";
	return;
}

/**
 * Adds the reusable complements page.
 *
 * @since 0.5.0
 * @return void
 */
add_filter( 'admin_init', function () {
	$l_s_option_name  = 'anrghg_reuse';
	$l_s_option_group = $l_s_option_name;
	register_setting( $l_s_option_group, $l_s_option_name );
	$l_s_section_name = 'anrghg_reusables';
	add_settings_section(
		$l_s_section_name,
		__( 'Form to add and edit reusable complements', 'anrghg' ),
		$l_s_section_name . '_cb',
		'anrghg_reuse'
	);
	return;
} );

/**
 * Reusable complements list section callback function.
 *
 * @since 0.5.0
 * @return void
 * The duplicate names check warns about no more than a single pair of duplicates
 * at a time, consistently with the real-world case scenario, not overloading the
 * edge cases with undisambiguated duplicate fragment identifiers.
 */
function anrghg_reusables_cb() {
	global $g_a_anrghg_reuse;

	/**
	 * Empty, duplicate, disintegrated name warnings.
	 *
	 * @since 0.5.0
	 */
	$l_b_warned  = false;
	foreach ( $g_a_anrghg_reuse[ 'key' ] as $l_n_index => $l_s_name ) {
		if ( empty( $l_s_name ) && ! empty( $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] ) && ! $l_b_warned ) {
			echo "\r\n<div class='noname'><p><strong><em>";
			echo __( 'Missing name:', 'anrghg' );
			echo "</em></strong> ";
			// Translators: The placeholders are the tags of the anchor element linking to a list item.
			echo sprintf( __( '%sThis%s complement has no name.', 'anrghg' ), "<a href='#noname'>", '</a>' ) . '</p>';
			echo '<p>' . __( 'Please give it a name.', 'anrghg' ) . '</p></div>';
			$l_b_warned = true;
		}
		if ( preg_match( '/\s/', $l_s_name ) && ! $l_b_warned ) {
			echo "\r\n<div class='problem'><p><strong><em>";
			echo __( 'Name cannot be used:', 'anrghg' );
			echo "</em></strong> ";
			// Translators: The placeholders are the tags of the anchor element linking to a list item.
			echo sprintf( __( '%sThis%s name has space in it.', 'anrghg' ), "<a href='#problem'>", '</a>' ) . '</p>';
			echo '<p>' . __( 'Please replace this, e.g. with underscore.', 'anrghg' ) . '</p></div>';
			$l_b_warned  = true;
		}
	}
	$l_a_warn = array();
	foreach ( $g_a_anrghg_reuse[ 'key' ] as $l_n_index => $l_s_name ) {
		$l_n_lookup = $l_n_index;
		do {
			$l_n_lookup++;
			if ( ! isset( $g_a_anrghg_reuse[ 'key' ][ $l_n_lookup ] ) || empty( $l_s_name ) || $l_b_warned ) {
				break;
			}
			if ( $l_s_name === $g_a_anrghg_reuse[ 'key' ][ $l_n_lookup ] ) {
				$l_a_warn[] = $l_s_name;
				if ( $l_a_warn[ 0 ] === $l_s_name ) {
					echo "\r\n<div class='warn'><p><strong><em>";
					echo __( 'Warning duplicate name:', 'anrghg' );
					echo "</em></strong><span class='warn'> $l_s_name </span>";
					// Translators: The placeholders are the tags of the anchor element linking to a list item.
					echo sprintf( __( 'already used %shere%s.', 'anrghg' ), "<a href='#2$l_s_name'>", '</a>' ) . '</p>';
					echo '<p>' . __( 'Please change the new instance, to keep your previous content displayed.', 'anrghg' ) . '</p></div>';
					$l_b_warned = true;
				}
			}
		} while ( true && ! $l_b_warned );
	}
	if ( $l_b_warned ) {
		echo "\r\n<style>";
		echo preg_replace( '/\s+/', ' ', "

			td.warn input[type=text] {
				font-weight: bold;
				color: #BE0022;
				background: #EDC800;
			}

			div.noname,
			div.warn,
			div.problem {
				border: 3px solid #A52435;
				padding: 5% 10%;
				margin: 5px;
			}

			div.noname {
				background: #B1A2FF;
			}

			div.warn {
				background: #C4FF00;
			}

			div.problem {
				background: #C0166E;
				color: white;
			}

			div.problem a {
				color: #7CC2EE;
			}

			div.noname p,
			div.warn p,
			div.problem p {
				font-size: 20px;
			}

			span.warn {
				font-weight: bold;
				font-family: monospace;
				font-size: 200%;
			}

		" );
		echo "\r\n</style>\r\n";
	}

	/**
	 * Reusable complements table.
	 */
	echo "\r\n<table class='complements'><tbody>";

	/**
	 * New complement row.
	 */
	echo '<tr id="new"><td><input type="text" id="name" name="anrghg_reuse[key][]" value=""';
	echo ' placeholder="' . __( 'Name', 'anrghg' ) . '" autocomplete="off"';
	if ( ! $l_b_warned ) {
		echo ' autofocus';
	}
	echo ' /><div id="use" tabindex="-1"><div id="caption">';
	echo __( 'Names in use, alphabetically ordered:', 'anrghg' ) . '</div>';
	foreach ( $g_a_anrghg_reuse[ 'key' ] as $l_s_name ) {
		if ( ! empty( $l_s_name ) ) {
			$l_a_names[] = $l_s_name;
		}
	}
	if ( ! empty( $l_a_names ) ) {
		sort( $l_a_names );
		echo implode( '<br />', $l_a_names );
	}
	echo "</div></td><td>\r\n";
	if ( true ) {

		/**
		 * Swaps default behavior in TinyMCE to insert `<br />` tags on Enter.
		 *
		 * @link https://shellcreeper.com/wordpress-editor-tinymce-how-to-make-it-use-br-break-on-enter-instead-of-creating-p-paragraph/
		 * @link https://shellcreeper.com/?p=1041
		 * @link http://codex.wordpress.org/Plugin_API/Filter_Reference/tiny_mce_before_init
		 * @link http://www.tinymce.com/wiki.php/Configuration:forced_root_block
		 */
		add_filter( 'tiny_mce_before_init', function ( $l_a_settings ) {
			$l_a_settings[ 'forced_root_block' ] = false;
			return $l_a_settings;
		});

		/**
		 * Integrates Tiny Moxiecode Content Editor.
		 *
		 * @since 0.19.0
		 *
		 * @since 0.20.1.1
		 * Advise to install the TinyMCE Color Picker plugin to enable fore- and background color formatting.
		 * @link https://wordpress.org/plugins/tinymce-colorpicker/
		 *
		 * @since 0.20.2
		 * Switch off reliance on wpautop to get TinyMCE insert `<br />` tags on Shift+Enter.
		 * @link https://stackoverflow.com/questions/27508157/p-p-missing-with-wp-editor-get-option
		 * Restrict toolbar to functionalities working with typical inline complements.
		 * @link https://wordpress.stackexchange.com/questions/141708/tinymce-4-x-how-to-customize-toolbar-on-wp-editor
		 * @link https://www.tiny.cloud/blog/custom-font-sizes-in-tinymce/
		 * @see wp-includes/class-wp-editor.php:632..686
		 *
		 * @since 0.22.0
		 * Super- and subscript, underline formatting buttons in the toolbar.
		 * Fore- and background color in the toolbar optional.
		 * Warn about installing TinyMCE Color Picker plugin.
		 * @link https://securityboulevard.com/2019/04/how-to-protect-wordpress-websites-from-sql-injection/
		 */
		$l_a_args = array(
			'textarea_name' => 'anrghg_reuse[val][]',
			'wpautop'       => false,
			'tinymce'       => array(
				'toolbar1' => 'link,bold,italic,superscript,subscript,underline,strikethrough',
				'toolbar2' => '',
				'toolbar3' => '',
			),
		);
		if ( true ) {
			$l_a_args[ 'tinymce' ][ 'toolbar1' ] .= ',forecolor,backcolor';
		}
		$l_a_args[ 'tinymce' ][ 'toolbar1' ] .= ',wpemoji,charmap,fontsizeselect,selectall,copy,cut,paste,pastetext,removeformat,undo,redo';
		if ( ! wp_is_mobile() ) {
			$l_a_args[ 'tinymce' ][ 'toolbar1' ] .= ',fullscreen,wp_help';
		}
		wp_editor( '', 'editor', $l_a_args );
		echo "\r\n<!--edit buttons--><script>" . preg_replace( '/\s+/', ' ', "

			function moveIntoEditor(index) {
				var editor  = tinymce.get('editor');
				if ( editor.getContent() === '' && document.getElementById('name').value === '' ) {
					document.getElementById('name').value =
					document.getElementById('name' + index).value;
					document.getElementById('name' + index).value = '';
					var content = document.getElementById('content' + index).innerHTML;
					content = content.replace(/&lt;/g, '<');
					content = content.replace(/&gt;/g, '>');
					editor.setContent(content);
					document.getElementById('content' + index).innerHTML = '';
					document.getElementById('scroll').scrollIntoView();
				}
			}

		" ) . "\r\n</script>\r\n";
	} else {
		echo '<textarea name="anrghg_reuse[val][]" placeholder="';
		echo __( 'Content', 'anrghg' );
		echo '"></textarea>';
	}
	echo '</td></tr>';

	/**
	 * Existing complement rows.
	 *
	 * @since 0.5.0
	 */
	$l_n_number = 0;
	$l_n_warn   = 0;
	$l_b_init   = true;
	$l_b_fix    = false;
	foreach ( $g_a_anrghg_reuse[ 'key' ] as $l_n_index => $l_s_name ) {
		if ( ! empty( $l_s_name ) || ! empty( $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] ) ) {
			echo "\r\n<tr><td";
			if ( empty( $l_s_name ) ) {
				echo ' class="warn"';
				if ( $l_n_warn < 1 ) {
					$l_n_warn = 1;
					$l_b_fix = true;
					echo "><span class='offset base'><span class='anchor' id='noname'></span></span";
				}
			}
			if ( preg_match( '/\s/', $l_s_name ) ) {
				echo ' class="warn"';
				if ( $l_n_warn < 1 ) {
					$l_n_warn = 1;
					$l_b_fix = true;
					echo "><span class='offset base'><span class='anchor' id='problem'></span></span";
				}
			}
			if ( isset ( $l_a_warn ) ) {
				if ( in_array( $l_s_name, $l_a_warn ) && $l_n_warn < 2 ) {
					$l_n_warn++;
					$l_b_fix = true;
					echo " class='warn'><span class='offset base'><span class='anchor' id='$l_n_warn$l_s_name'></span></span";
				}
			}
			echo '><input type="text" id="name' . $l_n_index . '" name="anrghg_reuse[key][]" value="';
			echo $l_s_name . '" placeholder="' . __( 'Name', 'anrghg' ) . '" autocomplete="off"';
			if ( $l_b_fix ) {
				echo ' autofocus';
				$l_b_fix = false;
			}
			echo ' /><button type="button" class="edit"';
			if ( true ) {
				echo ' tabindex="-1"';
			}
			echo ' onclick="moveIntoEditor(\'' . $l_n_index . '\')">';
			echo __( 'Move into editor', 'anrghg' ) . '</button></td><td>';
			if ( true ) {
				if ( $l_b_init ) {
					echo '<div id="init">' . $l_n_index . '</div>';
					echo '<div id="len">' . count( $g_a_anrghg_reuse[ 'key' ], COUNT_RECURSIVE ) . '</div>';
					$l_b_init = false;
				}
				echo '<div contenteditable class="edit" id="content' . $l_n_index;
				echo '">' . $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] . '</div>';
				echo '<input type="hidden" id="save' . $l_n_index . '" />';
				echo '<textarea class="content" id="fallback' . $l_n_index;
				echo '" name="anrghg_reuse[val][]" placeholder="' . __( 'Content', 'anrghg' ) . '">';
				echo $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] . '</textarea>';
			} else {
				echo '<textarea id="content' . $l_n_index;
				echo '" name="anrghg_reuse[val][]" placeholder="' . __( 'Content', 'anrghg' ) . '">';
				echo $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] . '</textarea>';
			}
			echo "</td></tr>\r\n";
			$l_n_number++;
		}
	}
	echo '</tbody></table>';

	/**
	 * Adapt page to JavaScript disabled status.
	 *
	 * @since 0.20.0
	 */
	echo "\r\n<noscript><style>";
	echo preg_replace( '/\s+/', ' ', "

		button.edit,
		div.edit {
		  display: none;
		}

		textarea#editor {
		  height: 374px;
		  border: 1px solid #7E8993;
		  border-radius: 4px;
		  font-size: 15px;
		}

		#use {
		  height: 340px;
		}

	" );
	echo "</style></noscript>\r\n";

	/**
	 * Reusable complements form internal JavaScript.
	 *
	 * @since 0.20.2
	 * Manages getting content saved out of editable divs.
	 * The condition is only to get rid of a pointless console error message.
	 */
	if ( true ) {
		echo "\r\n<!--editable divs--><script>" . preg_replace( '/\s+/', ' ', "

			window.onload = function() {
				var areas = document.getElementsByClassName('content');
				for ( var i = 0; i < areas.length; i++ ) {
					areas[i].style.display = 'none';
				}
			};

			function copyEditableDivs() {
				const init = document.getElementById('init').innerHTML;
				const len  = document.getElementById('len').innerHTML;
				for ( let index = init; index < ( init + len - 1 ); index++ ) {
					if ( document.getElementById('content' + index) !== null ) {
						let content = document.getElementById('content' + index).innerHTML;
		" );
		if ( true ) {
			echo preg_replace( '/\s+/', ' ', "
							content = content.replace(/&lt;/g, '<');
							content = content.replace(/&gt;/g, '>');
			" );
		}
		echo preg_replace( '/\s+/', ' ', "
						document.getElementById('save' + index).value = content;
						document.getElementById('fallback' + index).removeAttribute('name');
						document.getElementById('save' + index).setAttribute('name', 'anrghg_reuse[val][]');
					}
				}
				return true;
			}

		" ) . "\r\n</script>\r\n";
	}

	/**
	 * Displays current number of reusable complements.
	 *
	 * @since 0.5.0
	 */
	echo '<p class="number">';
	if ( 0 === $l_n_number ) {
		echo __( 'No reusable complements yet.', 'anrghg' );
	} else {
		echo sprintf( _n( '%d reusable complement.', '%d reusable complements.', $l_n_number, 'anrghg' ), $l_n_number );
	}
	echo "</p>\r\n";
	return;
}

/**
 * ANRGHG settings page callback function.
 * Options page 2.
 *
 * @since 0.5.0
 * When in Settings submenu, 'Settings saved.' is also displayed. Let it be.
 *
 * @since 0.24.9
 * Fix duplicate message displayed on saving options pages when they are in the Settings submenu.
 * Then the confirmation message is not customizable, since the default message is loaded anyway.
 * @see wp-admin/options.php:333
 *
 * @todo Check that no option appears as if it were hidden.
 * @reporter** @fondanalys
 * @link https://wordpress.org/support/topic/error-in-required-structured-data-element-48/#post-14729350
 *
 * @todo Check all wordings whether they can be misunderstood.
 * @reporter** @solu78
 * @link https://wordpress.org/support/topic/center-alignment-available/#post-14731445
 *
 * @return void
 */
function anrghg_settings_page() {
	global $g_a_anrghg_config;
	if ( ! current_user_can( 'manage_options' ) ) {
		return;
	}
	if ( 'top' === $g_a_anrghg_config[ 'anrghg_menu_level' ] ) {
		if ( isset( $_GET[ 'settings-updated' ] ) ) {
			add_settings_error(
				'anrghg_messages',
				'anrghg_message',
				__( 'Your Configuration has been saved.', 'anrghg' ),
				'success'
			);
		}
	}
	settings_errors( 'anrghg_messages' );

	/**
	 * Settings form internal CSS.
	 *
	 * @since 0.5.0
	 *
	 * @since 0.27.4
	 * Invert glide switch appearance to make it consistent with the radio buttons.
	 * Tune On and Off colors, for consistency with a Material Design glide switch.
	 * Optimize On/Off color scheme for a11y:
	 * On:  #09D260 ➔ #007F37 ➔ #005D29
	 * Off: #E00300 ➔ #FF6C6A ➔ #FF8C8A
	 */
	$l_s_state_label = __( 'Effect of the current state:', 'anrghg' );
	// Translators: The placeholder is the 'Save Settings' button label text.
	$l_s_state_alert = sprintf( __( 'Effective after clicking %s.', 'anrghg' ), _x( 'Save Settings', 'Settings submit button label', 'anrghg' ) );
	echo "\r\n<style>";
	echo preg_replace( '/\s+/', ' ', "

		:root {
			--on: #005D29;
			--off: #FF8C8A;
			--label_on: #00400E;
			--label_off: #7F0200;
			--active: #4CFFD0;
		}

		div.wrap {
			color: black;
		}

		div.toc {
			font-size: 18px;
		}

		.toc li {
			margin: 0;
		}

		.toc li a {
			padding: 12px 5px 14px;
			color: #003960;
			text-decoration: none;
			font-weight: bold;
			display: block;
		}

		.toc li:hover {
			background: #8BFFA3;
		}

		.toc li a:hover {
			color: #000360;
		}

		.toc li a:active {
			color: #117A3F;
		}

		.toc li.subsection {
			margin: 0 30px;
		}

		.backtotop {
			position: fixed;
			z-index: 100;
			top: 60px;
			right: 30px;
			font-size: 14px;
			padding: 8px 12px 10px;
			background: white;
			color: #00400E;
			border: 1px solid #B7B7B7;
			border-radius: 5px;
			text-decoration: none;
		}

		html[dir=rtl] .backtotop {
			right: unset;
			left: 30px;
		}

		.backtotop:hover {
			background: #00400E;
			color: white;
		}

		div.submit {
			position: fixed;
			bottom: -8px;
			width: 0;
			margin-top: -30px;
		}

		div.preview {
			width: 97%;
		}

		div.json {
			max-width: 500px;
			height: 87px;
			overflow: scroll;
			padding: 0 10px;
			margin: 3px 1px;
			background: white;
			border: 2px dashed #00B798;
			font-family: monospace;
			white-space: pre-wrap;
			user-select: all;
			resize: both;
		}

		legend.screenreadertext {
			position: absolute;
			width: 0px;
			height: 0px;
			overflow: hidden;
		}

		.form-table td fieldset label {
			line-height: 1.3;
			margin: 0 !important;/*overrides !important prop in wp-admin/load-styles.php?c=1&dir=ltr&load%5Bchunk_0%5D=dashicons,admin-bar,common,forms,admin-menu,dashboard,list-tables,edit,revisions,media,themes,about,nav-menus,wp-pointer,widgets&load%5Bchunk_1%5D=,site-icon,l10n,buttons,wp-auth-check&ver=5.8:15*/
		}

		code {
			font-weight: bold;
			color: #00400E;
		}

		code.label {
			font-weight: unset;
			color: unset;
		}

		h2 {
			border-top: 3px solid #006EBF;
			padding-top: 10px;
			font-size: 25px;
			line-height: 1em;
			margin-bottom: -8px;
			z-index: 99;
		}

		div.subsections h2 {
			border-top: 2px solid #006EBF;
			padding-top: 10px;
			font-size: 21px;
		}

		tr {
			border-top: 1px solid #006EBF;
		}

		table.form-table td {
			padding-top: 8px !important;
		}

		p.anrghg {
			font-size: 20px;
		}

		div.subsections p.anrghg {
			font-size: 18px;
		}

		.base {
			position: relative;
			vertical-align: top;
		}

		.base .id {
			position: absolute;
			bottom: 66px;
		}

		.base .link {
			display: block;
			position: absolute;
			bottom: -20px;
			height: 46px;
			width: 100%;
			z-index: 0;
		}

		.base .link:hover,
		.base .link:focus {
			background: linear-gradient(to right, #8BFFA300, #8BFFA300, #8BFFA366, #8BFFA3FF, #8BFFA3FF);
		}

		html[dir=rtl] .base .link:hover,
		html[dir=rtl] .base .link:focus {
			background: linear-gradient(to left, #8BFFA300, #8BFFA300, #8BFFA366, #8BFFA3FF, #8BFFA3FF);
		}

		table td p.description {
			font-size: 14px;
			font-style: italic;
			color: black;
			margin-bottom: 3px;
		}

		table.form-table {
			margin-bottom: -24px;
		}

		table.form-table td {
			padding: 15px 0;
		}

		div.aux {
			margin: 12px 30px;
		}

		div.flex {
			display: flex;
		}

		textarea,
		input[type=text] {
			width: 80%;
			margin-top: 5px;
			font-family: monospace;
			color: black;
			font-size: 1.3em;
		}

		input[type=text].small {
			width: 30%;
		}

		input[type=text].narrow {
			width: 70px;
		}

		input[type=number] {
			width: 200px;
			max-width: 100%;
		}

		input[type=number].font {
			width: 100px;
		}

		input[type=number].tiny {
			width: 50px;
		}

		textarea {
			width: 80%;
			max-width: 100%;
			overflow: auto;
			resize: both;
		}
		textarea.medium {
			height: 157px;
		}

		textarea#anrghg_additional_css {
			height: 800px;
			width: 100%;
		}

		textarea:focus,
		input[type=text]:focus,
		input[type=number]:focus {
			border: 3px solid #09D260;
		}

		.form-table input[type=radio] {
			display: block;
			z-index: 1;
			margin: 12px 0 11px 0;
			margin-inline-end: -32px;
		}

		.form-table input[type=radio].box {
			margin-top: 18px;
		}

		.form-table input[type=radio]+label {
			display: block;
			z-index: 0;
			width: 100%;
			padding: 13px 42px;
		}

		.form-table input[type=radio]+label:hover,
		.form-table input[type=radio]:hover+label,
		.form-table input[type=radio]:focus+label {
			background: var(--active);
		}

		.form-table input[type=radio]:checked::after {
			content: '';
			display: inline-block;
			position: relative;
			top: -18px;
			left: -.5px;
			width: 19px;
			height: 19px;
			border-radius: 50%;
			background-color: var(--on);
		}

		html[dir=rtl] .form-table input[type=radio]:checked::after {
			right: -.5px;
		}

		.form-table input[type=radio].no:checked::after {
			background-color: var(--off);
		}

		.form-table input[type=radio]:checked+label,
		.form-table input[type=radio]:checked+label input {
			font-weight: bold;
			color: var(--label_on);
		}

		.form-table input[type=radio].no:checked+label {
			color: var(--label_off);
		}

		p+input {
			margin-top: 5px;
		}

		#dates_scope label {
			display: block;
			padding: 15px;
		}

		#dates_scope label:hover,
		#dates_scope label:focus-within {
			background: var(--active);
		}

		input.delims {
			width: 200px;
			margin: 0 34px;
		}

		table.text tr td:last-child {
			width: 90%;
		}

		input.switch {
			opacity: 0;
			position: absolute;
			left: -999px;
		}

		input.switch + label div.flex {
			margin-bottom: 10px;
		}

		input.switch + label:hover div.flex,
		input.switch:focus + label div.flex {
			background: var(--active);
		}

		div.switch {
			position: relative;
			height: 60px;
			width: 50px;
			margin-inline-start: -12px;
			padding-top: 15px;
		}

		div.slot {
			position: absolute;
			top: 18px;
			left: 29px;
			height: 40px;
			width: 10px;
			border-radius: 5px;
			background: #B5B5B5;
		}

		html[dir=rtl] div.slot {
			right: 29px;
		}

		div.knob {
			position: absolute;
			top: 37px;
			left: 18px;
			height: 30px;
			width: 30px;
			border: 1px solid #E4E4E4;/*not #7B8594*/
			border-radius: 50%;
			box-shadow: -5px 4px 5px #BFBFBF;
			background-color: var(--off);
			transition: all .2s;
		}

		html[dir=rtl] div.knob {
			right: 18px;
			box-shadow: 5px 4px;
		}

		input.switch:checked + label div.knob {
			top: 7px;
			background-color: var(--on);
			transition: all .2s;
		}

		input.switch:focus + label div.knob {
			border: 1px solid #1C55C3;
		}

		div.knob::before {
			position: absolute;
			top: 4px;
			left: 7.5px;
			content: '⭘';
			font-size: 15px;
			color: white;
		}

		html[dir=rtl] div.knob::before {
			right: 7.5px;
		}

		input.switch:checked + label div.knob::before {
			left: 12.5px;
			content: 'I';
			font-size: 18px;
		}

		html[dir=rtl] input.switch:checked + label div.knob::before {
			right: 12.5px;
		}

		div.labels {
			width: 100%;
			padding-inline-start: 16px;
			padding-top: 15px;
		}

		div.labels div {
			height: 30px;
			width: 100%;
			vertical-align: center;
		}

		input.switch:checked + label .label1 {
			font-weight: bold;
			color: var(--label_on);
		}

		.state1,
		.state0 {
			border-bottom-width: 3px;
			border-bottom-style: solid;
		}

		.state1 {
			border-bottom-color: var(--label_on);
		}

		.state0 {
			border-bottom-color: var(--label_off);
		}

		.state1::before,
		.state0::before {
			content: '$l_s_state_label ';
			font-weight: normal;
			font-style: italic;
		}

		input.switch + label .state1 {
			display: none;
		}

		input.switch:checked + label .state1 {
			display: block;
			font-weight: bold;
			color: var(--label_on);
		}

		input.switch + label .state0 {
			font-weight: bold;
			color: var(--label_off);
		}

		input.switch:checked + label .state0 {
			display: none;
		}

		span.apply::after {
			content: '$l_s_state_alert';
			padding-inline-start: 26px;
			font-style: italic;
			color: black;
		}

		input.switch + label span.apply,
		input.switch:checked + label span.apply.checked {
			opacity: 0;
		}

		input.switch + label span.apply.checked,
		input.switch:checked + label span.apply {
			opacity: .8;
		}

		div.bottom_spacer {
			height: 42px;
		}

		@media (max-width: 768px) {

			.backtotop {
				top: 14px;
			}

			.form-table input[type=radio]:checked::after {
				top: -29px;
				left: 0px;
				width: 29px;
				height: 29px;
			}

			html[dir=rtl] .form-table input[type=radio]:checked::after {
				right: 0px;
			}

		}

	" );
	if ( ! $g_a_anrghg_config[ 'anrghg_dates_label_uni' ] ) {
		echo preg_replace( '/\s+/', ' ', "

			table#unified_labels {
				visibility: collapse;
			}

		" );
	}
	if ( $g_a_anrghg_config[ 'anrghg_dates_label_uni' ] ) {
		echo preg_replace( '/\s+/', ' ', "

			tr.anrghg_dates_label_post_top_modif,
			tr.anrghg_dates_label_page_top_modif,
			tr.anrghg_dates_label_post_top_publi,
			tr.anrghg_dates_label_page_top_publi,
			tr.anrghg_dates_label_post_end_modif,
			tr.anrghg_dates_label_page_end_modif,
			tr.anrghg_dates_label_post_end_publi,
			tr.anrghg_dates_label_page_end_publi {
				visibility: collapse;
			}

		" );
	} else {
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_post_top_modif' ] ) { echo 'tr.anrghg_dates_label_post_top_modif {visibility: collapse;}'; }
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_page_top_modif' ] ) { echo 'tr.anrghg_dates_label_page_top_modif {visibility: collapse;}'; }
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_post_top_publi' ] ) { echo 'tr.anrghg_dates_label_post_top_publi {visibility: collapse;}'; }
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_page_top_publi' ] ) { echo 'tr.anrghg_dates_label_page_top_publi {visibility: collapse;}'; }
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_post_end_modif' ] ) { echo 'tr.anrghg_dates_label_post_end_modif {visibility: collapse;}'; }
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_page_end_modif' ] ) { echo 'tr.anrghg_dates_label_page_end_modif {visibility: collapse;}'; }
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_post_end_publi' ] ) { echo 'tr.anrghg_dates_label_post_end_publi {visibility: collapse;}'; }
		if ( '0' === $g_a_anrghg_config[ 'anrghg_dates_page_end_publi' ] ) { echo 'tr.anrghg_dates_label_page_end_publi {visibility: collapse;}'; }
	}
	echo "</style>\r\n";

	/**
	 * Start of form.
	 */
	echo '<div class="wrap"><h1>' . esc_html( get_admin_page_title() );
	echo '</h1><form action="options.php" method="post">';
	settings_fields( 'anrghg' ); // Security.
	echo '<a tabindex="-1" href="#"><div class="backtotop">' . __( 'Back To Top', 'anrghg' ) . '</div></a>';

	/**
	 * Table of contents.
	 *
	 * Use class="subsection" to indent while keeping numbering continuous.
	 */
	echo '<div class="toc"><ol>';
	echo '<li class="section"><a href="#general">'           . __( 'General options'                        , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#security">'          . __( 'Security'                               , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#thank_you">'         . __( 'Append a message to posts or pages'     , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#dates">'             . __( 'Display last modified date'             , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#date_meta_tags">'    . __( 'Add date meta tags in the page head'    , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#paragraph_links">'   . __( 'Links to paragraphs and list items'     , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#table_of_contents">' . __( 'Table of contents and heading links'    , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#complements">'       . __( 'Complements as end notes and references', 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#slug_body_class">'   . __( 'Use the URL slug as a CSS selector'     , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#wptexturize">'       . __( 'Option to deactivate WPTexturize'       , 'anrghg' ) . '</a></li>';
	echo '<li class="section"><a href="#css">'               . __( 'Additional cascading style sheet'       , 'anrghg' ) . '</a></li>';
	echo '</ol></div>';

	/**
	 * Form body.
	 */
	do_settings_sections( 'anrghg' );
	echo '<div class="bottom_spacer"></div>';
	echo '<div class="submit">';
	submit_button( _x( 'Save Settings', 'Settings submit button label', 'anrghg' ) );
	echo '</div></form></div><!--.wrap-->';

	/**
	 * Settings form internal JavaScript.
	 *
	 * The argument of querySelectorAll() must be free from tabs and newlines.
	 */
	echo "\r\n<script>" . preg_replace( '/\s+/', ' ', "

		var elements = document.querySelectorAll('.anrghg_dates_label_post_top_modif, .anrghg_dates_label_page_top_modif, .anrghg_dates_label_post_top_publi, .anrghg_dates_label_page_top_publi, .anrghg_dates_label_post_end_modif, .anrghg_dates_label_page_end_modif, .anrghg_dates_label_post_end_publi, .anrghg_dates_label_page_end_publi');
		var checkboxes = document.querySelectorAll('#anrghg_dates_post_top_modif, #anrghg_dates_page_top_modif, #anrghg_dates_post_top_publi, #anrghg_dates_page_top_publi, #anrghg_dates_post_end_modif, #anrghg_dates_page_end_modif, #anrghg_dates_post_end_publi, #anrghg_dates_page_end_publi');
		var radiobutton = document.getElementById('anrghg_dates_label_uni');

		radiobutton.onchange = function() {
			document.getElementById('unified_labels').style.visibility = 'visible';
			for ( var i = 0; i < 8; i++ ) {
				elements[i].style.visibility = 'collapse';
			}
		};

		document.getElementById('anrghg_dates_label_uni1').onchange = function() {
			document.getElementById('unified_labels').style.visibility = 'collapse';
			for ( var i = 0; i < 8; i++ ) {
				if ( checkboxes[i].checked ) {
					elements[i].style.visibility = 'visible';
				} else {
					elements[i].style.visibility = 'collapse';
				}
			}
		};

		document.getElementById('dates_scope').onclick = function() {
			if ( ! radiobutton.checked ) {
				for ( var i = 0; i < 8; i++ ) {
					if ( checkboxes[i].checked ) {
						elements[i].style.visibility = 'visible';
					} else {
						elements[i].style.visibility = 'collapse';
					}
				}
			}
		};

	" ) . "\r\n</script>\r\n";
	return;
}

/**
 * Settings sections.
 *
 * @since 0.5.0
 * The order in this function determines the order of settings in the dashboard.
 * Delete ` 'label_for' => $l_s_field_name,` if the label of a particular field
 * should not activate the field. WordPress does disable the label element and
 * the for attribute when the field is a radio button group. By contrast, in this
 * plugin, clicking the field label selects the first radio button of the group,
 * because this behavior is more intuitive and straightforward and streamlines UX.
 * Don’t use `esc_html_e()` as it breaks the code (and criminalizes translators!).
 * Setting default values in register_setting() did not work; instead they are now
 * used as fallbacks when defining the constants used throughout so the plugin can
 * work out-of-the-box as usual.
 * The $option_group (1st) argument of register_setting() must be equal to the 2nd,
 * $option_name: “an easy solution is to make $option_group match $option_name.”
 * @link https://wordpress.stackexchange.com/a/140071
 * @link https://wordpress.stackexchange.com/questions/139660/error-options-page-not-found-on-settings-page-submission-for-an-oop-plugin
 * @link https://core.trac.wordpress.org/ticket/7277
 * And it must also be equal to the page slug defined in add_{menu|options}_page().
 * @link https://wordpress.stackexchange.com/a/316063
 * @return void
 */
function anrghg_settings_sections() {
	$l_s_option_name  = 'anrghg';
	$l_s_option_group = $l_s_option_name;
	register_setting( $l_s_option_group, $l_s_option_name );
	global $g_a_anrghg_config;
	$l_s_class_name   = 'main';

	/**
	 * General options.
	 *
	 */
	$l_s_label        = __( 'General options', 'anrghg' );
	$l_s_section_name = 'anrghg_general';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Back up reusable complements:', 'anrghg' );
	$l_s_field_name   = 'anrghg_keep_reusables_history';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Back up the settings:', 'anrghg' );
	$l_s_field_name   = 'anrghg_keep_settings_history';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Meta box:', 'anrghg' );
	$l_s_field_name   = 'anrghg_meta_box_active';
	$l_s_key_1        = 'anrghg_meta_box_contents_insert';
	$l_s_key_2        = 'anrghg_meta_box_complements_dir';
	$l_s_key_3        = 'anrghg_meta_box_list_labels';
	$l_s_key_4        = 'anrghg_meta_box_list_output_footer';
	$l_s_key_5        = '';
	$l_s_key_6        = '';
	$l_s_key_7        = '';
	$l_s_key_8        = '';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ],
		'key_2'     => $l_s_key_2,
		'val_2'     => $g_a_anrghg_config[ $l_s_key_2 ],
		'key_3'     => $l_s_key_3,
		'val_3'     => $g_a_anrghg_config[ $l_s_key_3 ],
		'key_4'     => $l_s_key_4,
		'val_4'     => $g_a_anrghg_config[ $l_s_key_4 ],
//		'key_5'     => $l_s_key_5,
//		'val_5'     => $g_a_anrghg_config[ $l_s_key_5 ],
//		'key_6'     => $l_s_key_6,
//		'val_6'     => $g_a_anrghg_config[ $l_s_key_6 ],
//		'key_7'     => $l_s_key_7,
//		'val_7'     => $g_a_anrghg_config[ $l_s_key_7 ],
//		'key_8'     => $l_s_key_8,
//		'val_8'     => $g_a_anrghg_config[ $l_s_key_8 ],
	) );

	$l_s_label        = __( 'Menu level:', 'anrghg' );
	$l_s_field_name   = 'anrghg_menu_level';
	$l_s_key_1        = 'anrghg_menu_remove_duplicate';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ]
	) );

	$l_s_label        = __( 'Menu position:', 'anrghg' );
	$l_s_field_name   = 'anrghg_menu_position';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Security.
	 */
	$l_s_label        = __( 'Security', 'anrghg' );
	$l_s_section_name = 'anrghg_security';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Login page profile:', 'anrghg' );
	$l_s_field_name   = 'anrghg_login_deactivation_profile';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Edit auth cookie lifespan:', 'anrghg' );
	$l_s_field_name   = 'anrghg_auth_duration_edit';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Lifespan duration in days:', 'anrghg' );
	$l_s_field_name   = 'anrghg_auth_expiration_days';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Append a message to posts or pages.
	 */
	$l_s_label        = __( 'Append a message to posts or pages', 'anrghg' );
	$l_s_section_name = 'anrghg_thank_you';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Activate and scope:', 'anrghg' );
	$l_s_field_name   = 'anrghg_thank_you_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Content for posts:', 'anrghg' );
	$l_s_field_name   = 'anrghg_thank_you_content';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Content for pages:', 'anrghg' );
	$l_s_field_name   = 'anrghg_thank_you_content_page';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Priority level:', 'anrghg' );
	$l_s_field_name   = 'anrghg_thank_you_priority_select';
	$l_s_key_1        = 'anrghg_thank_you_priority_input';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ]
	) );

	/**
	 * Display last modified date.
	 */
	$l_s_label        = __( 'Display last modified date', 'anrghg' );
	$l_s_section_name = 'anrghg_dates';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Activate and scope:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_active';
	$l_s_key_1        = 'anrghg_dates_post_top_modif';
	$l_s_key_2        = 'anrghg_dates_page_top_modif';
	$l_s_key_3        = 'anrghg_dates_post_end_modif';
	$l_s_key_4        = 'anrghg_dates_page_end_modif';
	$l_s_key_5        = 'anrghg_dates_post_top_publi';
	$l_s_key_6        = 'anrghg_dates_page_top_publi';
	$l_s_key_7        = 'anrghg_dates_post_end_publi';
	$l_s_key_8        = 'anrghg_dates_page_end_publi';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ],
		'key_2'     => $l_s_key_2,
		'val_2'     => $g_a_anrghg_config[ $l_s_key_2 ],
		'key_3'     => $l_s_key_3,
		'val_3'     => $g_a_anrghg_config[ $l_s_key_3 ],
		'key_4'     => $l_s_key_4,
		'val_4'     => $g_a_anrghg_config[ $l_s_key_4 ],
		'key_5'     => $l_s_key_5,
		'val_5'     => $g_a_anrghg_config[ $l_s_key_5 ],
		'key_6'     => $l_s_key_6,
		'val_6'     => $g_a_anrghg_config[ $l_s_key_6 ],
		'key_7'     => $l_s_key_7,
		'val_7'     => $g_a_anrghg_config[ $l_s_key_7 ],
		'key_8'     => $l_s_key_8,
		'val_8'     => $g_a_anrghg_config[ $l_s_key_8 ],
	) );

	$l_s_label        = __( 'Ordering of the dates:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_chronological';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Date label configuration:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_uni';
	$l_s_key_1        = 'anrghg_dates_label_modified';
	$l_s_key_2        = 'anrghg_dates_label_published';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ],
		'key_2'     => $l_s_key_2,
		'val_2'     => $g_a_anrghg_config[ $l_s_key_2 ],
	) );

	$l_s_label        = __( 'Label of the modified date at the top of a post:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_post_top_modif';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label of the modified date at the top of a page:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_page_top_modif';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label of the published date at the top of a post:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_post_top_publi';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label of the published date at the top of a page:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_page_top_publi';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label of the modified date at the end of a post:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_post_end_modif';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label of the modified date at the end of a page:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_page_end_modif';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label of the published date at the end of a post:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_post_end_publi';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label of the published date at the end of a page:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_label_page_end_publi';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_class_name   = 'main';
	$l_s_label        = __( 'Priority level:', 'anrghg' );
	$l_s_field_name   = 'anrghg_dates_priority_select';
	$l_s_key_1        = 'anrghg_dates_priority_input';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ]
	) );

	/**
	 * Add date meta tags in the page head.
	 */
	$l_s_label        = __( 'Add date meta tags in the page head', 'anrghg' );
	$l_s_section_name = 'anrghg_date_meta_tags';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Activate:', 'anrghg' );
	$l_s_field_name   = 'anrghg_date_meta_tags_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Format:', 'anrghg' );
	$l_s_field_name   = 'anrghg_date_meta_tags_format';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Links to paragraphs and list items.
	 */
	$l_s_label        = __( 'Links to paragraphs and list items', 'anrghg' );
	$l_s_section_name = 'anrghg_paragraph_links';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Activation:', 'anrghg' );
	$l_s_field_name   = 'anrghg_paragraph_links_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Character:', 'anrghg' );
	$l_s_field_name   = 'anrghg_paragraph_link_character';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Table of contents and heading links.
	 */
	$l_s_label        = __( 'Table of contents and heading links', 'anrghg' );
	$l_s_section_name = 'anrghg_table_of_contents';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Heading links:', 'anrghg' );
	$l_s_field_name   = 'anrghg_heading_links_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Table of contents:', 'anrghg' );
	$l_s_field_name   = 'anrghg_table_of_contents_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Minimum number of headings:', 'anrghg' );
	$l_s_field_name   = 'anrghg_table_of_contents_min_number';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Label:', 'anrghg' );
	$l_s_field_name   = 'anrghg_table_of_contents_label';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Table of contents depth:', 'anrghg' );
	$l_s_field_name   = 'anrghg_table_of_contents_depth';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Default position of the table:', 'anrghg' );
	$l_s_field_name   = 'anrghg_table_of_contents_position';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Code to manually position the table:', 'anrghg' );
	$l_s_field_name   = 'anrghg_table_of_contents_positioner';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Collapsible and collapsed states:', 'anrghg' );
	$l_s_field_name   = 'anrghg_table_of_contents_collapse';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Complements as end notes and references.
	 */
	$l_s_label        = __( 'Complements as end notes and references', 'anrghg' );
	$l_s_section_name = 'anrghg_complements';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Activate:', 'anrghg' );
	$l_s_field_name   = 'anrghg_complements_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Note delimiters:', 'anrghg' );
	$l_s_field_name   = 'anrghg_note_delimiter_set';
	$l_s_key_1        = 'anrghg_free_note_start';
	$l_s_key_2        = 'anrghg_free_note_end';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ],
		'key_2'     => $l_s_key_2,
		'val_2'     => $g_a_anrghg_config[ $l_s_key_2 ],
	) );

	$l_s_label        = __( 'Reference delimiters:', 'anrghg' );
	$l_s_field_name   = 'anrghg_ref_delimiter_set';
	$l_s_key_1        = 'anrghg_free_ref_start';
	$l_s_key_2        = 'anrghg_free_ref_end';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
		'key_1'     => $l_s_key_1,
		'val_1'     => $g_a_anrghg_config[ $l_s_key_1 ],
		'key_2'     => $l_s_key_2,
		'val_2'     => $g_a_anrghg_config[ $l_s_key_2 ],
	) );

	$l_s_label        = __( 'Note list layout options:', 'anrghg' );
	$l_s_field_name   = 'anrghg_note_list_layout';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Reference list layout options:', 'anrghg' );
	$l_s_field_name   = 'anrghg_reference_list_layout';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Footer deferral of complement lists:', 'anrghg' );
	$l_s_field_name   = 'anrghg_complement_list_footer_deferral';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
	//	'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Automatic positioning in the footer:', 'anrghg' );
	$l_s_field_name   = 'anrghg_complement_list_output_buffering';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Display link URL as visible text in note lists:', 'anrghg' );
	$l_s_field_name   = 'anrghg_display_urls_note_list';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	$l_s_label        = __( 'Display link URL as visible text in reference lists:', 'anrghg' );
	$l_s_field_name   = 'anrghg_display_urls_reference_list';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Option to deactivate WPTexturize.
	 */
	$l_s_label        = __( 'Option to deactivate WPTexturize', 'anrghg' );
	$l_s_section_name = 'anrghg_wptexturize';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'State:', 'anrghg' );
	$l_s_field_name   = 'anrghg_wptexturize_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Use the URL slug as a CSS selector.
	 */
	$l_s_label        = __( 'Use the URL slug as a CSS selector', 'anrghg' );
	$l_s_section_name = 'anrghg_slug_body_class';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Activate:', 'anrghg' );
	$l_s_field_name   = 'anrghg_slug_body_class_active';
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label'     => $l_s_label,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	/**
	 * Additional cascading style sheet
	 */
	$l_s_label        = __( 'Additional cascading style sheet', 'anrghg' );
	$l_s_section_name = 'anrghg_css';
	add_settings_section( $l_s_section_name, $l_s_label, $l_s_section_name . '_cb', 'anrghg' );

	$l_s_label        = __( 'Custom CSS style rules:', 'anrghg' ); // Selectors recommended for customization:
	$l_s_field_name   = 'anrghg_additional_css';
	$l_s_class_name   = $l_s_field_name;
	add_settings_field( $l_s_field_name, $l_s_label, $l_s_field_name . '_cb', 'anrghg', $l_s_section_name, array(
		'class'     => $l_s_class_name,
		'label_for' => $l_s_field_name,
		'key'       => $l_s_field_name,
		'val'       => $g_a_anrghg_config[ $l_s_field_name ],
	) );

	return;
}
add_filter( 'admin_init', 'anrghg_settings_sections' );

/**
 * Callback functions of settings sections and fields.
 *
 * The values are not escaped throughout given that most of them are plain ASCII.
 * The `$p_a_params[ 'val' ]` value is escaped only in text/textarea field values.
 * The `$p_a_params[ 'label_for' ]` value is used only to configure the main label.
 *
 * Void is a valid return type and should be mentioned.
 * @link https://www.php.net/manual/en/migration71.new-features.php
 * @link https://stackoverflow.com/questions/29792827/void-as-return-type
 * @link https://stackoverflow.com/questions/2061550/phpdoc-return-void-necessary
 */

/**
 * General settings section callback function.
 *
 * @since 0.24.7
 * @param array $p_a_params
 * @return void
 */
function anrghg_general_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'In this section, you may deactivate the storage of backup files with a datestamp in their name, configure the meta box, or opt in to add a dedicated item in the Admin main menu.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Keep reusables history field callback function.
 *
 * @since 0.24.7
 *
 * @since 0.24.8
 * Checkbox-based vertical glide switch.
 *
 * @since 0.24.10
 * Display reusables backup file.
 * @global array $g_a_anrghg_config
 * @global array $g_a_anrghg_reuse
 * @param array  $p_a_params
 * @return void
 */
function anrghg_keep_reusables_history_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Keep a date-stamped backup file whenever accessing or saving the Reusables page.', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'The backup history is stored.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'The backup history is lost.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';

	/**
	 * Display of backup.
	 */
	global $g_a_anrghg_config, $g_a_anrghg_reuse;
	$l_a_convert = array();
	foreach ( $g_a_anrghg_reuse[ 'key' ] as $l_n_index => $l_s_name ) {
		if ( ! empty( $l_s_name ) || ! empty( $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] ) ) {
			$l_a_convert[] = array( $l_s_name => $g_a_anrghg_reuse[ 'val' ][ $l_n_index ] );
		}
	}
	$l_s_json = json_encode( $l_a_convert, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT );
	if ( $g_a_anrghg_config[ 'anrghg_keep_reusables_history' ] ) {
		$l_s_scope = __( 'Full revision history:', 'anrghg' );
		$l_s_stamp = '_[datetime]';
	} else {
		$l_s_scope = __( 'Last revision:', 'anrghg' );
		$l_s_stamp = '';
	}
	$l_s_uploads_path = wp_upload_dir();
	echo '<div class="preview">' . __( 'Your data is stored in the DB and backed up in JSON as in this frame. ', 'anrghg' );
	echo $l_s_scope . ' ';
	echo trailingslashit( wp_basename( content_url() ) ) . wp_basename( $l_s_uploads_path[ 'basedir' ] );
	echo "/anrghg/anrghg-reusable-complements$l_s_stamp.json.";
	echo '<div class="json"><pre>' . esc_attr( $l_s_json ) . "</pre></div></div>\r\n";

	return;
}

/**
 * Keep settings history field callback function.
 *
 * @since 0.24.7
 *
 * @since 0.24.8
 * Checkbox-based vertical glide switch.
 * Move settings backup here.
 * @global array $g_a_anrghg_config
 * @param array  $p_a_params
 * @return void
 */
function anrghg_keep_settings_history_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Keep a date-stamped backup of your configuration whenever accessing or saving this page.', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'The backup history is stored.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'The backup history is lost.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';

	/**
	 * Backup and display.
	 */
	global $g_a_anrghg_config;
	$l_s_json = json_encode( $g_a_anrghg_config, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT );
	$l_s_uploads_path = wp_upload_dir();
	if ( ! is_dir( $l_s_uploads_path[ 'basedir' ] . '/anrghg' ) ) {
		mkdir( $l_s_uploads_path[ 'basedir' ] . '/anrghg' );
	}
	$l_o_json = fopen( $l_s_uploads_path[ 'basedir' ] . '/anrghg/anrghg-settings_latest.json', 'w' );
	fwrite( $l_o_json, $l_s_json );
	$l_s_scope = __( 'Last revision:', 'anrghg' );
	$l_s_stamp = '';
	if ( $g_a_anrghg_config[ 'anrghg_keep_settings_history' ] ) {
		$l_s_datetime = current_time( 'Y-m-d\TH:i:sO' );
		$l_o_json = fopen( $l_s_uploads_path[ 'basedir' ] . "/anrghg/anrghg-settings_$l_s_datetime.json", 'w' );
		fwrite( $l_o_json, $l_s_json );
		$l_s_scope = __( 'Full revision history:', 'anrghg' );
		$l_s_stamp = '_[datetime]';
	}
	echo '<div class="preview">' . __( 'Your data is stored in the DB and backed up in JSON as in this frame. ', 'anrghg' );
	echo $l_s_scope . ' ';
	echo trailingslashit( wp_basename( content_url() ) ) . wp_basename( $l_s_uploads_path[ 'basedir' ] );
	echo "/anrghg/anrghg-settings$l_s_stamp.json.";
	echo '<div class="json"><pre>' . esc_attr( $l_s_json ) . "</pre></div></div>\r\n";

	return;
}

/**
 * Meta box field callback function.
 *
 * @since 0.30.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_meta_box_active_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Add a meta box in the editor side pane', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'The meta box displays in the editor.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'No meta box added.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . sprintf( __( 'The settings checked below are displayed in the ‘%s’ meta box:', 'anrghg' ), _x( 'Complements', 'Meta box title', 'anrghg' ) ) . "</p>\r\n";
	echo '<div class="aux"><label><input type="hidden" id="' . $p_a_params[ 'key_1' ] . '_0" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_1' ] . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="1"' . checked( $p_a_params[ 'val_1' ], '1', false ) . ' />' . __( 'Table of contents', 'anrghg' ) . '</label></div>';
	echo '<div class="aux"><label><input type="hidden" id="' . $p_a_params[ 'key_2' ] . '_0" name="anrghg[' . $p_a_params[ 'key_2' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_2' ] . '" name="anrghg[' . $p_a_params[ 'key_2' ] . ']" value="1"' . checked( $p_a_params[ 'val_2' ], '1', false ) . ' />' . __( 'Complement lists and referrer tooltips: Writing direction', 'anrghg' ) . '</label></div>';
	echo '<div class="aux"><label><input type="hidden" id="' . $p_a_params[ 'key_3' ] . '_0" name="anrghg[' . $p_a_params[ 'key_3' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_3' ] . '" name="anrghg[' . $p_a_params[ 'key_3' ] . ']" value="1"' . checked( $p_a_params[ 'val_3' ], '1', false ) . ' />' . __( 'Complement list labels', 'anrghg' ) . '</label></div>';
	echo '<div class="aux"><label><input type="hidden" id="' . $p_a_params[ 'key_4' ] . '_0" name="anrghg[' . $p_a_params[ 'key_4' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_4' ] . '" name="anrghg[' . $p_a_params[ 'key_4' ] . ']" value="1"' . checked( $p_a_params[ 'val_4' ], '1', false ) . ' />' . __( 'Complement list footer deferral', 'anrghg' ) . '</label></div>';
	return;
}

/**
 * Menu_level field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_menu_level_cb( $p_a_params ) {
	$l_s_value_0 = 'sub';
	$l_s_value_1 = 'top';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Settings and Tools submenus', 'anrghg' ) . '<br />';
	echo '<div class="aux"><p class="description" style="font-weight: normal !important;">' . __( 'The submenu labels are ‘Reusables’ under ‘Tools’, and ‘Complements’ under ‘Settings’.', 'anrghg' ) . '</p></div></label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Admin main menu', 'anrghg' ) . '</label></div>';
	echo '<div class="aux"><label><input type="hidden" id="' . $p_a_params[ 'key_1' ] . '_0" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="0" /><input tabindex="-1" type="checkbox" id="' . $p_a_params[ 'key_1' ] . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="1"' . checked( $p_a_params[ 'val_1' ], '1', false ) . ' />' . __( 'Remove duplicate menu entry from Settings submenu', 'anrghg' ) . '</label>';
	echo '<p class="description">' . __( 'Please check the box only after accessing this page from the new main menu entry submenu.', 'anrghg' ) . "</p></div>\r\n";
	echo '</fieldset>';
	return;
}

/**
 * Menu_position field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_menu_position_cb( $p_a_params ) {
	$l_s_value_0 = 'top';
	$l_s_value_1 = 'mid';
	$l_s_value_2 = 'low';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Beneath ‘Comments’ (WordPress default position for plugins)', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Beneath ‘Settings’', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Lowest Zone (this plugin’s default position)', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'This setting is effective only in the Main menu, where it matters most.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Security section callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_security_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'Security is increased by deactivating the Admin login dialog.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description" tabindex="0">' . __( 'The first step is to define a constant in wp-config.php as <code>false</code>. The login screen is available while it is set to <code>true</code>:', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . '<code>// Turn login off by replacing true with false:</code><br /><code>define( \'ANRGHG_WP_LOGIN_ACTIVE\', true );</code>' . "</p>\r\n";
	return;
}

/**
 * Login page profile field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_login_deactivation_profile_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '0';
	$l_s_value_2 = '-1';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'High profile', 'anrghg' ) . '</label></div>';
	echo '<p class="description">' . __( 'The login dialog is replaced with a red-banded message.', 'anrghg' ) . "</p>\r\n";
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Intermediate', 'anrghg' ) . '</label></div>';
	echo '<p class="description">' . __( 'The login dialog disappears but without elaborating.', 'anrghg' ) . "</p>\r\n";
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Low profile', 'anrghg' ) . '</label></div>';
	echo '<p class="description">' . __( 'Only the auth cookie generation/sending is blocked.', 'anrghg' ) . "</p>\r\n";
	echo '</fieldset>';
	return;
}

/**
 * Edit auth cookie lifespan field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_auth_duration_edit_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Activate authentication cookie expiration control', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'Cookies are valid as long as defined below.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'Default validities apply.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . __( 'Normally, authentication cookies expire after 14 days if “☐ Remember me” is checked, after 2 days if the checkbox is unchecked.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Lifespan duration in days field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_auth_expiration_days_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="number" class="font" id="' . $p_a_params[ 'key' ] . '" step="1" min="1" max="373" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $p_a_params[ 'val' ] . '" /> <label for="' . $p_a_params[ 'key' ] . '">' . __( 'days', 'anrghg' ) . '</label><br />';
	echo '<p class="description">' . __( 'This facility helps coordinate collaborators by scheduling the next login rush.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'In this input field, the maximum duration is set to one year plus a week.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Thank-you message section callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_thank_you_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	// Translators: The placeholder is the hyperlinked article title 'Creating A WordPress Plugin Is Easier Than You Think' at https://www.wpbeaverbuilder.com/creating-wordpress-plugin-easier-think/
	echo '<p class="anrghg">' . sprintf( __( 'Feature suggested by Justin Busa (FastLine Media) in %s', 'anrghg' ), '<a tabindex="-1" href="https://www.wpbeaverbuilder.com/creating-wordpress-plugin-easier-think/" target="_blank" style="font-style: italic;">Creating A WordPress Plugin Is Easier Than You Think</a>' ) . "</p>\r\n";
	return;
}

/**
 * Activate and set scope of Thank-you message field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_thank_you_active_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '3';
	$l_s_value_2 = '2';
	$l_s_value_3 = '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Append a message to posts', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Append one message to posts and another one to pages', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Append a message to pages only', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="no" id="' . $p_a_params[ 'key' ] . '3" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_3 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_3, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '3">' . __( 'Deactivate this feature', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'The feature is deactivated by default.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Priority level of thank you message field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_thank_you_priority_select_cb( $p_a_params ) {
	$l_s_value_0 = '-1';
	$l_s_value_1 = '0';
	$l_s_value_2 = '1';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si"     id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Highest priority (least negative number in PHP)', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si box" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">';
	$l_s_range = 4 === PHP_INT_SIZE ? '−2×10⁹…2×10⁹' : '−9×10¹⁸…9×10¹⁸';
	// Translators: The placeholder is either −2×10⁹…2×10⁹ or −9×10¹⁸…9×10¹⁸ approximately.
	echo '<input type="number" id="' . $p_a_params[ 'key' ] . '1in" step="1" min="' . ( 4 === PHP_INT_SIZE ? -2147483648 : -9223372036854775808 ) . '" max="' . PHP_INT_MAX . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="' . $p_a_params[ 'val_1' ] . '" /> ' . sprintf( __( 'Range %s', 'anrghg' ), $l_s_range ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si"     id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Lowest priority (greatest positive number in PHP)', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'The priority level determines the relative position with respect to other features. The default priority level is 10, negative levels are allowed.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Content of post Thank-you message field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_thank_you_content_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<textarea class="medium" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']">' . esc_attr( $p_a_params[ 'val' ] ) . '</textarea>';
	echo '<p class="description">' . __( 'The input field supports HTML formatting.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_post_thanks p</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Content of page Thank-you message field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_thank_you_content_page_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<textarea class="medium" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']">' . esc_attr( $p_a_params[ 'val' ] ) . '</textarea>';
	echo '<p class="description">' . __( 'The input field supports HTML formatting.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_page_thanks p</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Display last modified date section callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( '', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Activate display date metadata field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_active_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="no" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Deactivate', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Activate as specified below:', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<div id="dates_scope">';
	echo '<div class="flex"><label><input type="hidden" id="' . $p_a_params[ 'key_1' ] . '_0" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_1' ] . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="1"' . checked( $p_a_params[ 'val_1' ], '1', false ) . ' />' . __( 'Last-modified date at the top of posts', 'anrghg' ) . '</label>';
	echo '<label><input type="hidden" id="' . $p_a_params[ 'key_2' ] . '_0" name="anrghg[' . $p_a_params[ 'key_2' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_2' ] . '" name="anrghg[' . $p_a_params[ 'key_2' ] . ']" value="1"' . checked( $p_a_params[ 'val_2' ], '1', false ) . ' />' . __( '…of pages', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><label><input type="hidden" id="' . $p_a_params[ 'key_3' ] . '_0" name="anrghg[' . $p_a_params[ 'key_3' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_3' ] . '" name="anrghg[' . $p_a_params[ 'key_3' ] . ']" value="1"' . checked( $p_a_params[ 'val_3' ], '1', false ) . ' />' . __( 'Last-modified date at the end of posts', 'anrghg' ) . '</label>';
	echo '<label><input type="hidden" id="' . $p_a_params[ 'key_4' ] . '_0" name="anrghg[' . $p_a_params[ 'key_4' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_4' ] . '" name="anrghg[' . $p_a_params[ 'key_4' ] . ']" value="1"' . checked( $p_a_params[ 'val_4' ], '1', false ) . ' />' . __( '…of pages', 'anrghg' ) . '</label></div>';
	echo '<p class="description">' . __( 'For posts, the last modified date information may add below the already present published date.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'Add the published date as well:', 'anrghg' ) . "</p>\r\n";
	echo '<div class="flex"><label><input type="hidden" id="' . $p_a_params[ 'key_5' ] . '_0" name="anrghg[' . $p_a_params[ 'key_5' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_5' ] . '" name="anrghg[' . $p_a_params[ 'key_5' ] . ']" value="1"' . checked( $p_a_params[ 'val_5' ], '1', false ) . ' />' . __( 'Published date at the top of posts', 'anrghg' ) . '</label>';
	echo '<label><input type="hidden" id="' . $p_a_params[ 'key_6' ] . '_0" name="anrghg[' . $p_a_params[ 'key_6' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_6' ] . '" name="anrghg[' . $p_a_params[ 'key_6' ] . ']" value="1"' . checked( $p_a_params[ 'val_6' ], '1', false ) . ' />' . __( '…of pages', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><label><input type="hidden" id="' . $p_a_params[ 'key_7' ] . '_0" name="anrghg[' . $p_a_params[ 'key_7' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_7' ] . '" name="anrghg[' . $p_a_params[ 'key_7' ] . ']" value="1"' . checked( $p_a_params[ 'val_7' ], '1', false ) . ' />' . __( 'Published date at the end of posts', 'anrghg' ) . '</label>';
	echo '<label><input type="hidden" id="' . $p_a_params[ 'key_8' ] . '_0" name="anrghg[' . $p_a_params[ 'key_8' ] . ']" value="0" /><input type="checkbox" id="' . $p_a_params[ 'key_8' ] . '" name="anrghg[' . $p_a_params[ 'key_8' ] . ']" value="1"' . checked( $p_a_params[ 'val_8' ], '1', false ) . ' />' . __( '…of pages', 'anrghg' ) . '</label></div>';
	echo "</div>\r\n";
	return;
}

/**
 * Date metadata ordering field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_chronological_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Published date first, last-modified date second', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Last-modified date first, published date second', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'To apply style rules to the first or the second line only, the <code>span</code> in the selectors as shown below becomes <code>span:first-child</code> or <code>span:last-child</code> respectively.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label configuration field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_uni_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Unified labels', 'anrghg' ) . '</label></div>';
	echo '<table class="text" id="unified_labels"><tbody><tr><td><label for="' . $p_a_params[ 'key_1' ] . '">' . __( 'Modified:', 'anrghg' ) . '</label></td><td>';
	echo '<input type="text" id="' . $p_a_params[ 'key_1' ] . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="' . esc_attr( $p_a_params[ 'val_1' ] ) . '" />';
	echo '</td></tr><tr><td><label for="' . $p_a_params[ 'key_2' ] . '">' . __( 'Published:', 'anrghg' ) . '</label></td><td>';
	echo '<input type="text" id="' . $p_a_params[ 'key_2' ] . '" name="anrghg[' . $p_a_params[ 'key_2' ] . ']" value="' . esc_attr( $p_a_params[ 'val_2' ] ) . '" />';
	echo '</td></tr><tr><td colspan="2">';
	echo '<p class="description">' . __( 'To apply style rules, the CSS selectors are respectively:', 'anrghg' ) . '<br />';
	echo '<code>div.anrghg_dates_post_top span a</code><br />';
	echo '<code>div.anrghg_dates_page_top span a</code><br />';
	echo '<code>div.anrghg_dates_post_end span a</code><br />';
	echo '<code>div.anrghg_dates_page_end span a</code>' . "</p>\r\n";
	echo '</td></tr></tbody></table>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Differentiated labels', 'anrghg' ) . '</label></div>' . "\r\n";
	echo '</fieldset>';
	echo '<noscript><p class="description">' . __( 'As JavaScript is currently deactivated in your browser, please make sure by saving that the label configuration field list displayed below is up-to-date.', 'anrghg' ) . "</p></noscript\r\n";
	return;
}

/**
 * Date metadata label post top modified field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_post_top_modif_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_post_top span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label page top modified field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_page_top_modif_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_page_top span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label post top published field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_post_top_publi_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_post_top span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label page top published field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_page_top_publi_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_page_top span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label post end modified field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_post_end_modif_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_post_end span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label page end modified field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_page_end_modif_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_page_end span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label post end publishd field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_post_end_publi_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_post_end span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Date metadata label page end published field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_label_page_end_publi_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /><label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label>' . "\r\n";
	echo '<p class="description">' . __( 'To apply style rules, the CSS selector is <code>div.anrghg_dates_page_end span a</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Priority level of date metadata field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_dates_priority_select_cb( $p_a_params ) {
	$l_s_value_0 = '-1';
	$l_s_value_1 = '0';
	$l_s_value_2 = '1';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si"     id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Highest priority (least negative number in PHP)', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si box" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">';
	$l_s_range = 4 === PHP_INT_SIZE ? '−2×10⁹…2×10⁹' : '−9×10¹⁸…9×10¹⁸';
	// Translators: The placeholder is either −2×10⁹…2×10⁹ or −9×10¹⁸…9×10¹⁸ approximately.
	echo '<input type="number" id="' . $p_a_params[ 'key' ] . '1in" step="1" min="' . ( 4 === PHP_INT_SIZE ? -2147483648 : -9223372036854775808 ) . '" max="' . PHP_INT_MAX . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="' . $p_a_params[ 'val_1' ] . '" /> ' . sprintf( __( 'Range %s', 'anrghg' ), $l_s_range ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si"     id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Lowest priority (greatest positive number in PHP)', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'The priority level determines the relative position with respect to other features. The default priority level is 10, negative levels are allowed.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Add date meta tags in the page head section callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_date_meta_tags_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'These tags may be parsed by bibliography software and are useful in case structured data is not added already for SEO.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Activate date meta tags field callback function.
 *
 * @since 0.5.0
 *
 * @since 0.24.8
 * Checkbox-based vertical glide switch.
 * @param array $p_a_params
 * @return void
 */
function anrghg_date_meta_tags_active_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Add date meta tags', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'The date meta tags are added.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'No extra meta tags are added.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . __( 'These meta tags encompass both published and last modified dates.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . sprintf( __( 'For SEO, we highly recommend using a plugin like %s instead of this feature (deactivated by default).', 'anrghg' ), '<a href="https://wordpress.org/plugins/wordpress-seo/" target="_blank">Yoast SEO</a>' ) . "</p>\r\n";
	return;
}

/**
 * Activate Open Graph metadata for dates field callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_date_meta_tags_format_cb( $p_a_params ) {
	$l_s_value_0 = 'common';
	$l_s_value_1 = '1';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Use common format:', 'anrghg' ) . '</div>';
	echo '<code class="label">&lt;meta name="date" content="' . current_time( 'Y-m-d\TH:i:sO' ) . '" /&gt;</code><br />';
	echo '<code class="label">&lt;meta name="last-modified" content="' . current_time( 'Y-m-d\TH:i:sO' ) . '" /&gt;</code></label><br />';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Use Open Graph format:', 'anrghg' ) . '</div>';
	echo '<code class="label">&lt;meta property="article:published_time" content="' . current_time( 'Y-m-d\TH:i:sO' ) . '" /&gt;</code><br />';
	echo '<code class="label">&lt;meta property="article:modified_time" content="' . current_time( 'Y-m-d\TH:i:sO' ) . '" /&gt;</code></label><br />' . "\r\n";
	// Translators: The placeholder inserts a link to the WordPress plugin directory page of the Yoast SEO plugin, with 'Yoast SEO' as link text.
	echo '</fieldset>';
	return;
}

/**
 * Links to paragraphs and list items section callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_paragraph_links_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'Based on the content, the identifier is in the link on a character that is prepended visually only, not in the text stream.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Activation field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_paragraph_links_active_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Activate paragraph links', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'Links to paragraphs are added.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'No links are added to paragraphs.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . __( 'Links are added to list items as well if this feature is active.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Character field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_paragraph_link_character_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" class="narrow" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /> <label for="' . $p_a_params[ 'key' ] . '">' . __( 'Suggestions:', 'anrghg' ) . '</label> ¶, §, 🔗 (U+1F517)<br />';
	echo '<p class="description">' . __( 'The input may be a Unicode character, HTML entity or numeric character reference.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Table of contents and heading links section callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( '', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Heading links field callback function.
 *
 * @since 0.27.4
 * @param array $p_a_params
 * @return void
 */
function anrghg_heading_links_active_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Make headings link text of their fragment identifier', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'Headings are hyperlinked to their ID', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'No ID is accessible on the headings', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . __( 'The hyperlink extends over the whole heading in its full length.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Table of contents field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_active_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '-1';
	$l_s_value_2 = '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Insert the table of contents by default if applicable', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Insert a table of contents if specified accordingly', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="no" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Deactivate this feature', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'The heading numbers are added by CSS counters and are not part of the text stream.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Minimum number of headings field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_min_number_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="number" class="tiny" id="' . $p_a_params[ 'key' ] . '" step="1" min="1" max="10" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $p_a_params[ 'val' ] . '" /> <label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label><br />';
	echo '<p class="description">' . __( 'Below this threshold, numbering the headings and listing them would be considered pointless.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Label field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_label_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /> <label for="' . $p_a_params[ 'key' ] . '">' . __( '', 'anrghg' ) . '</label><br />';
	echo '<p class="description">' . __( 'The input field supports HTML formatting.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Table of contents depth field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_depth_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="number" class="tiny" id="' . $p_a_params[ 'key' ] . '" step="1" min="2" max="6" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $p_a_params[ 'val' ] . '" /> <label for="' . $p_a_params[ 'key' ] . '">' . __( '(level number, not number of levels)', 'anrghg' ) . '</label><br />';
	echo '<p class="description">' . __( 'Best is to list all levels in the table so as to avoid dead links around headings.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'It may be worth noting that using H1 in the article body, or not using H1 for the post title, are both bad for SEO.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'However, if H1 are present inside an article, the table of contents will support them.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Default position of the table field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_position_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '0';
	$l_s_value_2 = '-1';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'At the top of the post', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Before the first heading', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'At the end of the post', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'The default position is ignored when a positioner is present in the article.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Code to manually position the table field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_positioner_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="text" class="small" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . esc_attr( $p_a_params[ 'val' ] ) . '" /> <label for="' . $p_a_params[ 'key' ] . '">' . __( 'May be any string in your script and language.', 'anrghg' ) . '</label><br />';
	echo '<p class="description">' . __( 'The Block Editor has a block to position the table (automatically inserting a position code) if the feature is active.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Collapsible and collapsed states field callback function.
 *
 * @since 0.27.2
 * @param array $p_a_params
 * @return void
 */
function anrghg_table_of_contents_collapse_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '-1';
	$l_s_value_2 = '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Collapsed when the page loads, expands on click', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Displays expanded on load, collapsible on click', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Always displays in full length, cannot collapse', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( '', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Complements as end notes and references section callback function.
 *
 * @since 0.24.7
 * @param array $p_a_params
 * @return void
 */
function anrghg_complements_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'Many settings in this section are still missing; our apologies.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Activate field callback function.
 *
 * @since 0.27.3
 * @param array $p_a_params
 * @return void
 */
function anrghg_complements_active_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Process complements as notes and references', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'Both notes and references are processed.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'Inline complements are not processed.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . __( '', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Note delimiters field callback function.
 *
 * @since 0.21.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_note_delimiter_set_cb( $p_a_params ) {
	$l_s_value_0 = '-1';
	$l_s_value_1 =  '1';
	$l_s_value_2 =  '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Easy input: <code>{{{</code> … <code>}}}</code>', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Unambiguous: <code>[note]</code> … <code>[/note]</code>', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Freely configured:', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<input type="text" class="delims" id="' . $p_a_params[ 'key_1' ] . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="' . esc_attr( $p_a_params[ 'val_1' ] ) . '" /><label for="' . $p_a_params[ 'key_1' ] . '">' . __( ' … ', 'anrghg' ) . '</label>';
	echo '<input type="text" class="delims" id="' . $p_a_params[ 'key_2' ] . '" name="anrghg[' . $p_a_params[ 'key_2' ] . ']" value="' . esc_attr( $p_a_params[ 'val_2' ] ) . '" /><label for="' . $p_a_params[ 'key_2' ] . '">' . __( '', 'anrghg' ) . '</label>';
	echo '<p class="description">' . __( 'Notes are processed last, listed first, and cannot be nested in references (nor in themselves).', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Reference delimiters field callback function.
 *
 * @since 0.21.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_ref_delimiter_set_cb( $p_a_params ) {
	$l_s_value_0 = '-1';
	$l_s_value_1 =  '1';
	$l_s_value_2 =  '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Easy input: <code>[[[</code> … <code>]]]</code>', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Unambiguous: <code>[ref]</code> … <code>[/ref]</code>', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Freely configured:', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<input type="text" class="delims" id="' . $p_a_params[ 'key_1' ] . '" name="anrghg[' . $p_a_params[ 'key_1' ] . ']" value="' . esc_attr( $p_a_params[ 'val_1' ] ) . '" /><label for="' . $p_a_params[ 'key_1' ] . '">' . __( ' … ', 'anrghg' ) . '</label>';
	echo '<input type="text" class="delims" id="' . $p_a_params[ 'key_2' ] . '" name="anrghg[' . $p_a_params[ 'key_2' ] . ']" value="' . esc_attr( $p_a_params[ 'val_2' ] ) . '" /><label for="' . $p_a_params[ 'key_2' ] . '">' . __( '', 'anrghg' ) . '</label>';
	echo '<p class="description">' . __( 'References are processed first, listed last, and may be nested in notes (but not in themselves).', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Note list layout options field callback function.
 *
 * @since 0.28.0
 * Initially unified with reference lists.
 * @since 0.30.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_note_list_layout_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '2';
	$l_s_value_2 = '3';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Display the note lists in one column', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Display the note lists in two columns', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Display the note lists in three columns', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'On mobiles, the lists display in a single column.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Reference list layout options field callback function.
 *
 * @since 0.30.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_reference_list_layout_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '2';
	$l_s_value_2 = '3';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Display the reference lists in one column', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Display the reference lists in two columns', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '2" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_2 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_2, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '2">' . __( 'Display the reference lists in three columns', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'On mobiles, the lists display in a single column.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Defer complement lists to the footer field callback function.
 *
 * @since 0.26.0
 * Setting to activate footer output globally or per post, or deactivate.
 * @since 0.30.0
 * Only two states, as post configuration must not be overridden.
 *
 * @param array $p_a_params
 * @return void
 */
function anrghg_complement_list_footer_deferral_cb( $p_a_params ) {
	$l_s_value_0 = '1';
	$l_s_value_1 = '0';
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<fieldset><legend class="screenreadertext">' . $p_a_params[ 'label' ] . '</legend>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '"  name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_0 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_0, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '" >' . __( 'Defer complement lists to the footer', 'anrghg' ) . '</label></div>';
	echo '<div class="flex"><input type="radio" class="si" id="' . $p_a_params[ 'key' ] . '1" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="' . $l_s_value_1 . '"' . checked( $p_a_params[ 'val' ], $l_s_value_1, false ) . ' /><label for="' . $p_a_params[ 'key' ] . '1">' . __( 'Keep complement lists in the article', 'anrghg' ) . '</label></div>';
	echo '</fieldset>';
	echo '<p class="description">' . __( 'This configuration can be overridden on a per-post basis.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'By default, there is no way to output anything in the footer, only below the footer. The classic way to output content in the footer is by moving <code>wp_footer()</code> to the appropriate position while editing a child theme.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'Alternatively, output buffering—activatable below—offers an opportunity to move the lists from below the footer to a best-guess position in the footer, right after the last opening <code>&lt;footer&gt;</code> tag or, if absent, after the element with <code>id="footer"</code>.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Automatic positioning in the footer field callback function.
 *
 * @since 0.26.0
 * Setting to activate output buffering with notice that PHP 7.3 requires zlib extension.
 * @param array $p_a_params
 * @return void
 */
function anrghg_complement_list_output_buffering_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Activate output buffering to automatically position complement lists in the footer', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'Output buffering for editing the footer is activated.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'No output buffer will be started for editing the footer.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . __( 'If using PHP 7.3, please be aware that the distribution may be lacking the zlib extension, triggering a notice. Best is to upgrade to a more recent version of PHP.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'This output buffering will be used only if complement lists may be output below the footer.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Display link URL as visible text in note lists field callback function.
 *
 * @since 0.30.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_display_urls_note_list_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Replicate URLs as parenthesized text in note lists', 'anrghg' ) . '</div>';
	// Translators: The placeholders are the anchor element start and end tags plus the URL in parentheses.
	echo '<div class="state1" aria-hidden="true">' . sprintf( __( '“%1$sExample%2$s” becomes “%1$sExample%2$s%3$s”.', 'anrghg' ), '<a href="https://example.net" target="_blank">', '</a>', ' (https://example.net)' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	// Translators: The placeholders are the anchor element start and end tags.
	echo '<div class="state0" aria-hidden="true">' . sprintf( __( '“%1$sExample%2$s” stays “%1$sExample%2$s”.', 'anrghg' ), '<a href="https://example.net" target="_blank">', '</a>' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	return;
}

/**
 * Display link URL as visible text in reference lists field callback function.
 *
 * @since 0.30.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_display_urls_reference_list_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Replicate URLs as parenthesized text in reference lists', 'anrghg' ) . '</div>';
	// Translators: The placeholders are the anchor element start and end tags plus the URL in parentheses.
	echo '<div class="state1" aria-hidden="true">' . sprintf( __( '“%1$sExample%2$s” becomes “%1$sExample%2$s%3$s”.', 'anrghg' ), '<a href="https://example.net" target="_blank">', '</a>', ' (https://example.net)' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	// Translators: The placeholders are the anchor element start and end tags.
	echo '<div class="state0" aria-hidden="true">' . sprintf( __( '“%1$sExample%2$s” stays “%1$sExample%2$s”.', 'anrghg' ), '<a href="https://example.net" target="_blank">', '</a>' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	return;
}

/**
 * Option to deactivate WPTexturize section callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_wptexturize_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'For straight quotes to stay straight also outside code, while curly quotes are input as such.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Activate WPTexturize field callback function.
 *
 * @since 0.5.0
 *
 * @since 0.24.8
 * Checkbox-based vertical glide switch.
 * @param array $p_a_params
 * @return void
 */
function anrghg_wptexturize_active_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Keep WPTexturize activated', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'WPTexturize is active.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'WPTexturize is deactivated.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	// Translators: The placeholder is the article 'How to Properly Disable wptexturize in WordPress' hyperlinked to https://www.thewebflash.com/how-to-properly-disable-wptexturize-in-wordpress/
	echo '<p class="description">' . sprintf( __( 'This option has been copied from The Web Flash in %s.', 'anrghg' ), '<a tabindex="-1" href="https://www.thewebflash.com/how-to-properly-disable-wptexturize-in-wordpress/" target="_blank" style="font-style:normal;">How to Properly Disable wptexturize in WordPress</a>' ) . "</p>\r\n";
	return;
}

/**
 * Use the URL slug as a CSS selector section callback function.
 *
 * @since 0.5.0
 * @param array $p_a_params
 * @return void
 */
function anrghg_slug_body_class_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'Apply page-specific style rules using the page slug as a selector in Custom CSS.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * URL slug as a CSS selector field callback function.
 *
 * @since 0.5.0
 *
 * @since 0.24.8
 * Checkbox-based vertical glide switch.
 * @param array $p_a_params
 * @return void
 */
function anrghg_slug_body_class_active_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<input type="hidden" id="' . $p_a_params[ 'key' ] . '_0" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="0" />';
	echo '<input type="checkbox" class="switch" id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']" value="1"' . checked( $p_a_params[ 'val' ], '1', false ) . ' />';
	echo '<label for="' . $p_a_params[ 'key' ] . '"><div class="flex"><div class="switch"><div class="slot"></div><div class="knob"></div></div><div class="labels">';
	echo '<div class="label1">' . __( 'Add the post’s slug as a class to the <code>&lt;body&gt;</code> element.', 'anrghg' ) . '</div>';
	echo '<div class="state1" aria-hidden="true">' . __( 'The slug is a class name.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div>';
	echo '<div class="state0" aria-hidden="true">' . __( 'The slug is no class name.', 'anrghg' ) . '<span class="apply' . substr( checked( $p_a_params[ 'val' ], '1', false ), 0, 8 ) . '"></span></div></div></div></label>';
	echo '<p class="description">' . __( 'With this, you may use the page slug instead of the number <code>postid-1234</code> or <code>page-id-1234</code> already included in the <code>&lt;body&gt;</code> class names.', 'anrghg' ) . "</p>\r\n";
	echo '<p class="description">' . __( 'Important: Slugs starting with a digit will be prepended with an underscore <code>_</code> to make valid CSS class names out of them.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Additional cascading style sheet section callback function.
 *
 * @since 0.27.4
 * @param array $p_a_params
 * @return void
 */
function anrghg_css_cb( $p_a_params ) {
	$l_s_ui_id = substr( $p_a_params[ 'id' ], 7 );
	echo '<div class="base"><a aria-label="' . $p_a_params[ 'title' ] . '" href="#' . $l_s_ui_id . '" class="link">';
	echo '</a><span class="id" id="' . $l_s_ui_id . '"></span></div>';
	echo '<p class="anrghg">' . __( 'For stability across themes, some rules from the Theme Additional CSS editor may be moved here.', 'anrghg' ) . "</p>\r\n";
	return;
}

/**
 * Custom CSS textarea field callback function.
 *
 * @since 0.27.4
 * @param array $p_a_params
 * @return void
 */
function anrghg_additional_css_cb( $p_a_params ) {
	echo '<span class="anrghg base"><span class="id" id="anchor_' . $p_a_params[ 'key' ] . '"></span></span>';
	echo '<textarea id="' . $p_a_params[ 'key' ] . '" name="anrghg[' . $p_a_params[ 'key' ] . ']">' . esc_attr( $p_a_params[ 'val' ] ) . '</textarea>';
	echo '<p class="description">' . __( '', 'anrghg' ) . "</p>\r\n";
	return;
}
